(* ******************************************************************** *)
(*  									*)
(* 	Project		: Ida - Isabelle's documentation assistant	*)
(*  	Author 		: B.Wolff					*)
(*  	Affiliation 	: University of Bremen, 			*)
(*			  Bremen Institute of Safe Systems (BISS)	*)
(*   	This file	: Numbering management (for References)		*)
(*  									*)
(*  	$Date: 1997/01/22 19:48:35 $					*)
(*      $Revision: 1.4 $						*)
(*  									*)
(*  ******************************************************************* *)


structure Num_mgr :
  sig
    val init          : unit -> unit
    val init_num      : unit -> unit

    val new_chapter   : unit -> unit
    val new_proof     : unit -> unit
    val new_subproof  : unit -> unit
    val get_docstate  : unit -> int * int * int
    
    val push_nums     : unit -> unit
    val pop_nums      : unit -> int
    val top_nums      : unit -> int
    val top_tag       : unit -> string
    val set_top_tag   : string -> unit
    val incr_top_nums : unit -> unit
    val dot_number    : unit -> string
    val mt_nums       : unit -> bool
    val indent_level  : unit -> int
  end =
struct

   val STATE = ref(0,0,0);

   fun new_chapter () = let val (c,ps,sp) = !STATE
                        in  STATE:=(c+1,0,0) end;

   fun new_proof ()   = let val (c,ps,sp) = !STATE
                        in  STATE:=(c,ps+1,0) end;
   
   fun new_subproof ()= let val (c,ps,sp) = !STATE
                        in  STATE:=(c,ps,sp+1) end;

   fun get_docstate ()= (!STATE);


   val NUM_STACK = ref ([(1,"")]);

   fun init () = (NUM_STACK:=[(1,"")]; STATE:=(0,0,0) );
   fun init_num () = (NUM_STACK:=[(1,"")]);


   fun incr_top_nums () = 
		let val (a,t) :: S = !NUM_STACK in NUM_STACK:=(a+1,t)::S end;

   fun push_num_tag tag = ( NUM_STACK:=(1,tag)::(!NUM_STACK));
   fun push_nums () = push_num_tag "";

   fun pop_nums () =  let val (a,_) :: S = !NUM_STACK in NUM_STACK:=S;a end;
   fun top_nums () =  let val (a,_) :: S = !NUM_STACK in a end;
   fun top_tag ()  =  let val (_,t) :: S = !NUM_STACK in t end;
   fun set_top_tag t = 
		let val (a,_) :: S = !NUM_STACK in NUM_STACK:=(a,t)::S end;

   fun mt_nums () = (null(!NUM_STACK));

   fun dot_number () = 
    let fun h [(a,_)] = makestring a
           |h ((a,_)::b::S) = (makestring a)^"."^(h (b::S))
    in h(!NUM_STACK) end;

   fun indent_level () = length(!NUM_STACK);

 end;
