(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa/isa_toolkit/trafos/BM.thy,v $
 
   Top theory for the Bird-Meertens calculus. Augments Bremen's SList-theory
   with a few more functions.
  
   $Date: 1997/08/14 10:02:06 $
   $Revision: 1.6 $
   Author: tm (Last modification by $Author: cxl $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

BM = SList + Nat +  

consts

  last		:: "'a list => 'a"
  single	:: "'a => 'a list"
  foldr_ac	:: "[['a,'b] => 'b, 'b, 'a list] => 'b list"
  foldl_ac	:: "[['b,'a] => 'b, 'b, 'a list] => 'b list"
  inits		:: "'a list => 'a list list"
  tails		:: "'a list => 'a list list"
  h		:: "'a list list => 'a"
  allappl	:: "[('a => 'b) list, 'a] => 'b list"
  Id		:: "'a => 'a"

defs

  last_def	"last xs == list_rec xs (@x.True) 
					(%x xs r.if (null xs) then x else r)"

  single_def	"single x == [x]"

  Id_def	"Id x == x"

  foldr_ac_def	"foldr_ac f a xs == list_rec xs [a] (%x xs r.(f x (hd r))#r)"

  foldl_ac_def	"foldl_ac f a xs == list_rec xs (%a.[a]) 
						(%x xs r a.[a] @ r(f a x)) a"

  inits_def	"inits xs == foldl_ac (op @) [] (map single xs)"

  tails_def	"tails xs == foldr_ac (op @) [] (map single xs)"

  allappl_def	"allappl xs a == list_rec xs [] (%f xs r. f a # r)"

 
(*** Rules ***)

rules		

  map_prom	"map f (foldl (op @) [] xs) = foldl (op @) [] (map (map f) xs)"

  fold_prom	"foldl f e (foldl (op @) [] xs) = foldl f e (map (foldl f e) xs)"

  horner_rule	"! f a g e . (foldl f a) o (map (foldl g e) o tails)  = \
	        \foldl (% x y. f (g x y) e) e"

  filter_hom_def "filter p = ((foldl (op @) []) o (map (%x.if (p x) then (allappl [Id] x)  \
		\else (allappl [] x))))"

  fold_map2   "foldl f a o  (map g) = foldl (%x. (%y. f x(g y))) a"

  semi		"(foldl (max_f length) []) o (filter (allp p)) = \
		\(foldl (max_f length) []) o map ((foldl (op @) []) o (map (%x.if p x then (allappl [Id] x) else [])))"
end


