(*  Title:      GlobalSearch.ML
    $Id: GlobalSearch.ML,v 1.4 1997/04/10 20:53:40 cxl Exp $
    Authors:    Thomas Santen (GMD FIRST, Berlin), 
                Burkhart Wolff (Universitaet Bremen)
    Copyright   1995

Theory of Global Search Algorithms and Transformation of Specs to GS Algorithms
*)


 
(* *********************************************************************** *)
(*                                                                         *)
(* Global Manipulations of System state                                    *)
(*                                                                         *)
(* *********************************************************************** *)

fun rotate_tac n =
  let fun rot(n) = EVERY'(replicate n (dtac asm_rl));
  in if n >= 0 then rot n
     else SUBGOAL (fn (t,i) => rot(length(Logic.strip_assums_hyp t)+n) i)
  end;

(* *********************************************************************** *)
(*                                                                         *)
(* load Global Search Theory                                               *)
(*                                                                         *)
(* *********************************************************************** *)

open GlobalSearch;

(* *********************************************************************** *)
(*                                                                         *)
(* Proof of Conservativity of rep_x                                        *)
(*                                                                         *)
(* *********************************************************************** *)

goalw thy [rep_n_def] "rep_n f 0 x rh sh = (rh=sh)";
by(asm_simp_tac arith_ss 1);
qed "rep_n_0";

goalw thy [rep_n_def]   "rep_n f(Suc k) x rh th =               \
                        \ (EX sh. f x rh sh & rep_n f k x sh th)";
by(asm_simp_tac arith_ss 1);
qed "rep_n_Suc";


(* Noetherian Induction on Rh along < *)
val [GS5,prem] = goal thy 
"[|     GS5 (subspace::('Rh * 'Rh) set);                                \
\       (!!x::('Rh). ! y.(y,x) : subspace^+ --> (P y)==>(P x))|]     \
\ ==> P a";
by (resolve_tac [(rewrite_rule [GS5_def] GS5) RS wf_trancl RS wf_induct] 1);
by (resolve_tac [prem] 1);
by (assume_tac 1);
qed "Rh_induct";



(* GS1 as a rule *)
val GS1::prems = goal thy 
"[| GS1 I Ih Split subspace;                                    \
\   I x; Ih x rh; Split x rh sh |] ==> Ih x sh  & (sh,rh) : subspace^+";
by (resolve_tac [(rewrite_rule [GS1_def] GS1) RS spec RS spec RS spec RS mp] 1);
by (cut_facts_tac prems 1);
by (fast_tac HOL_cs 1);
qed "GS1_rule";


(* GS2 as a rule *)
val GS2::prems = goal thy 
"[| GS2 I Out Satisfies r0; I x; Out x z|] ==> Satisfies z(r0 x)";
by (cut_facts_tac prems 1);
by (cut_facts_tac [(rewrite_rule [GS2_def] GS2) RS spec RS spec]  1);
by (fast_tac HOL_cs 1);
qed "GS2_rule";


(* GS3 as a rule *)
val GS3::prems = goal thy 
"[|GS3 I Ih Split Satisfies Extract; I x; Ih x rh |] ==>        \
\  Satisfies z rh = (EX sh. rep_s Split x rh sh  & Extract z sh)";
by (resolve_tac [(rewrite_rule [GS3_def] GS3) RS spec RS spec RS spec RS mp] 1);
by (rtac conjI 1);
by (ALLGOALS (resolve_tac prems));
qed "GS3_rule";


(*--------------------------------------------------------------------*)
(* Lemmas for correctness of Fgs                                      *)
(*--------------------------------------------------------------------*)


(* Invariance of Satisfies over Split *)
val GS3::prems = goal thy
    "[| GS3 I Ih Split Satisfies Extract; I x; Ih x r; Ih x s |] ==> \
    \   Satisfies z s & Split x r s --> Satisfies z r";
(* by (cut_facts_tac GS3::prems 1);
   by (asm_simp_tac (HOL_ss addsimps [GS3_rule]) 1);
   This loops, because the simplifier matches the parameter
   "simplifies" of GS3 with (GS3 ?I7 ?Ih7 ?Split7) and tries to
   rewrite
   [| GS3 ?I9 ?Ih9 ?Split9 (GS3 ?I7 ?Ih7 ?Split7) ?Extract9; ?I9 ?x9;
      ?Ih9 ?x9 ?Extract7 |] ==>
   GS3 ?I7 ?Ih7 ?Split7 (GS3 ?I5 ?Ih5 ?Split5) ?Extract7 ==
   ? sh. rep_s ?Split9 ?x9 ?Extract7 sh & ?Extract9 (GS3 ?I5 ?Ih5
   ?Split5) sh

   We prevent this by resolving the first premise of GS3_rule explicitly 
*)
by (cut_facts_tac prems 1);
by (asm_simp_tac (HOL_ss addsimps [GS3 RS GS3_rule]) 1);

by (step_tac HOL_cs 1);
by (rtac exI 1);
by (rewrite_goals_tac [rep_s_def]);
by (eresolve_tac [exE] 1);
by (rtac conjI 1);
by (rtac exI 1);
by (assume_tac 2);
by (resolve_tac [rep_n_Suc RS ssubst] 1);
by (rtac exI 1);
by (rtac conjI 1);
by (assume_tac 1);
by (assume_tac 1);
qed "Satisfies_Split_inv";


(*--------------------------------------------------------------------*)
(* Correctness of Fgs                                                 *)
(*--------------------------------------------------------------------*)

val [rec_pred,GS1,GS3,GS5] = goalw thy []
" [| REC(Fgs::['a, 'b, 'c] => bool) I Ih Extract Out Split;       \
\    GS1 I Ih Split subspace; GS3 I Ih Split Satisfies Extract;        \
\    GS5 subspace |]                                                   \
\  ==>                                                                 \
\ (I x) & (Ih x r) & Fgs(x::'a)(r::'b)(z::'c) --> Satisfies z r & Out x z";

(* by (resolve_tac [Rh_induct] 1);
   by (resolve_tac [GS5] 1);
   This fails, because the type variable ?a of ?subspace : (?'a * ?'a)
   set is unified with 'b only in the 16th unifier! 
*)
by (resolve_tac [GS5 RS Rh_induct] 1);
by (cut_facts_tac [rec_pred] 1);
by (rewrite_goals_tac [REC_def]) ;
by (asm_simp_tac (HOL_ss addcongs [conj_cong]) 1);
by (rtac impI 1);
by (REPEAT (ALLGOALS (eresolve_tac [conjE,disjE,exE])));
by (rtac conjI 1);
by (assume_tac 2);
by (asm_simp_tac (HOL_ss addsimps [GS3 RS GS3_rule]) 1);
by (rtac exI 1);
by (rtac conjI 1);
by (assume_tac 2);
by (rewrite_goals_tac [rep_s_def]);
by (rtac exI 1);
by (resolve_tac [rep_n_0 RS ssubst] 1);
by (rtac refl 1);
(* now for the recursive case ... *)
by (etac conjE 1);
by (etac allE 1);
by (etac impE 1);
by (cut_facts_tac [GS1] 1);
by (resolve_tac [GS1_rule RS conjunct2] 1);
(* assume 1 first to instantiate mv's *)
by (assume_tac 1);
by (TRYALL assume_tac);
by (cut_facts_tac [GS1] 1);
by (forward_tac [GS1_rule RS conjunct1] 1);
by (TRYALL assume_tac);
by (step_tac HOL_cs 1);
by (asm_simp_tac (HOL_ss addsimps [GS3 RS Satisfies_Split_inv]) 1);
qed "Fgs_correctness";

(*--------------------------------------------------------------------*)
(* Lemmas for completeness of Fgs                                     *)
(*--------------------------------------------------------------------*)

val [rec_def,GS1,Ix,Ihxr,Oxz,rep_n,extract] = 
    goalw thy [REC_def,GS1_def]
"[| REC(Fgs::['a, 'b, 'c] => bool)I Ih Extract Out Split; \
\   GS1 I Ih Split subspace;                                     \
\   I x; Ih x rh; Out x z;                                     \
\   rep_n Split k x rh sh;                                     \
\   Extract z sh |]                                            \
\ ==> Fgs(x::'a)(rh::'b)(z::'c)";
by (cut_facts_tac [Ix,Ihxr,Oxz,rep_n,extract]  1);
by (REPEAT (eresolve_tac [rev_mp] 1));
by (res_inst_tac [("x", "sh")] spec 1);
by (res_inst_tac [("x", "rh")] spec 1);
by (nat_ind_tac "k" 1);
(* base case *)
by (simp_tac (nat_ss addsimps [rep_n_0,rec_def]) 1);

(* step case *)
(* NOTE: using asm_full_simp_tac here succeeds but prevents
   the classical prover from closing the goal!! *)
by (asm_simp_tac (nat_ss addsimps [rep_n_Suc,rec_def]) 1);
by (cut_facts_tac [GS1] 1);
by (fast_tac HOL_cs 1);
qed "Fgs_Split_n_complete";

(*--------------------------------------------------------------------*)
(* Completeness of Fgs                                                *)
(*--------------------------------------------------------------------*)


val [rec_pred, GS1, GS3, Ix, Ihxr ] = goal thy
"[| REC(Fgs::['a, 'b, 'c] => bool)I Ih Extract Out Split;        \
\   GS1 I Ih Split subspace;                                          \
\   GS3 I Ih Split Satisfies Extract;                                 \
\   I x; Ih x rh |] ==>                                               \
\   Satisfies z rh & Out x z --> Fgs(x::'a)(rh::'b)(z::'c)";
(* Why does the simplifier not rewrite Satisfies??????
  val HOL_ss_fsolve = HOL_ss 
                    setsolver (fn prems => resolve_tac(TrueI::refl::prems)
                               ORELSE' atac 
                               ORELSE' etac FalseE 
                               ORELSE' fast_tac HOL_cs);
trace_simp := true;
by (simp_tac (HOL_ss_fsolve addsimps [(rewrite_rule [GS3_def] GS3)]) 1);

nor succeeds: where GS3 obtained by goalw thy [GS3_def]

by (cut_facts_tac [GS3,Ix,Ihxr] 1);
by (asm_full_simp_tac (HOL_ss addcongs [conj_cong]) 1);
*)

by (resolve_tac [((Ihxr RS (Ix RS (GS3 RS GS3_rule)))) RS ssubst] 1);
by (rewrite_goals_tac [rep_s_def]);
by (cut_facts_tac [rec_pred,GS1,Ix,Ihxr]  1);
by (best_tac (HOL_cs addIs [Fgs_Split_n_complete]) 1);
qed "Fgs_completeness";

(*--------------------------------------------------------------------*)
(* Closed Form of Fgs                                                 *)
(*--------------------------------------------------------------------*)

val [rec_pred, GS1, GS3, GS5, Ix, Ihxr] = goal thy 
"[| REC(Fgs::['a, 'b, 'c] => bool)I Ih Extract Out Split;         \
\   GS1 I Ih Split subspace;                                           \
\   GS3 I Ih Split Satisfies Extract;                                  \
\   GS5 subspace;                                                      \
\   I x; Ih x rh |] ==>                                                \
\   ((Satisfies z rh) & (Out x z)) = Fgs(x::'a)(rh::'b)(z::'c)";
by (cut_facts_tac 
      ( [Ix, Ihxr, ([rec_pred, GS1, GS3, GS5] MRS Fgs_correctness)] @
      ([[rec_pred], [GS1], [GS3], [Ix], [Ihxr]] MRL [Fgs_completeness])) 1);
by (fast_tac HOL_cs 1);
qed "Fgs_closed_form";

(*--------------------------------------------------------------------*)
(* Correctness of Global Search                                       *)
(*--------------------------------------------------------------------*)

val [rec_def,GS0,GS1,GS2,GS3,GS5,Ix] = goal thy
" [| REC(Fgs::['a, 'b, 'c] => bool) I Ih Extract Out Split;     \
\   GS0 I Ih r0 ;                                                    \
\   GS1 I Ih Split subspace;                                        \
\   GS2 I Out Satisfies r0;                                         \
\   GS3 I Ih Split Satisfies Extract;                               \
\   GS5 subspace;                                                   \
\   I x  |] ==>                                                     \
\   Out x z = Fgs(x::'a)(r0(x)::'b)(z::'c)";
by (resolve_tac (([[rec_def],[GS1],[GS3],[GS5],[Ix],[(Ix RS (((rewrite_rule
[GS0_def] GS0) RS spec) RS mp))]] MRL [Fgs_closed_form]) RL [subst]) 1);
by (cut_facts_tac [Ix,(rewrite_rule [GS2_def] GS2)]  1);
by (fast_tac HOL_cs 1);
qed "gs_correctness";

(*--------------------------------------------------------------------*)
(* Satisfiability of REC                                              *)
(*--------------------------------------------------------------------*)

val prems = goalw thy [GS1_def]
" [| GS1 I Ih Split subspace  |] ==> \
\ (%x r s.Split x r s) = (%x r s. (Split x r s) & (I x & Ih x r  --> (Ih x s)))";
by (cut_facts_tac prems 1);
by (REPEAT (resolve_tac [ext] 1));
by (fast_tac HOL_cs 1);
qed "SplitIh";


val [GS1,GS3] = goalw thy [GS3_def,REC_def]
" [|GS1 I Ih Split subspace;                                        \
\   GS3 I Ih Split Satisfies Extract |]                             \
\ ==>  REC (% x rh z. (Satisfies(z::'c)(rh::'b)) & (Out x z))    \
\           I Ih Extract Out Split ";

(* with this information, the simplifier should unfold the second Satisfies, too
-- but it does not!! 
by (resolve_tac [(GS1 RS SplitIh RS ssubst)] 1);
*)

by (asm_full_simp_tac (HOL_ss addcongs [conj_cong] addsimps [GS3,rep_s_def]) 1);
by (REPEAT (resolve_tac [allI,impI,iffI] 1));
by (REPEAT (eresolve_tac [exE,conjE] 1));
by (res_inst_tac [("n", "k")] natE 1);
by (REPEAT (eresolve_tac [rev_mp] 1));
by (asm_full_simp_tac (arith_ss addcongs [conj_cong] addsimps [rep_n_0]) 1);


by (REPEAT (eresolve_tac [rev_mp] 1));
by (asm_full_simp_tac (arith_ss addcongs [conj_cong] addsimps [rep_n_Suc]) 1);
by (REPEAT (resolve_tac [impI] 1));
by (REPEAT (eresolve_tac [exE,conjE] 1));
by (REPEAT (resolve_tac [disjI2,exI,conjI] 1));
by (assume_tac 1);
by (forward_tac [GS1 RS GS1_rule] 1);
by (TRYALL assume_tac);

(* WHY DOES THIS NOT WORK??? 
by (eresolve_tac [conjE] 1);
by (asm_full_simp_tac (HOL_ss addcongs [conj_cong] addsimps [GS3,rep_s_def]) 1);
*)

by (resolve_tac [(GS3 RS spec RS spec RS spec RS mp RS ssubst)] 1);
by (fast_tac HOL_cs 1);
by (rewrite_goals_tac [rep_s_def]);
by (fast_tac HOL_cs 1);

(* second implication *)
by (eresolve_tac [disjE] 1);
by (REPEAT (resolve_tac [exI,conjI] 1));
by (resolve_tac [rep_n_0 RS ssubst] 1);
by (resolve_tac [refl] 1);
by (fast_tac HOL_cs 1);
by (fast_tac HOL_cs 1);
by (REPEAT (eresolve_tac [exE,conjE] 1));
by (forward_tac [GS1 RS GS1_rule] 1);
by (TRYALL assume_tac);

by (rotate_tac 3 1);
by (eresolve_tac [rev_mp] 1);
by (resolve_tac [(GS3 RS spec RS spec RS spec RS mp RS ssubst)] 1);
by (fast_tac HOL_cs 1);
by (rewrite_goals_tac [rep_s_def]);
by (resolve_tac [impI] 1);
by (REPEAT (eresolve_tac [exE,conjE] 1));
by (asm_simp_tac HOL_ss 1);
by (REPEAT (resolve_tac [exI,conjI] 1));
by (assume_tac 2);
by (resolve_tac [rep_n_Suc RS ssubst] 1);
by (fast_tac HOL_cs 1);
qed "RECsatisfiability";


(* *********************************************************************** *)
(*                                                                         *)
(* Global Search synthesis theormem                                        *)
(*                                                                         *)
(* *********************************************************************** *)

val prems = goalw thy [GSA_def]
" ALL Ih r0 Split Extract subspace Satisfies.                               \
\                                                                           \
\    (GSTHEORY I Out Ih r0 Split Extract subspace Satisfies) -->            \
\                                                                           \
\    (((I x) --> (F x z) = (Out x z))                                       \
\     =                                                                     \
\     ((I x) --> (EX Fgs.                                                   \
\        ((REC Fgs I Ih Extract Out Split) & (F x z) = (Fgs x (r0 x) z))))) ";

by (REPEAT (resolve_tac [allI,impI] 1));
by (REPEAT (eresolve_tac [conjE] 1));
by (resolve_tac [iffI RS sym] 1); (* original synthesis theorem was other way around *)
by (resolve_tac [impI] 1);
by (eresolve_tac [impE] 1);
by (assume_tac 1);
by (REPEAT (eresolve_tac [exE,conjE] 1));
by (dresolve_tac [gs_correctness] 1);
by (TRYALL assume_tac);
by (fast_tac HOL_cs 1);
by (REPEAT (resolve_tac [impI,exI,conjI] 1));
by (resolve_tac [RECsatisfiability] 1);
by (TRYALL assume_tac);
by (rewrite_goals_tac [GS2_def]);
by (fast_tac HOL_cs 1);
qed "global_search_synthm";

