(*  Title:      GlobalSearch.thy
    $Id: GlobalSearch.thy,v 1.3 1997/04/10 20:53:41 cxl Exp $
    Authors:    Thomas Santen (GMD FIRST, Berlin),
                Burkhart Wolff (Universitaet Bremen)
    Copyright   1995

Theory of Global Search Algorithms.
*)


GlobalSearch  =  Nat + Prod +

consts

  rep_n     :: "[['D, 'Rh, 'Rh] => bool, nat, 'D, 'Rh, 'Rh] => bool"
  rep_s     :: "[['D, 'Rh, 'Rh] => bool, 'D, 'Rh, 'Rh] => bool"


  for_n     :: "[nat,['a,'b] => 'b] =>['a,'b] => 'b"

  space      :: "[['a,'a] => bool] =>('a*'a) set"

  REC       :: "[ ['D, 'Rh, 'R] => bool,                                \
                \ 'D => bool,                                           \
                \ ['D, 'Rh] => bool,                                    \
                \ ['R, 'Rh] => bool,                                    \ 
                \ ['D, 'R] => bool,                                     \ 
                \ ['D, 'Rh, 'Rh] => bool] => bool  "                 

  GS0       :: "['D => bool,['D,  'Rh] => bool,'D => 'Rh] => bool"
  GS1       :: "['D => bool,['D,  'Rh] => bool,                         \
                \       ['D, 'Rh, 'R] => bool,('Rh * 'Rh) set] => bool"
  GS2       :: "['D => bool,['D, 'R] => bool,                           \
                \       ['R, 'Rh] => bool,'D => 'Rh] => bool"
  GS3       :: "['D => bool,['D,  'Rh] => bool,                         \
                \       ['D, 'Rh, 'Rh] => bool,                         \
                \       ['R, 'Rh] => bool,['R, 'Rh] => bool] => bool"
  GS5       :: "('Rh*'Rh) set => bool"

  GSTHEORY:: "['D => bool,['D,'R] => bool,['D,'Rh]=>bool,'D=>'Rh,       \
                \       ['D, 'Rh, 'Rh] => bool,['R, 'Rh] => bool,       \
                \       ('Rh * 'Rh) set,['R, 'Rh] => bool] => bool"


 
defs
 rep_n_def  "rep_n f n x     == nat_rec (op =)                        \
             \                  (% pred rec.%th rh.(EX sh.(f x th sh &  \
             \                                            rec sh rh))) n"
 rep_s_def  "rep_s f x rh sh == (EX k. rep_n f k x rh sh)"

(* thms :
 rep_n_0    "rep_n f 0 x rh sh = (rh=sh)"
 rep_n_Suc  "rep_n f (Suc k) x rh th = (EX sh. f x rh sh & 
                                                rep_n f k x sh th)" *)

(*  space      :: "[['a,'a] => bool] =>('a*'a) set"*)

space_def   "space less  == {(X,Y) . less X Y}"


 REC_def "REC H I Ih Extract Out Split ==                               \
          \       (!x r z. (I x) & (Ih x r) -->                         \
          \                (H x r z) = ((Extract z r) & (Out x z) |     \
          \                           (EX sh.Split x r sh & H x sh z)))"

 GS0_def "GS0 I Ih r0 == ! x. (I x) --> Ih x (r0 x)"

 GS1_def "GS1 I Ih Split subspace ==                                    \
          \    ! x rh sh. (I x) & (Ih x rh) & (Split x rh sh)           \
          \               --> (Ih x sh) & (sh,rh) : subspace^+"

 GS2_def "GS2 I Out Satisfies r0 ==                                     \
          \             ! x z. (I x) & (Out x z) --> (Satisfies z (r0 x))"

 GS3_def "GS3 I Ih Split Satisfies Extract ==                           \
          \             ! x z rh. (I x) & (Ih x rh) -->                 \
          \                       (Satisfies z rh) =                    \
          \                             (EX sh. (rep_s Split x rh sh) & \
          \                                     (Extract z sh))"

(* Better would be:
 GS5_def "GS5 subspace == (wf {(X,Y) . less X Y})"
*)
 GS5_def "GS5 subspace == wf subspace"

 GSA_def "GSTHEORY I Out Ih r0 Split Extract subspace Satisfies ==      \
          \     GS0 I Ih r0                        &                    \       
          \     GS1 I Ih Split subspace            &                    \
          \     GS2 I Out Satisfies r0             &                    \
          \     GS3 I Ih Split Satisfies Extract   &                    \
          \     GS5 subspace"


(* Theorem:
  
  global_search_synthm "                                                \ 
  \                                                                     \
  \     ! Ih r0 Split Extract subspace Satisfies Phi.                   \
  \                                                                     \
  \       GSTHEORY I Out Ih r0 Split Extract subspace Satisfies      -->\
  \                                                                     \
  \     (I x  --> EX Fgs.((REC Fgs I Ih Extract Out Split &             \
  \                      F x z = Fgs x (r0 x) z))))                     \
  \    =                                                                \
  \     (I x --> (Out x z) = (F x z)))                                  "

*)
  

end


(* Reminder for the typing of the Components of the transformation:

  I         :: "'D => bool"
  Out       :: "['D, 'R] => bool"       (* identifier O is already used.  *)
  F         :: "['D, 'R] => bool"       (* Even sets of output values are *)
                                        (* described by relations! *)
  Fgs       :: "['D, 'Rh, 'R] => bool"     

  Ih        :: "['D,  'Rh] => bool"
  r0        :: "'D => 'Rh"
  subspace  :: "('Rh * 'Rh) set"   (* subspace relation on searchspaces *)

  Satisfies :: "['R, 'Rh] => bool"
  Split     :: "['D, 'Rh, 'Rh] => bool"
  Extract   :: "['R, 'Rh] => bool"

*)
