(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/isa_win/sel_tactics.sml,v $
 
   IsaWin-- the "selection" tactics. 
   
   $Date: 1997/04/12 17:04:05 $
   $Revision: 1.2 $
   Author: cxl (Last modification by $Author: cxl $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)
 

(* These are two tactics making use of the tagging mechanism. They apply a
 * rewrite inside a term. The first argument has to be a string
 * representing an encoded path into the n-th subgoal.
 * 
 * SelTactics.getgoal extracts a subgoal from the current proof state and
 * removes the surrounding Const(Trueprop,_)$_ embedding. You have to use 
 * this function before displaying a subgoal as well, otherwise the paths 
 * will be wrong.
 *)

structure SelTactics :
    sig
	val getgoal    : int-> term

	val FOLD_AT_PATH   : string-> thm-> int-> tactic 
	    
	val UNFOLD_AT_PATH : string-> thm-> int-> tactic 
    end
=
struct

    fun getgoal i =  
	let val (_ $ t)= Goals.getgoal i 
	in t 
	end
     
    fun subst_at subst_thm path t n =
	let val sg= #sign(rep_thm(topthm()))
	            (* signature of the current goal *)
	    val (_$ (p $ _))= Lib.List.nth(prems_of subst_thm, 1)
		    (* extract the variable modelling the context *)
		    (* -- this is very specific to subst/ssubst of course *)
	    val cv= cterm_of (#sign (rep_thm subst_thm)) p
		    (* certify the variable *)
	    val ct= TermSel.cut_context sg (getgoal n)
		                           (TermSel.decode path)
		    (* cterm representing the context *)
	in
	    (rtac (Fix.cterm_instantiate [(cv, ct)] subst_thm) n)
	    THEN (REPEAT (rtac ext n))
	    THEN (rtac t n)
	         (* (if null(t) then (atac n) else ( rtac (hd t) n) *)
	end

    fun FOLD_AT_PATH path t n = subst_at subst path t n

    fun UNFOLD_AT_PATH path t n = subst_at ssubst path t n

end
