(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/isa_win/types.sml,v $
    
   IsaWin -- the basic data structures (and some auxiliary functions)
   
   $Date: 1998/09/11 09:08:48 $
   $Revision: 1.12.4.1 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

    
structure IsaWinTypes = 

struct

    local open TkTypes SmlTk in

    exception IsaWinError   of string
    exception InternalError of string
   
    datatype thmKind     = introThm | elimThm | destrThm | rewrRule | unknown

    datatype objtype	= thm_type
			| thy_type
			| proof_type
			| simpset_type
			| claset_type
			| tac_type
			| subst_type
			| text_type

    (*  the type variable 'a will be instantiated with the ``state'' as
     *  provided by the history module in due turn (in ConArea).
     *)
			  
    datatype 'a object'	= isaThm   of thm * thmKind ref * string ref 
	                | isaSimpS of simpset* string ref
			| isaCS    of claset* bool ref* string ref
			| isaThy   of Thy.theory * string ref
			| isaPrf   of 'a * string ref
			| isaTac   of (int -> tactic) * string ref
			| substObj of (string*string) list ref * string ref 
			| textObj  of string ref * string ref

    type 'a new_object' = 'a object' * TkTypes.Coord* TkTypes.AnchorKind

   fun obj_type(isaThm _)   = thm_type
     | obj_type(isaThy _)   = thy_type
     | obj_type(isaSimpS _) = simpset_type
     | obj_type(isaCS _)    = claset_type
     | obj_type(isaPrf _)   = proof_type
     | obj_type(isaTac _)   = tac_type
     | obj_type(substObj _) = subst_type
     | obj_type(textObj _)  = text_type

 
   fun is_tool _ = false
  
   fun is_primary proof_type = true
     | is_primary _	    = false

   (* --- selector functions ---------------------------------- *)

   fun selNameRef (isaThm(_, _, t))= t
     | selNameRef (isaThy(_, t))   = t
     | selNameRef (isaSimpS(_, t)) = t
     | selNameRef (isaCS(_, _, t)) = t
     | selNameRef (isaPrf(_, t))   = t
     | selNameRef (isaTac(_, t))   = t
     | selNameRef (substObj(_, t)) = t
     | selNameRef (textObj(_, t))  = t
       
   val sel_name = ! o selNameRef
       
   fun set_name (ob, nu) = (selNameRef ob):= nu
 

   (* --- partial selectors  ---------------------------------- *)

   fun selThmKindRef (isaThm(_, k, _))=  k
       
   val selThmKind     = ! o selThmKindRef
   fun setThmKind t k = (selThmKindRef t):= k
              
   fun selTextRef (textObj(t, _))= t

   val selText      = ! o selTextRef
   fun setText ob t = (selTextRef ob):= t

   fun selThm (isaThm(th, _, _)) = th

   fun selSubstRef (substObj(s, _))= s
   val selSubst        = ! o selSubstRef
   fun setSubst(oo, s) = (selSubstRef oo):= s

   (* typing lists of objects and theorems 
    * if all have the same type, that'd be the type of the list, otherwise
    * it's some bottom type
    *)
       
   fun typeList typesingle bottom [x]     = typesingle x
     | typeList typesingle bottom (x:: r) = 
       let val t= typesingle x
       in  if t= typeList typesingle bottom r then t
	   else bottom
       end
     | typeList typesingle bottom []      = bottom
       
    fun type_list typesingle [] = NONE
      | type_list typesingle (l as h::_) = 
        let val tl= map typesingle l
        in  if forall (fn t=> forall (fn s=> s=t) tl) tl 
		then SOME (typesingle h)
            else NONE
        end

   val objlist_type = type_list obj_type
       
   fun thmListKind tl = case type_list selThmKind tl of 
                          SOME tt => tt
                        | NONE    => unknown

    (* some auxiliaries *)

    fun printAsList p []     = ""
      | printAsList p [x]    = (p x)
      | printAsList p (t::l) = (p t)^", "^(printAsList p l)
	
    fun prtThms l = printAsList sel_name l

   (*
    * Debug me!
    *)
   fun debugmsg msg = Debug.print 21 ("IsaWin: "^msg)
   fun debugon  ()  = (Debug.on 20;  Debug.on 21)
   fun debugoff ()  = (Debug.off 20; Debug.off 21)

   end
   
end







