(* ***************************************************************************

   $Source: /home/uniform/rep/isa_gui/isa_win/Attic/zarea.sml,v $

   IsaWin -- the workspace, erh, construction area 
   
   $Date: 1998/09/11 09:08:50 $
   $Revision: 1.1.2.1 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen

  ************************************************************************** *)
	

functor ConArea(conf: ISAWINCONF) =

struct

    structure Widgets= ConAreaWidgets(conf)

    local open TkTypes SmlTk IsaWinTypes Lib.List Widgets
    in

	structure utilWin = UtilWin(conf.uw)
	structure enterWin= EnterWindows(conf.uw)

    (*
     * Instantiate the focus package
     *)

	fun string_of_topthm topthm =
	let val m    = nprems_of topthm
	    (* get rid of Trueprop(_) embedding surrounding each subgoal 
	     * This only works for HOL, of course!
	     *)
	    fun extract (_ $ t) = t
	    fun subgoalStr i = "<SUB "^(Int.toString i)^">\n"^
		(Int.toString i)^". "^ 
		(TermSel.string_of (theory_of_thm topthm) (SelTactics.getgoal i))^
		"\n<\\SUB>\n"
	in  Lib.String.concat (map subgoalStr (BasicUtil.-- (1,m)))
	end
		
	structure focus = Focus(type prfstate= thm
				val string_of_prfstate= string_of_topthm)
				

	fun updPrfState (w, (t, p, d), level)= 
			        (focus.display(w, topthm());
				 updLastCmd w p)

	fun exn_handler (_, desc, _) ERROR = 
	               utilWin.warning ("Application of the following tactic \
	                                \failed:\n\n"^desc^"\n") (K())
	  | exn_handler _ (SIMPLIFIER (str, th)) =
	               utilWin.warning "Application of tactic failed:\n\n\
				     \Rewrite Rule not applicable\n" (K())
            (* exceptions from the induction tactics *)		       
	  | exn_handler _ (InductChooser.InductAmbiguous s) =
	    utilWin.warning ("There's more than one induction possible:\n"^
			     (BasicUtil.concatWith "\n" s)) (K())
          | exn_handler _ (InductChooser.Induct err)= utilWin.warning err (K())

	(* Please note that uncaught exceptions arising when executing a tac-
	 * tic will show up as ``uncaught exection Match'' for this function.
	 *)
	    
	structure history= AncientHistory(structure Thy= Thy
                                          val upd_prfstate= updPrfState 
					  val error_handle= exn_handler
					  structure conf= conf)

	 
    (* --- theworkspace type ----------------------------------------------
     *
     * The workspace holds the Ids of the widgets displaying the subgoals,
     * and the last command. 
     *) 
    type ca = WidId* WidId
	


    (* --- the tactics ----------------------------------------------------- *)

    (*
     * Following are functions which ultimately take a list of theorems 
     * and figure out a reasonable tactic corresponding to the operation
     * of "applying" the theorems to the construction area. 
     *)

    
    (*
     * allToSingle makes tactic tac into a tactic only applied 
     * to the selected subgoal, if there is any. singleToAll does the 
     * opposite, it makes a tactic tac into a tactic applied to
     * all goals if there is no subgoal selected. Both functions
     * return a pair of a tactic and its textual description.
     *)
    fun allToSingle (tac, str) =
	case (focus.selSubgoal()) of
	    NONE => (tac, str) (* ALLGOALS *)
	  | SOME i => (SELECT_GOAL tac i, 
		       "SELECT_GOAL "^(Int.toString i)^" "^str)
		
    fun singleToAll (tac, str) = 
	case (focus.selSubgoal()) of
	    NONE    => (ALLGOALS tac, "ALLGOALS "^str)
	  | SOME i  => (tac i, str^" "^(Int.toString i))


    (* this takes a list of theorems and returns a single theorem.
     * It generates a warning if passed more than one argument.
     * This is for use with tactics where it doesn't make sense to have
     * more than one argument (eg. FOLD_AT_PATH)
     *)
    fun listToSingleThm []      = raise (InternalError "Empty list of theorems.")
      | listToSingleThm [th]    = th
      | listToSingleThm (th::_) = 
        (utilWin.warning "Warning: only the first argument will be used." (K());
	 th)

	    
    fun getRewrRule thms =
	let fun selRewrTac(tac, name) =
	    (tac (map selThm thms), name^" ["^(prtThms thms)^"]")
	in
	    case (hd (explode (readVarValue rewrTacVar))) of
		"r" => selRewrTac(rewrite_tac, "rewrite_tac")
	      | "s" => selRewrTac(rewrite_goals_tac, "rewrite_goals_tac")
	      | "f" => selRewrTac(fold_tac, "fold_tac")
	      | "g" => selRewrTac(fold_goals_tac, "fold_goals_tac")
	      | str => raise (InternalError 
			       ("rewrTacVar has undefined value \""^str^"\""))
	end

    fun getRewrRuleAt path thm =
	(* get rewrite rules for rewriting a subgoal *)
	(* bit of a hack, this-- there should be a specific 	
	 * selection between fold/unfold and rewrite goals/no rewrite goals 
	 *)
	let fun mkTac(tac, nm) = (tac path (selThm thm), 
				  nm^" "^path^" "^(sel_name thm))
	in
	    case (hd (explode (readVarValue rewrTacVar))) of
		"r" => mkTac(SelTactics.UNFOLD_AT_PATH, "UNFOLD_AT_PATH")
	      | "s" => mkTac(SelTactics.UNFOLD_AT_PATH, "UNFOLD_AT_PATH")
	      | "f" => mkTac(SelTactics.FOLD_AT_PATH, "FOLD_AT_PATH")
	      | "g" => mkTac(SelTactics.FOLD_AT_PATH, "FOLD_AT_PATH")
	end

    (* Execute a tactic.
     * In order to have IsaWin running in parallel with TAS,
     * executing a tactic should leave the proof state alone. 
     * (But it doesn't, for the time being.)
     *)
    fun exec w (tac, desc) =
	(
	 history.exec w (tac, desc, desc)
	)
    		   
  
    (* gets passed a tactic and a description, it queries for a substitution
     * and exceutes the instantiated tactic. 
     *)
    fun execInstTac(tw, tac, descr, thm) =
	let fun getVars (s $ t) =(getVars s) union_string (getVars t)
	      | getVars (Var ((s, i), t))= [s] (* could be wrong *)
	      | getVars (Free (s, t))= [s] (* ?? *)
	      | getVars (Abs(_, _, b))= getVars b
	      | getVars _ = []
	    val variables= getVars (#prop(rep_thm (selThm thm)))
	    val thnm= (sel_name thm)
            val prtSubst    = printAsList (fn (v, i)=> "("^v^", "^i^")")
	    fun exTac subst = exec tw 
			       (singleToAll
			          (tac subst (selThm thm),
				   descr^" ["^(prtSubst subst)^"] "^thnm))
	in  enterWin.newSubst("Enter subsitution for "^thnm, variables,
		              fn s=> exTac (filter (fn (_,v)=> (not (v= ""))) s))
                                       (* get rid of variables w/o value *)
	    
	end
		
    (* generate a tactic (along with a description of it) from
     * a list of theorems, according to the types of theorems
     * and the current tactical settings, and execute it.
     * 
     * The rational here is as follows:
     * - If the "tac" button on the con/area is set to "rtac", "etac" or
     *   "dtac", then resolve_tac, eresolve_tac or dresolve_tac are selected.
     * - If the "tac" button is set to "auto", the kind of the theorem deter-
     *   mines the tactic.
     * - If additionally the "inst" button is set to "inst", then the 
     *   explicit substitution variants are chosen, except for the rewrite
     *   rules, which do not have such a variant.
     *)
	
    fun execThmsTac(tw, thms) =
	let val rTac = (resolve_tac,  rtac, res_inst_tac,
			"resolve_tac",  "rtac", "res_inst_tac")
	    val eTac = (eresolve_tac, etac, eres_inst_tac,
			"eresolve_tac", "etac", "eres_inst_tac")
	    val dTac = (dresolve_tac, dtac, dres_inst_tac,
			"dresolve_tac", "dtac", "dres_inst_tac")
	    fun exTac (tac, shortcut, insttac, name, shortnm, instnm) thms =
		(if (readVarValue explSubstVar) = "0" then
		     if (length thms)= 1 then
			 exec tw (singleToAll(shortcut (selThm (hd thms)), 
					      shortnm^" "^(sel_name (hd thms))))
		     else
			 exec tw (singleToAll(tac (map selThm thms), 
					      name^" ["^(prtThms thms)^"]"))
		 else
		      execInstTac(tw, insttac, instnm, listToSingleThm thms)
		 )
	in case (hd (explode (readVarValue backTacVar))) of
	    "x" => (case (thmListKind thms) of
			introThm => exTac rTac thms
		      | elimThm  => exTac eTac thms
		      | destrThm => exTac dTac thms
		      | rewrRule => 
			    (case (focus.selFocus()) of
				 NONE => 
				     exec tw (allToSingle(getRewrRule thms))
			       | SOME path => 			    
				     exec tw (singleToAll(getRewrRuleAt path 
						     (listToSingleThm thms))))
		      | unknown  => exTac rTac thms)
	  | "r" => exTac rTac thms
	  | "e" => exTac eTac thms
	  | "d" => exTac dTac thms
	  | str  => raise (InternalError ("backTacVar has undefined value \""^str^"\""))
	end

    fun getSimpTac(sset, nm) =
	(* get the appropriate tactic for a simp_set; i.e. 
	 * asm_full_simp_tac, asm_tac or simp_tac as decided by the
	 * menue button in the con/area.
	 *)
	let fun mkTac(tac, td)= (tac sset, td^" "^nm)
	in case (hd (explode (readVarValue simpTacVar))) of
	      "a" => mkTac(asm_simp_tac, "asm_simp_tac")
	    | "c" => mkTac(asm_full_simp_tac, "asm_full_simp_tac")
	    | "f" => mkTac(full_simp_tac, "full_simp_tac")
	    | "s" => mkTac(simp_tac, "simp_tac")
	    | str => raise (InternalError ("backTacVar has undefined value \""^str^"\""))
	end

    fun getClaSetTac(cs, nm) =
	(* get the appropriat tactic for a claset, ie. fast_tac, best_tac,
	 * slow_tac or slow_best_tac depending on the menue button in the
	 * con/area.
	 *)
	let fun mkTac(t, d)= (t cs, d^" "^nm)
	in case (hd (explode (readVarValue clasTacVar))) of
	       "b" => mkTac(best_tac, "best_tac")
	     | "f" => mkTac(fast_tac, "fast_tac")
	     | "s" => mkTac(slow_tac, "slow_tac")
	     | "l" => mkTac(slow_best_tac, "slow_best_tac")
	     | str => raise (InternalError ("clasTacVar has undefined value \""^str^"\""))
	end 
	 
				   
    (* 
     * -- special tactics --
     *
     * The special tactics, bound to the buttons in the lower left-hand
     * side of the w/s, are implemented by calling history.exec (see above)
     * with a pair of a tactic and its textual description (is that
     * English?). 
     *
     *)
	
    fun doAtac w = 
	exec w (singleToAll (atac, "assume_tac"))

    fun doStripTac w =
	exec w (singleToAll (strip_tac, "strip_tac"))
	
    fun doStripBindTac w =
	 exec w (singleToAll (strip_sbinder, "strip_sbinder"))
	 
    fun doExpSchemaTac w =
    	 exec w (singleToAll (fn i=> (expand_schema AddBirthday_def i)
    	 			     THEN (expand_schema BirthdayBook_def i),
    	 		      "expand_schema"))
    fun doRepeatConjETac w =
    	 exec w (singleToAll (repeatConjETac, "repeatConjETac"))

    fun doRepeatDeferTac w =
    	 exec w (singleToAll (repeatDeferTac, "repeatDeferTac"))

    fun doFastTac1 w =
	exec w (allToSingle  (Fast_tac 1, "FastTac1"))
  fun doFastTac2 w =
	exec w (allToSingle  (Fast_tac 2, "FastTac2"))

    local 
	open InductChooser 
    in  fun doInduct w = 
	let fun firstTry variable = 
	    exec w (singleToAll (INDUCT "" variable, 
				 "INDUCT \"\" \""^variable^"\""))
	    handle InductAmbiguous s => 
			    utilWin.warning ("There's more than one induction possible:\n"^(BasicUtil.concatWith "\n" s)) (K())
		 | Induct err => utilWin.warning err (K())
        in	           
	    utilWin.enterLine "Induction Variable"
	      "Please enter the variable you want to do induction on: "
	      10 firstTry (K())
	end
    end


    fun doQED cc = 
	let val np= nprems_of (topthm())
	in case np of
	    0 => (cc (result())
	          handle ERROR => utilWin.error 
		                 "Result can't be proven yet.\n" (K())
		 )
	  | 1 => utilWin.error ("There is still one unproven subgoal!") (K())
	  | _ => utilWin.error ("There are still "^(makestring np)^
				" unproven subgoals!") (K())
	end

	
    (* the operations *)
	
    fun area_ops thm_type   (tw, lw) thms    = execThmsTac(tw, thms)
      | area_ops thy_type    ws _            = ()
      | area_ops simpset_type (tw, lw) []    = ()
      | area_ops simpset_type (tw, lw) (isaSimpS(ss, nm)::_) = 
	                          exec tw (singleToAll(getSimpTac(ss, !nm)))
      | area_ops claset_type (tw, lw) []     = ()
      | area_ops claset_type (tw, lw) (isaCS(cs, _, nm)::_) =
                                  exec tw (singleToAll(getClaSetTac(cs, !nm)))
      | area_ops proof_type  ws _ = ()
      | area_ops tac_type    ws _ = ()
      | area_ops subst_type  ws _ = ()
      | area_ops text_type   ws _ = ()
      | area_ops bottom_type ws _ = ()

    fun area_init ()= initWidgets()

       	
    fun area_open(win, isaPrf(state, nm), cc) =
	let
	    val wid       = newWidgetId() 
	    val twid      = subgoalTextWidId wid
	    val thms      = history.restore state
	    fun close ()  = cc (isaPrf(history.save(), nm))
		              (* close as an ongoing proof *)
	    fun qed th    = cc (isaThm(th, ref unknown, nm))
	                      (* close as theorem *)
	    val widgs     = wsWidgets(win, wid, 
				      fn()=> doInduct twid,
				      fn()=> doStripTac twid,
				      fn()=> doAtac twid,
				      fn()=> history.forward twid,
				      fn()=> history.backward twid,
				      fn()=> history.show "Proof history",
				      !nm, close, fn()=> doQED qed,
				      fn()=> doStripBindTac twid,
				      fn()=> doExpSchemaTac twid,
				      fn()=> doRepeatConjETac twid,
				      fn()=> doRepeatDeferTac twid,
				      fn()=> doFastTac1 twid,
				      fn()=> doFastTac2 twid )
	in  ((twid, lastCmdTextWidId wid), widgs,
	     fn()=> updPrfState (twid, history.last(), history.level()))
	end

    end

end 
     

