(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/isa_win/Attic/zwidgets.sml,v $
 
   IsaWin -- The widgets for the construction area.
   
   $Date: 1998/09/11 09:08:52 $
   $Revision: 1.1.2.1 $
   Author: cxl (Last modification by $Author: cxl $)
 
   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)
 


functor ConAreaWidgets(conf: ISAWINCONF) =

struct
    
    local open TkTypes SmlTk IsaWinTypes Lib.List in    


    (* The "Tactic Selection Buttons" *)
	
    (* names of the Tcl variables holding the selected tactics     *)
    val backTacVar   = "backTac"
    val explSubstVar = "subst"
    val rewrTacVar   = "rewrTac"
    val simpTacVar   = "simpTac"
    val clasTacVar   = "clasTac"
	
    (* names for the text widgets holding the subgoals (proof state) and
     * the last command 
     *)
    fun subgoalTextWidId wid = subWidId(wid, "xSubgoals")
    fun lastCmdTextWidId wid = subWidId(wid, "xLastCmd")
	

    (* --- All the buttons on the left side ------------------------------ *)
	
    fun tacButtons wid specTacs =
	
	let
	    (* given a list of label-value pairs, generates a menu 
	     * button with a menu of radiobuttons attached... 
	     * the list must be non-empty, and the first entry is the
	     * default. 
	     *)
	    fun mkMenuButton varname ls =
		let val wid   = Paths.newWidgetId()
		    fun selButtonConfig wid varname (label, value) =
			MRadiobutton([Command (fn () => addConf wid [Text label]), 
				      Variable varname, Value value, Text label])
		    val defLab = fst (hd ls)
		in  Menubutton(wid, true,
			       map (selButtonConfig wid varname) ls,
			       [Side Top, Fill X], 
			       [Width 10, Text defLab, Relief Groove,
				Font conf.wsButtonFont], [])
		end

	    (* Upper Part: the standard tactic settings *)
    
	    (* The backward resolution tactics: auto, rtac, etac, dtac *)
	    val backTacMenu = 
		mkMenuButton backTacVar
		[("auto", "x"),
		 ("rtac", "r"),
		 ("etac", "e"),
		 ("dtac", "d")]
	
	    (* rewriting tactics: unfold/fold with or without main goal *)		    val rewrTacMenu =
		mkMenuButton rewrTacVar
		[("rewrite_goals", "s"),
		 ("fold", "f"),
		 ("rewrite", "r"),
		 ("fold_goals", "g")]
	
	    val explSubstMenu = 
		mkMenuButton explSubstVar 
		[("no inst", "0"),
		 ("inst", "1")
		]
	
	    (* simplifier set tactics *)
	    val simpTacMenu =
		mkMenuButton simpTacVar
		[("asm_simp", "a"), 
		 ("asm_full_simp", "c"),
		 ("simp",     "s"),
		 ("full_simp", "f")
		]
		
	    (* claset tactics *)	
	    val clasTacMenu =
		mkMenuButton clasTacVar 
		[("fast", "f"),
		 ("best", "b"),		
		 ("slow", "s"),
		 ("slow_best", "l")]
 	
(*
	    (* explicit substitutions yes or no (as checkbutton) *)
	    val explSubst =
		Checkbutton(SubWidId(wid, "xSubst"), [Side Top, Fill X], 
			    [Variable explSubstVar, Text "Subst"], [])
 *)		
		
	    (* Lower part: the (more or less) special tactics. *)

	    fun specialTacBut(label, tictac) =
		Button(newWidgetId(),
		       [Side Bottom, Fill X], 
		       [Width 10, Text label,
			Relief Groove, Font conf.wsButtonFont,
			Command tictac], [])
		
	    (* title lables *)
	    val tacSetTitle =
		Label(subWidId(wid, "xTacSetTitle"),
		      [Side Top, PadY 3],
		      [Text "Tactic\nsettings", Font conf.wsButtonFont], [])
		
	    val tacButTitle =
		Label(subWidId(wid, "xTacButTitle"), 
		      [Side Bottom, PadY 3],
		      [Text "Tactics", Font conf.wsButtonFont], [])
	in	
	    
	    Frame(subWidId(wid, "xTacBut"),
		  [tacSetTitle, 
		   backTacMenu, explSubstMenu, rewrTacMenu, simpTacMenu, clasTacMenu] 
		  @ (map specialTacBut specTacs),
		  [Side Left, Fill Y, PadX 5, PadY 5, Expand true], [], [])
	end
	
    (* --- The main goal widgets on the right side ------------------------ *)
	
    val bitOnTheLeft = 120 (* an obscure constant *)

    (* These two variables toggle the display of the main goal and 
     * last cmd, respectively 
     *)
    val showLastVar = "showLast"
    val showGoalVar = "shallWeShowTheMainGoalDoYouThink"	


    (* Id's for some of the widgets used below *)
    fun lastCmdWidId w     = subWidId(w, "xLCmdFrm")
    fun mainGoalWidId w    = subWidId(w, "xMainGoal")
    fun goalFrameWidId w   = subWidId(w, "xGoalFrm")
   
    fun showLastCmd () = 
	case readVarValue showLastVar of
	    "1" => true
	   | _  => false

    fun showMainGoal () = 
	case readVarValue showGoalVar of
	    "1" => true
	   | _  => false

    fun mainGoal wid = 
	let val (txt, ann) = 
	      StdMarkup.getAnnotation (wid^"xMGTxt")
	           (SGML.string_of_term 
	                (theory_of_thm(topthm())) (concl_of(topthm())))	     
	in Frame(mainGoalWidId wid,
		 [Frame(newWidgetId(),
			[Message(newWidgetId(), [Side Top], 
				 [Text "To Show:  ", 
				  Width bitOnTheLeft], [])],
			[Side Left, Fill Y], [], []),
		  TextWid(wid^"xMGTxt", NoneScb, txt, ann,
			  [Side Top, Fill X], 
			  [Relief Ridge, Height 1, TextWidReadOnly false], [])
		  ], [PadY 5, Side Top, Fill X], [], [])
	end


    fun subgoals wid =
	Frame(newWidgetId(),
	      [Frame(newWidgetId(),
		     [Message(newWidgetId(), [Side Top], 
			      [Text "Subgoals:", Width bitOnTheLeft], [])
		      ], [Side Left, Fill Y], [], []),
	       TextWid(subgoalTextWidId wid, LeftScb, "", [], [], 
	            [TextWidReadOnly false], [])
	      ], [PadY 5, Fill Both], [], [])

		
    fun lastCmd wid =
	Frame(lastCmdWidId wid,
	      [Frame(newWidgetId(),
		     [Message(newWidgetId(), [Side Top], 
			      [Text "Cmds:      ", Width bitOnTheLeft], [])
		      ], [Side Left, Fill Y], [], []),
	      TextWid(lastCmdTextWidId (subgoalTextWidId wid), NoneScb, "", [],
		      [Side Top, Fill X], [Height 3, Font conf.ttFont], [])],
	      [PadY 5, Side Bottom, Fill X], [], [])
		
		
    (* all the widgets on the right/main side o/t workspace *)
    fun goalFrame wid  =
	let val widgs = (if showMainGoal() then [mainGoal wid] else [])@
		        (if showLastCmd()  then [lastCmd wid ] else [])@
			     [subgoals wid]
	in  Frame(goalFrameWidId wid, widgs,
		  [Side Top, Fill Both], [Relief Ridge], [])
	end

    (* toggle display of main goal and last command *)

    fun toggleMainGoal win wid =
	if showMainGoal() then
	    addWidget win (goalFrameWidId wid) (mainGoal wid)
	else
	    delWidget (mainGoalWidId wid)

    fun toggleLastCmd win wid =
	if showLastCmd() then
	    addWidget win (goalFrameWidId wid) (lastCmd wid)
	else
	    delWidget (lastCmdWidId wid)

    (* update the last command *)
    fun updLastCmd wid cmd =
	if showLastCmd() then 
	    replaceTextWidText (lastCmdTextWidId wid) cmd []
	else ()

	
    (* --- The menu ------------------------------------------------------ *)

    val menuWidth   = 8

    fun wsMenu(win, w, cc, qed, forw, backw, show, 
    	       stripBindTac, expSchemaTac, repeatConjETac, 
    	       repeatDeferTac, FastTac1,FastTac2) =
	let	    
	    val wsMenState =
		Menubutton(subWidId(w, "xMenState"), false,
			   [MCommand [Text "Qed", Command  qed],
			    MCommand [Text "Result"], 
			    MSeparator,
			    MCommand [Text "Close", Command cc]],
			   [Side Left], [Text "State"], [])
		
	    val wsMenViews =
		Menubutton(subWidId(w, "xMenView"), false,
			   [MCheckbutton [Text "Show last cmd", 
					  Variable showLastVar,
					  Command (fn ()=> toggleLastCmd win w)],
			    MCheckbutton [Text "Show main goal",
					  Variable showGoalVar,
					  Command (fn ()=> toggleMainGoal win w)]],
			   [Side Left], [Text "View", Width menuWidth], [])
		
	    val wsMenHistory =
		Menubutton(subWidId(w, "xMenHist"), false,
			   [MCommand [Text "Back", Command backw],
			    MCommand [Text "Forward", Command forw],
			    MCommand [Text "Show history", Command show]],
			   [Side Left], [Text "History", Width menuWidth], [])
            val	wsSpecTacticsMenu = 
            	Menubutton(subWidId(w, "xSpecTacMen"), false,
              [MCommand [Text "Strip binder",Command stripBindTac],
               MCommand [Text "Expand schema",Command expSchemaTac],                           MCommand [Text "Eliminate Conjunction",  Command repeatConjETac],               MCommand [Text "Introduce Conjunction", Command repeatDeferTac],
               MCommand [Text "FastTac_1", Command FastTac1],
               MCommand [Text "FastTac_2", Command FastTac2]],
               [Side Left], [Text "Special Tactics"], [])
 	
	    val wsHelpMenu =
		Menubutton(subWidId(w, "xMenHuelf"), false,
			   [MCommand [Text "About Tactic Settings"],
			    MCommand [Text "About Subgoals"],
			    MCommand [Text "About Isabelle"],
			    MCommand [Text "About This Theory"],
			    MCommand [Text "About the Authors"]],
			   [Side Right], [Text "Help"], [])
        in 
	    Frame(subWidId(w, "xMenu"), 
		  [wsMenState, wsMenHistory, wsMenViews, wsSpecTacticsMenu,
		   wsHelpMenu],
		  [Side Top, Fill X, Expand true], [Relief Raised], [])
	end

	


    (* Title board (the bit on the top) *)
	
    fun titleBoard nm = 
	Label(newWidgetId(), 
	      [Side Top, Fill X, PadY 2], 
	      [Text nm, Relief Groove, Borderwidth 3], [])
	
    (* all our widgets *)
	
    fun wsWidgets(win, wid,  doInduct, doStripTac, doAtac, 
		  forw, backw, show, nm, close, qed, 
		  stripBindTac, expSchemaTac, repeatConjETac,
		  repeatDeferTac, FastTac1,FastTac2 ) = 
	let val mws= wsMenu(win, wid, close, qed, forw, backw, show, 
		            stripBindTac, expSchemaTac,
		            repeatConjETac,repeatDeferTac, 
		            FastTac1,FastTac2) ::
	              tacButtons wid [("induct", doInduct),
				      ("strip_tac", doStripTac), 
				      ("assumption", doAtac)] ::
		      [goalFrame wid]
	in if conf.oneWindow then titleBoard nm :: mws
	   else mws
	end


    (* initialization function for the widgets
     * This is called only once, because values should be preserved
     * over open/closes 
     *)

    fun initWidgets() = 
	(setVarValue backTacVar   "x";
	 setVarValue explSubstVar "0";
	 setVarValue rewrTacVar   "s";
	 setVarValue simpTacVar   "a";
	 setVarValue clasTacVar   "f";
	 
	 setVarValue showLastVar  "1";
	 setVarValue showGoalVar  "1";
	 debugmsg "Initialised variables (Code RED)";
	 debugmsg ("Control Code = "^(readVarValue rewrTacVar))
	 )
    end

end      
