(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/appl.sml,v $
 
   TAS -- the application module

   This module instantiates the GenGUI with "TAS application". It also 
   provides sundry utility structures such as utilWin. 
  
   $Date: 1997/09/19 14:36:53 $
   $Revision: 1.27 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


signature TASGUI = 
    sig
 
	structure trafos: TRAFOS

	datatype dev_op = 
            opApplyTrafo   of trafos.trafo* trafos.inst
          | opApplyTrafoAt of trafos.trafo* trafos.inst* trafos.path
          | dischargeThm   of thm* int

        structure history : 
            sig include HISTORY
                sharing type opn= dev_op
            end

	type trafoSubst = (string* string) list

	datatype object =
	    progDev  of history.history* string* string ref* string ref
	  | trafoObj of trafos.trafo* trafoSubst* string ref
	  | subst    of trafoSubst ref* string ref
	  | sign     of trafos.signatur* string ref
	  | spec     of string ref*  string ref

	include GENGUI 
                sharing type object= object
		    

	structure utilWin : UTIL_WIN	    
	structure enterWin :ENTER_UTILWIN

    end



local functor TasApp(structure trafothy  : TRAFOTHEORY
		     and  conf : TASCONF
		     and isa_clip : ISA_CLIPBOARD) =
    
  struct

    open TkTypes SmlTk 

	
    exception Error of string

    (* Instantiate the basic tactics, the focus and the history package *)
    structure trafos  = Trafos(trafothy)

    structure focus   = Focus(type prfstate= trafos.progdev
			      val string_of_prfstate= trafos.prt_prfstate)

    datatype dev_op = 
	opApplyTrafo   of trafos.trafo* trafos.inst
      | opApplyTrafoAt of trafos.trafo* trafos.inst* trafos.path
      | dischargeThm   of thm* int	
	
    structure history = History(
        structure conf= conf
	structure state= 
	struct
	    type state= trafos.progdev			
	    type opn= dev_op
				
	    fun apply (opApplyTrafo(t, i), p)      = trafos.apply(p, t, i)
	      | apply (opApplyTrafoAt(t, i, c), p) = trafos.apply_at(p, t, i, c)
	      | apply (dischargeThm(t, n), p)      = trafos.discharge_thm(p, t, n)
	
	    fun print (opApplyTrafo(t, i))     = 
		"Apply transformation "^(#name t)
	      | print (opApplyTrafoAt(t, i, c)) = 
		"Apply transformation "^(#name t)^" at "^c
	      | print (dischargeThm(t, n))      = 
		"Discharge proof obligation #"^(Int.toString n)
	end)  
		
    structure utilWin= UtilWin(conf.uw)
    structure enterWin= EnterWindows(conf.uw)


    (* --- Basic Objects ------------------------------------------------- *)

    datatype objtype =
	progDevT | trafoT | substT | signT | specT

    type trafoSubst = (string* string) list

    datatype object =
	progDev  of history.history* string* string ref* string ref
      | trafoObj of trafos.trafo* trafoSubst* string ref
      | subst    of trafoSubst ref* string ref
      | sign     of trafos.signatur* string ref
      | spec     of string ref*  string ref

    type objectlist= object list

    type new_object= object*TkTypes.Coord* TkTypes.AnchorKind
	
    (* --- Basic Operations ---------------------------------------------- *)

    fun obj_type (progDev _)   = progDevT
      | obj_type (trafoObj _)  = trafoT
      | obj_type (subst _)     = substT
      | obj_type (sign _)      = signT
      | obj_type (spec _)      = specT

    fun eq s t= (s= t)

    fun objlist_type [] = NONE
      | objlist_type (l as h::_) = 
	let val tl= map obj_type l 
	in  if forall (fn t=> forall (eq t) tl) tl then SOME (obj_type h)
	    else NONE
	end
    fun is_tool _           = false
	
    fun is_primary progDevT = true
      | is_primary _        = false

(*  fun is_auxiliary specT  = true
      | is_auxiliary substT = true
      | is_auxiliary _      = false *)

    fun selNameRef (progDev(_, _, _, nm)) = nm
      | selNameRef (trafoObj(_, _, nm)) = nm
      | selNameRef (subst(_, nm))       = nm
      | selNameRef (sign(_, nm))        = nm
      | selNameRef (spec(_, nm))        = nm

    fun sel_name  oo           = !(selNameRef oo)		   
    fun set_name  (oo, nuname) = (selNameRef oo) := nuname

    (* Debugging *)	
    fun debugmsg msg = Debug.print 22 ("TAS: "^msg)

    (* --- Icons! ------------------------------------------------------ *)
    
    fun iconName (progDev _)  = "dev.gif"
      | iconName (trafoObj(t, i, _)) = if (not (null (#params t))) 
	                                  andalso (null i) then "untrafo.gif"
					                    (* uninst trafo *)
				       else "trafo.gif" (* inst or no params *)
      | iconName (subst _)    = "sub.gif"
      | iconName (sign _)     = "signature.gif"
      | iconName (spec _)     = "spec.gif"	
	
    fun icon obj = Icons.getIcon(conf.iconPath(), iconName obj)


    (* --- Proof Obligations and the IsaWin connection -------------------- *)

    (*
     * The IsaWin connection is used to discharge proof obligations. 
     * Technically, we exchange IsaWin-objects via IsaWin's clipboard 
     * which is packaged up more nicely in the isa_clip structure.
     *)

    fun prfObMsg(msg, num, wid, aid)=
	debugmsg(msg^" subgoal #"^(Int.toString num)^
		 ", Widget: "^wid^", Ann.: "^aid)

    fun putPrfOb(txwid, aid, pd, num) ev =
	let val po= trafos.get_proof_obl(pd, num)
	    val ct= cterm_of (trafos.get_sign pd) po
	in  (prfObMsg("Unmarking ", num, txwid, aid); 
	     addAnnotationConf txwid aid [Relief Flat];
	     isa_clip.putGoal(ct, "* subgoal #"^(Int.toString num)^"*") ev
	    )
	end

    (* mark a proof obligation *)
    fun markPrfOb(txwid, aid, num) _ = 
        (prfObMsg("Marking ", num, txwid, aid);
	 addAnnotationConf txwid aid [Relief Sunken])

    fun getTheorem (wid, hist, updProofObl, num) ev =
	let val thm= isa_clip.getTheorem ev
	    val pd = history.cur_state (!hist)
	in  (* This is not satisfactory-- it means discharging one
	     * proof obligation is one step in the history. 
	     * One would rather like them to be part of applying the trafo,
	     * but then we would have to remember which obligation arose from
	     * applying which trafo... rather complicated, that.
	     *)
	    if  trafos.can_discharge_thm(pd, thm, num) then
		(hist:= history.exec(!hist, dischargeThm(thm, num));
		 updProofObl(wid, hist))
		handle ERROR => 
		    utilWin.error "Theorem not applicable." (K())
		    (* shouldn't happen, actually :-} *)
	     else
		utilWin.error "Theorem does not prove this obligation." (K())
	end handle isa_clip.Empty=> ()

    structure PrfOblBindings = 
        struct
	    exception parsingError of string 
	    exception BTagError of string
 
            type bindTag= unit
	    type widgetinfo= WidId* (history.history ref)*
		             (WidId* (history.history ref)-> unit)
 
            fun matchingBTag "sub" = SOME () 
              | matchingBTag  _    = NONE
 
            fun annotationForBTag () [] _ _ = 
                    raise (BTagError "<SUB> tag with no argument")
              | annotationForBTag () (num::_) (wid, hist, upd) where = 
		    let val aid=     newAnnotationId()
			val txwid=   Widgets.prfOblWidId wid
			val subgoal= BasicUtil.atoi num
			val pd=      history.cur_state (!hist)
			val binds=   [Bind("<ButtonRelease-1>",
					 putPrfOb(txwid, aid, pd, subgoal)),
				      Bind("<ButtonPress-1>",
					 markPrfOb(txwid, aid, subgoal)),
				      Bind("<Enter>", 
					   getTheorem(wid, hist, upd, subgoal))]
		    in  TATag(aid, [where], [Borderwidth 2], binds)
		    end
        end

    (* instantiate the markup-language parser *)
    structure PrfObTags= SmlTkMarkup(StdTags(PrfOblBindings))

    fun updProofObligations (w, cur_hist) =
	let val pd= history.cur_state (!cur_hist)
	    val num_pos= trafos.num_proof_obl pd
	in
	    (if occursWin Widgets.prfOblWinId then
		 let val txwid= Widgets.prfOblWidId w
		     val num_pos= trafos.num_proof_obl pd
		     fun oneObligation n =
			 "<SUB "^(Int.toString n)^">"^(Int.toString n)^". "^ 
			 (SGML.string_of_term 
			        (theory_of_thm (trafos.top_thm pd))
			        (trafos.get_proof_obl(pd, n)))^
			 "\n<\\SUB>"
		     val (txt, ann)= 
			 if num_pos= 0 then ("", [])
			 else
			     PrfObTags.getAnnotation (w, cur_hist, 
						      updProofObligations)
			     (BasicUtil.concatWith "\n"
			      (Lib.List.tabulate(num_pos,
						 fn i=> oneObligation (i+1))))
		 in   (debugmsg ("Replacing proof obligations in "^txwid);
		       replaceTextWidText txwid txt ann)
		 end
	     else ();
	     debugmsg("Updating widget w="^w);
	     Widgets.setNumPrfOb w num_pos
	     )
	end
		 

    (* Initial objects-- there's just one, TAS' top theory. You get the 
     * transformations via the trafo chooser.
     *)
    
    fun init () = 
	let val thynm = trafothy.Thy.name_of trafothy.topthy
	    val thy   = (sign(trafothy.topthy, ref thynm), 
			 conf.specsHere, conf.moreSpecs)
	    (* might want to check theory-name here *)
	in [thy]
	end


    (* --- Basic Operations for Substitutions and Trafos ------------------ *)

    (* Check for more than one value per parameter          *)    	     
    fun check_uniq sbst =
	let fun checkOne [] = []
	      | checkOne ((p, _):: s) = 
	        if (exists (fn(q, _)=> (p=q)) s) then 
		    p:: (checkOne s) 
		else checkOne s
	in checkOne sbst
	end

    (* check if there is at least one value for all parameters;
     * returns the list of all parameters for which no value has been given 
     *)
    fun check_all(trf:trafos.trafo, sub) = 
	filter (fn p=> (forall (fn (_, q)=> p= q) sub)) (#params trf)

    (* check if there are values for parameters which don't exist
     * returns a list of subst for parameters which actually exist,
     * and those who don't 
     *)
    fun check_toomany(trf:trafos.trafo, sub) =
	partition (fn (p, _)=> (exists (fn q=> p= q) (#params trf))) sub

    (* check if a substitution is ok *)
    fun check_subst(trf, subs) = 
	(null (check_uniq subs)) andalso 
	(null (check_all(trf, subs))) andalso 
	(null (#2 (check_toomany(trf, subs))))

    (* get the value for a parameter from a substitution*) 
    fun get_val sub par = (#2 (hd (filter (fn (q, _)=> par= q) sub)))
	                  handle Empty=> ""

    (*
     * instantiate a trafo with a list of substitutions 
     *)
    fun instTrafo (trafoObj(trf, s0, nm), substs) =
	let val nusub = s0 @ (Lib.List.concat (map (fn subst(s, _)=> !s) 
	                                            substs))
	    val dbl   = check_uniq(nusub)
            val (ok, tm) = check_toomany(trf, nusub)
	in  if (null dbl) then 
	        if (null tm) then 
		    trafoObj(trf, ok, nm)
                else
		    (utilWin.warning ("Following parameters are not relevant \
                                      \for this transformation:\n"^
				      (BasicUtil.concatWith ", " (map #2 tm)))
				     (K());
		     trafoObj(trf, ok, nm))
	    else raise Error ("More than one value for the following parameters:\n\n"^(BasicUtil.concatWith ", " dbl)) 
	end

    (*
     * "Concatenate" two substitutions. Raises an error if there is more
     * than one value for any given parameter
     *)
    fun concatSubst sss =
        let val nusub = Lib.List.concat (map (fn subst(s, _)=> !s) sss)
            val nunm  = BasicUtil.concatWith "+ " (map (fn subst(_, nm)=> !nm) sss)		                             
	    val dbl   = check_uniq(nusub)
	in  if (null dbl) then
	        subst(ref nusub, ref nunm)
            else
		raise Error ("More than one value for the following parameters:\n\n"^(BasicUtil.concatWith ", " dbl))
        end

    (* --- Meta-development: create a transformation ---------------------- *)

    fun progDevToTrafo(hist, sp, inf, nm, where)= 
	let val nutrf= trafos.trafo_from_dev(history.cur_state hist, nm)
	in  (trafoObj(nutrf, [], ref (#name(nutrf))), where, South)
	end


    (* --- the Construction Area ------------------------------------------ *)

    type ca= WidId* history.history ref

    fun updateArea (wid, cur_hist) =
	(updProofObligations(wid, cur_hist);
	 Widgets.setLevel wid ((history.level (!cur_hist))+1);
	 focus.display(Widgets.prfStateWidId wid, 
		       history.cur_state (!cur_hist)))

    fun applTrafo (wid, cur_hist, trf, inst) = 
        let val oo = case focus.selFocus() of
                NONE   => opApplyTrafo(trf, inst)
              | SOME p => opApplyTrafoAt(trf, inst, p) 
        in  (cur_hist := history.exec(!cur_hist, oo);
             updateArea (wid, cur_hist)
            ) handle THM (why, num, thms) =>
	             utilWin.error ("Application of transformation failed: \n"^
				    why) (K())
		   | ERROR => 
		     utilWin.error ("Can't apply transformation!") (K())
		   | trafos.Error err =>
		     utilWin.error ("Can't apply transformation: "^err) (K())
        end

    fun completeInst (w, h) (t as trafoObj(trf, sub, nm)) =
        if check_subst(trf, sub) then 
	   (* turn substitution into instantiation, call applTrafo *)
	     applTrafo(w, h, trf, map (get_val sub) (#params trf))
        else (* query for missing values, call completeInst as continuation
	      * so parameters are checked again. *)
	    let val complSub = map (fn p=> (p, get_val sub p)) (#params trf)
	    in  enterWin.subst("Please enter parameters", complSub,
			   fn sub=> completeInst(w, h) (trafoObj(trf, sub, nm)))
	    end

    fun forward (wid, cur_hist) = 
	(cur_hist := history.forward (!cur_hist);
	 updateArea(wid, cur_hist) 
	                   (* note application of oo above sets proof_state *)
	) handle (history.Error err) => utilWin.error err (K())
	

    fun backward (wid, cur_hist) = 
	(cur_hist := history.backward (!cur_hist);
	 updateArea(wid, cur_hist)
	) handle (history.Error err) => utilWin.error err (K())

    fun area_ops trafoT ca trfs = (Lib.List.app (completeInst ca) trfs
	                           handle trafos.Error err => utilWin.error err (K()))
      | area_ops _ _ _          = ()

    fun area_open(win, progDev(hist, goal, info, nm), cc) =
	let
	    val w        = newWidgetId() (* ``master widget'' *)
	    val cur_hist = ref hist
	    val ca       = (w, cur_hist)
	    fun close () = (debugmsg ("About to close "^(!nm));
			    cc (progDev(!cur_hist, goal, info, nm)))
	    val widgs = Widgets.area_widgs(win, conf.oneWindow, w, !nm, 
			  fn()=> updProofObligations(w, cur_hist), close,
			  fn()=> history.show("Development history: "^ (!nm),
					      !cur_hist),
			  fn()=> forward ca,
			  fn()=> backward ca,
			  fn()=> history.level (!cur_hist)+1)
	    val _     = debugmsg ("About to open "^(!nm))
	    val _     = debugmsg ("We have "^(Int.toString (length widgs))^" widgets")
	in (ca, widgs, (fn()=> updateArea ca))
	end

    fun area_init ()= ()


    (* --- Show operations ------------------------------------------------ *)

    local
	
	fun freeze thm = fst(freeze_thaw thm)

	fun split []     = raise Subscript
	  | split [x]    = ([], x)
	  | split (h::t) = let val (h1, t1)= split t in (h::h1, t1) end      

	fun ifSome (SOME _) t = t
	  | ifSome  NONE    t = ""	    	

 	
	
	fun section h = "<h2>"^h^"</h2>\n"
	
	fun subsection h = "<h4>"^h^"</h4>\n"

	fun prt_term sg t = "<pre>"^(Sign.string_of_term sg t)^"</pre>"

	fun image url     = "<img align=middle src="^url^">"

	fun paramTable parinfolist = 
	    let fun entry (par, inf) =
		"<TD>"^par^"<TD>"^inf^"<TR>\n" in "<TABLE>"^(concat (map entry
								 parinfolist))^ "</TABLE>\n" end
							     
	fun instParams l =
	    let fun entry ((par, valu), info)= "<TD>"^par^"<TD>"^valu^
		"<TD>"^info^"<TR>\n"
	    in  "<TABLE><TD>Parameter<TD>Value<TD>Description<TR>"^
		(concat (map entry l))^"</TABLE>\n"
	    end
    in

    
	fun showTrafo(t:trafos.trafo, s, nm) =
	    let fun sortSubst [] = []
		  | sortSubst (p::rest) = 
		    let val vs= map #2 (filter (fn (q,_)=> (q= p)) s)
		    in  (BasicUtil.concatWith ", " vs):: (sortSubst rest)
		    end
		val partxt= 
		    instParams (ListPair.zip (ListPair.zip(#params t,
							   sortSubst (#params t)),
					      #param_info t))
		val txt =
		    (section ("Transformation "^nm))^	            
		    (#info t)^"<p><p><hr>\n"^
		    (if length (#params t) > 0 then
			 (subsection "Parameters:")^
			 partxt^"<p><hr><p>\n"
		     else "<p>This transformation has no parameters.<p><p><hr><p>\n"
			 )^(subsection "Core Theorem: ")^
			 ("<PRE>"^(string_of_thm (freeze (#core_thm t)))^"</PRE><p>\n")^
			 (ifSome (#sugar t) "This transformation provides additional proof support.</P>")^
			 "</HR></P>"
	    in  HTML.mkHtml("Transformation "^nm, txt)
	    end

	fun prt_state sg pd= 
	    ((prt_term sg (trafos.current_spec pd))	         
		 handle trafos.Error _ => 
		     "<it>Malformed specification.</it>")^"<P>"

        fun prt_devop (opApplyTrafo(trf, inst)) =
	    "Apply transformation <bf>"^(#name trf)^"</bf>"
	  | prt_devop (opApplyTrafoAt(trf, inst, path))=
	    "Apply Transformation "^(#name trf)^"</bf>"
	  | prt_devop (dischargeThm(th, n))=
	    "Discharge p/o #"^(Int.toString n)^" by "^(string_of_thm th)

	fun prt_op opn =
	    image((conf.graphicsURL())^"squiggel.gif")^
	    (prt_devop opn)^"<p>\n"

	fun showDev(hist, nm, info)= 
	    let	val cur = history.cur_state hist
		val sg  = trafos.get_sign cur
		val inf = (subsection "General Information")^info^"<P>\n"
		val ts  = (subsection "Initial Spec")
		    ^(prt_term sg (trafos.initial_spec cur))^"<P><P>\n"
		val cg  = (subsection "Current Spec")
		    ^(prt_state sg cur)^
		    "<P>Currently at level "^
		    (Int.toString (history.level hist))^"<P>\n"
		val pos = trafos.all_proof_obl cur
		val po  = if (length pos)> 0 then
		    (subsection "Proof obligations")
		    ^"<OL>"^(concat (map (fn t=> "<LI>"^(prt_term sg t)) pos))^
		    "</OL><P>\n"
			  else
			      (subsection "There are no proof obligations")
		val hist = (subsection "Development history")^
		    (history.print(hist, prt_state sg, prt_op))
		val txt = (section ("Program Development: "^nm))^info^ts^cg^po^hist
	    in HTML.mkHtml ("Program Development: "^nm, txt)
	    end 
    end
	    	
    fun show (progDev (hist, _, info, nm)) = 
	                                HTML.display(showDev(hist, !info, !nm))
      | show (trafoObj(trf, s, nm))   = HTML.display(showTrafo(trf, s, !nm))
      | show (subst(s, nm))           = enterWin.subst(!nm, !s, fn _ => ())
      | show (sign _)                 = ()
      | show (spec(s, nm))            = utilWin.display (!nm) 300 NONE (!s) (fn _=> ()) 


    (* --- Other operations ----------------------------------------------- *)
	    
    fun stat _ = ()

    fun delete _ = ()

    fun editInfo (info, nm) =
	(utilWin.enterText ("Infotext for "^nm) 
	                   "Please edit information:"
			   (!info) 40 25
			   (fn t=> info:= t) (K());
         ())			  

    fun editSpec (sp, nm) =
	(utilWin.enterText ("Specification "^nm) 
	                   "Please edit specification below:"
			   (!sp) 80 25
			   (fn t=> sp:= t) (K());
         ())			  


    fun editSubst(s, nm) =   
	(enterWin.subst(nm, !s, fn nusubst=> s:= nusubst); 
	 ()) 


    (* --- The mon_ops table and the drag& drop table --------------------- *)


    fun mon_ops substT   = [(fn (subst(s, nm), _) => fn _=> editSubst(s, !nm), 
			    "Edit")]
      | mon_ops progDevT = [(fn (progDev(h, sp, info, nm), wh) => fn cc =>
			     (cc (progDevToTrafo(h, sp, !info, !nm, wh))
			      handle ERROR => 
				  utilWin.error "Can't close this development."
				  (K())),
			     "Create transformation"),			     
			    (fn (progDev(_, _, info, nm), _) => 
			                           fn _ => editInfo(info, !nm),
			    "Edit info")]
      | mon_ops specT    = [(fn (spec(sp, nm), _) => fn _=> editSpec(sp, !nm),
			    "Edit")]
      | mon_ops _        = []
	(* more possible mon_ops include editing specs and starting a 
	 * prov in a given signature.
	 *)


    fun bin_ops (trafoT, substT) =
	let fun inst(trf, c) subs = [(instTrafo(trf, subs), c, South)]
	                            handle Error err=>
					(utilWin.error err (K());
					 [])
	in  SOME inst
	end
      | bin_ops (substT, substT) =
	let fun concat(s0, c) subs = [(concatSubst (s0::subs), c, South)]
	                             handle Error err=>
					 (utilWin.error err (K());
					  [])
	in  SOME concat
	end
      | bin_ops (signT, specT) =
	let fun startDevs(sign(thy, _), c) (spec(txt, nm)) = 	      
	             (SOME (progDev(history.new(trafos.start_dev(thy, !txt)),
			     !txt, ref "", nm), c, South))
		     handle ERROR => (utilWin.error 
				      ("An Error occured.\n\
				       \Text can't be parsed, typechecked, or\n\
				       \it doesn't make sense.") (K()); NONE)
					            
	in  SOME (fn sgc=> Lib.List.mapPartial (startDevs sgc))
	end
				       
      | bin_ops (_, _) = NONE

  (* other substructures *)

    structure Conf= conf

    structure CB =  Clipboard(struct type obj=objectlist end)

  end


in  (* Instantiate GenGUI *)

  functor TasGUI(structure trafothy : TRAFOTHEORY
		     and  conf      : TASCONF
		     and isa_clip   : ISA_CLIPBOARD) : TASGUI =
    struct
	structure appl= TasApp(structure trafothy= trafothy
			       structure conf= conf
			       structure isa_clip= isa_clip)
	structure gui= GenGUI(structure appl= appl)
	    
	open gui appl (* Note order matters here ! *)
	    
	val  init= gui.init (* want to export init from gen_gui *)
    end

end




