(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/examples/init.sml,v $
    
   TAS -- initial objects.
   
   $Date: 1997/04/11 12:30:19 $
   $Revision: 1.9 $
   Author: cxl (Last modification by $Author: tm $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


structure HOLExamples =
    
struct


(* --- Initial Goals ----------------------------------------------------- *)

val simpleGoal = 
    (Arith.thy, "Prog((a::nat)+b) = ?X", 
     "Just a very simple example.", 
     "Wee Example")


val globalGoal = 
    (Fmap.thy,
     "((% (U,V). U:Fin(U) & V:Fin(V))(A,B) --> \
      \(fin_maps (A,B) M = (% (U,V) N. N : Map(U,V)) (A,B) M)) = ?X", 
     "Finite maps-- an example development using global search", 
     "GSearch")

val sopcGoal = 
    (lex.thy,
     "(ALL R S. (R= (Scan S) -->(EX st rs2 ds dst st1. \
     \ ((flat st) @ (fst S)) = (fst R) & \
     \ length (snd S) = length st & \
     \ st ~= [] --> (ALL j.  1 <j & j < (length (snd S))) & \
     \ nth j st ~= [] & st1 = nth j st & rs2 = nth j (snd S) & \
     \ ds = flat (drop j st) &  dst =  (ds @ (fst S)) & \
     \ ((st1, rs2) longestM (dst, snd R)))) & \
     \(ALL r2 s2. (s2, r2) prefixM (fst S, snd R) --> s2 = [])) = ?X",
     "An example development of a lexical scanner.",
     "SplitPost")

val goals = [simpleGoal, globalGoal, sopcGoal]



(* --- Initial Substitutions -------------------------------------------- *)


val GS_Subst =
    [("Ih", "% (U,V) (S,T,M). S Un T = U & S Int T = {} & M:Map(S,V)"),
     ("r0", "% (U,V). ({}, U,{|})"),
     ("Split", "(%(U,V) (S,T,M) (S',T',M').(EX a b. (a:T) & (b:(V::'b set)) & ((S',T',M') = ((S :: 'a set) Un {a},(T:: 'a set) - {a}, M <+ {|a := b|}))))"),
     ("Extract", "% N (S,T,M). T ={} & N = M"),
     ("subspace", "space(% (S,T,M) (S',T',M'). S <= S' & S ~= S')"),
     ("Satisfies", "% N (S,T,M). (ALL x. x : S--> N^x = M^x)")];


val SoPC_Subst =  
    [("f", "Scan"),               
     ("I", "% R S. (EX st rs2 ds dst st1. ((flat st) @ fst S) = (fst R) & \
           \length (snd S) = length st & st ~= [] -->(ALL j. 1 < j & \
           \j < (length (snd S))) & nth j st ~= [] &  st1 = nth j st & \
	   \(rs2 = (nth j (snd S))) & ds = flat (drop j st) & \
           \dst =  (ds @ fst S) & ((st1, rs2) longestM (dst, snd R)))"), 
     ("B","% R S. (ALL r2 s2. (s2, r2) prefixM (fst S, snd R) --> s2 = [])"), 
     ("E", "% R. (fst R, [])"), 
     ("H","% R S. (NextMatch (snd R) S)"), 
     ("steps","% R. Suc(Suc 0)")];


val substs = [(GS_Subst, "GS_Subst"), (SoPC_Subst, "SoPC_Subst")]


end


structure BMExamples =
    
struct


(* --- Initial Goals ----------------------------------------------------- *)

val mssGoal = (BM.thy, "mss xs = ?X", 
	               "An example development in the Bird-Meertens calculus.",
		       "mss Example")

val maxpGoal = (BM.thy, "maxp p xs = ?X", 
	               "An example development in the Bird-Meertens calculus.",
		       "maxp Example")

val sqsumGoal = (BM.thy, "sqsum xs = ?X", 
	               "An example development in the Bird-Meertens calculus.",
		       "sqsum Example")

val goals = [mssGoal, sqsumGoal]
	


(* --- Initial Substitutions --------------------------------- *)

val mssSubst = [("f","natmax"),("a","0"),("g","op +"),("e","0")]
val maxpSubst = [("f","max_f length"),("a","[]"),("g","% x a.x @ (if p a then [a] else [])"),("e","[]")]

val substs = [(mssSubst, "mssSubst")]


end

