(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/tas.sig,v $
 
   TAS Main Module -- signature part.

   This module provides a functor 

      functor TAS (structure thy    : TRAFOTHEORY 
	                 and conf   : TASCONF
	           structure isaWin : ISAWINSIG) : TAS =

   which implements the main functionality of TAS. The signature TAS (below)
   is the external interface of the transformation application system, 
   with external representations for all of TAS' objects. 

   There is a functor TasAPI which wraps up TAS into a string-based API. 
  
   $Date: 1997/08/22 15:07:28 $
   $Revision: 1.3 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

signature TAS_OBJECTS = 
    sig

	(* external representations for trafos *)
	type path= string
	type inst= string list

	type trafo
	val rep    : trafo-> string
 		
	datatype dev_op = 
	    opApplyTrafo   of trafo* inst
	  | opApplyTrafoAt of trafo* inst* path
	  | dischargeThm   of thm* int

	(* external representation of the history *)
	type history    = int* dev_op list

	type trafoSubst = (string* string) list


	(* Here is where we find external objects *)
	val external_dir : string ref

	(* external representation for objects *)
	datatype ext_obj =
	    Dev      of string* string* history* string
	  | TrafoObj of trafo* trafoSubst
	  | Subst    of trafoSubst
	  | Sign     of string
	  | Spec     of string

	type ext_obj_rep = ext_obj* string*TkTypes.Coord

	(* external representation of TAS state: a list of objects, with
	 * names and positions on the screen *)
	type state = ext_obj_rep list

    end

signature TAS =

    (* This is actually TAS_OBJECTS plus the two functions below. I tried
     * to do a signature inclusion here, but the compiler didn't like that. 
     * Bizarre. *)

    sig

	(* include TAS_OBJECTS *)

	(* external representations for trafos *)
	type path= string
	type inst= string list

	type trafo
	val rep    : trafo-> string
 		
	datatype dev_op = 
	    opApplyTrafo   of trafo* inst
	  | opApplyTrafoAt of trafo* inst* path
	  | dischargeThm   of thm* int

	(* external representation of the history *)
	type history    = int* dev_op list

	type trafoSubst = (string* string) list


	(* Here is where we find external objects *)
	val external_dir : string ref

	(* external representation for objects *)
	datatype ext_obj =
	    Dev      of string* string* history* string
	  | TrafoObj of trafo* trafoSubst
	  | Subst    of trafoSubst
	  | Sign     of string
	  | Spec     of string

	type ext_obj_rep = ext_obj* string*TkTypes.Coord

	(* external representation of TAS state: a list of objects, with
	 * names and positions on the screen *)
	type state = ext_obj_rep list

        (* the "main" function *)
        val go : state* (state-> unit)* (unit-> unit)* (unit-> unit)-> unit
 
        val initial_state : unit-> state
    end

