(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/tas_bm.sml,v $
 
   TAS instantiation for the Bird-Meertens calculus of functional programming.

   $Date: 1997/09/19 14:36:53 $
   $Revision: 1.3 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen

  ************************************************************************** *)

(* Put trafo theories in loadpath *)
val trafodir= tasdir^"/../../isa/isa_toolkit/trafos";
val morethys= tasdir^"/../../isa/isa_toolkit/thy";

(*  Where to find examples: *)
val examples= tasdir^"/examples/BM";

let
    fun addel(e, l) = if exists (fn ee=>ee= e) l then l else e:: l
in                  
    loadpath := addel(examples, addel(trafodir, addel(morethys, (!loadpath))))
end;

(* Load top theory *)
use_thy "BM0";


(* Load configuration *)
use "config.sml";

(* Load trafos and topthy structure *)
use "trafos/std_thy.sml";
use "trafos/un_fold.sml";
use "trafos/bm_trafos.sml";

(* --- Instantiate TAS ---------------------------------------------------- *)
structure TasBM=
    TasAPI(structure TAS= TAS(structure trafothy= BmTrafoThy
			      structure conf=   TasConfNormal
						     (val examples_dir=
							  "examples/BM")
			      structure isaWin= IsaWinHOL(IsaConfNormal)  
			     ));

open TasBM

(* --- Example States ----------------------------------------------------- *)


val Ex1=

[(Dev("BM0","sqsum xs = ?x\n\n\n",(2,[opApplyTrafo(BmTrafoThy.sqsum_def_trafo,[]),opApplyTrafo(BmTrafoThy.foldr_vs_foldl_trafo,[])]),""),"SsqSum dev.",(10,80)),
 (TrafoObj(BmTrafoThy.foldr_vs_foldl_trafo,[]),"Foldr to foldl",(246,98)),
 (TrafoObj(BmTrafoThy.sqsum_def_trafo,[]),"SqSum definition",(245,34)),
 (Spec("sqsum xs = ?x\n\n\n"),"SsqSum dev.",(80,10)),
 (Sign("BM0"),"BM0",(10,10))];

val Ex2= 

[(Dev("Segs","mss xs= ?x\n\n",(2,[opApplyTrafo(BmTrafoThy.mss_def_trafo,[]),opApplyTrafoAt(BmTrafoThy.segs_def_trafo,[],"220"),opApplyTrafoAt(BmTrafoThy.map_prom_trafo,[],"20")]),""),"*new spec*",(10,80)),
 (TrafoObj(BmTrafoThy.map_prom_trafo,[]),"Map promotion",(224,220)), 
 (TrafoObj(BmTrafoThy.segs_def_trafo,[]),"Segs definition",(14,264)), 
 (TrafoObj(BmTrafoThy.mss_def_trafo,[]),"MSS definition",(12,200)), 
 (TrafoObj(BmTrafoThy.map_compose_trafo,[]),"Map Composition",(155,218)), 
 (TrafoObj(BmTrafoThy.fold_prom_trafo,[]),"Fold promotion",(81,219)), 
 (TrafoObj(BmTrafoThy.horner_rule_trafo,[]),"Horner rule",(358,150)), 
 (TrafoObj(BmTrafoThy.accu_lemma_trafo,[]),"Accumulation lemma",(288,149)), 
 (TrafoObj(BmTrafoThy.sqsum_def_trafo,[]),"SqSum definition",(222,148)), 
 (TrafoObj(BmTrafoThy.fold_map_trafo,[]),"Fold/map",(153,148)), 
 (TrafoObj(BmTrafoThy.foldr_vs_foldl_trafo,[]),"Foldr to foldl",(83,149)), 
 (Spec("mss xs= ?x\n\n"),"MSS Spec",(80,10)), 
 (Sign("BM0"),"BM0",(10,10))];

(*
val Ex2= 

[(Dev("BM0","sqsum xs=?x\n\n\n",(4,[opApplyTrafo(BmTrafoThy.sqsum_def_trafo,[]),opApplyTrafo(BmTrafoThy.foldr_vs_foldl_trafo,[])]), ""),"SqSum",(9,150)),
 (TrafoObj(BmTrafoThy.foldr_vs_foldl_trafo,[]),"Foldr to foldl",(83,149)),
 (TrafoObj(BmTrafoThy.sqsum_def_trafo,[]),"SqSum definition",(222,148)),
 (Spec("sqsum xs=?x\n\n\n"),"*new spec*",(98,74)),
 (Sign("BM0"),"BM0",(10,10)),
 (TrafoObj(BmTrafoThy.fold_map_trafo,[]),"Fold/map",(153,148)),
 (TrafoObj(BmTrafoThy.accu_lemma_trafo,[]),"Accumulation lemma",(288,149)),
 (TrafoObj(BmTrafoThy.horner_rule_trafo,[]),"Horner rule",(358,150)),
 (TrafoObj(BmTrafoThy.fold_prom_trafo,[]),"Fold promotion",(81,219)),
 (TrafoObj(BmTrafoThy.map_compose_trafo,[]),"Map Composition",(155,218)),
 (TrafoObj(BmTrafoThy.mss_def_trafo,[]),"MSS definition",(12,200)),
 (TrafoObj(BmTrafoThy.segs_def_trafo,[]),"Segs definition",(14,264)),
 (TrafoObj(BmTrafoThy.map_prom_trafo,[]),"Map promotion",(224,220))];
*)

(* --- Message ------------------------------------------------------------- *)

writeln "\n\n\
\TAS for the Bird-Meertens calculus is now setup.\n\
\Use start(); to start TAS with its initial state, and resume <state>; to \n\
\resume execution. You may also want to try resuming the example states\n\
\Ex1 and Ex2.\n";
