(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/trafos/bm_trafos.sml,v $
    
   Transformations for the Bird-Meertens calculus.
   
   $Date: 1997/09/14 18:01:48 $
   $Revision: 1.15 $
   Author: cxl (Last modification by $Author: cxl $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

(* --- The theory ---------------------------------------------------------- *)
 
structure BmBasicThy=

struct

    structure Thy= StdThy

    val topthy= BM0.thy                        

    val mono=  arg_cong
    (* the variable representing the context rule congr, as a cterm     *)
    val context_var = 
	let val _$ (_$ (cv$ _)) = concl_of arg_cong
	in  cterm_of (#sign(rep_thm arg_cong)) cv
	end
    (* no need to prove monotonicity *)
    fun prove_mono _ = no_tac

    (* extensionality by repeated application of ext *)
    fun prove_ext n  = REPEAT (rtac ext n)

    (* transitivity & reflexivity *)
    val trans = [trans]
    val refl  = [refl]

    (* standard goal package and theory reader *)
    fun goal(t, s) = Goals.goal t s
    val use        = use_thy;

    type inst = string list

    type trafo	 = 
	   {params       : string list, 
	    instantiate : (Sign.sg-> inst -> thm) TkTypes.option,
	    sugar       : (int-> tactic) TkTypes.option,
	    core_thm    : thm,
	    name        : string,
	    id          : string,
	    info        : string,
	    param_info  : string list
	   }


end

 
 
(* --- The transformations ----------------------------------------------- *)

structure BmTrafoThy = 

struct

    open BmBasicThy FoldTrafos

    val mss_def_trafo : trafo = 
	{ params= [],
	 core_thm= Segs.mss_def RS meta_eq_to_obj_eq,
	 id= "BmTrafoThy.mss_def_trafo",
	 name = "MSS definition",
	 info= "Definition of the mss function",
	 param_info= [], 
	 instantiate= NONE,
	 sugar= NONE
	}


    val segs_def_trafo : trafo = 
	{ params= [],
	 core_thm= Segs.segs_def RS meta_eq_to_obj_eq,
	 info= "The segs function returns all segments of a list.",
	 id= "BmTrafoThy.segs_def_trafo",
	 name= "Segs definition",
	 param_info= [], 
	 instantiate= NONE,
	 sugar= NONE
	}


    val map_compose_trafo : trafo = 
	{ params= [],
	 core_thm= map_compose RS sym,
	 info= "If the argument of a map function is a map expression, \
	       \then this transformation eliminates the inner map.",
	 id= "BmTrafoThy.map_compose_trafo",
	 name= "Map Composition",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}


    val map_prom_trafo : trafo = 
	{ params= [],
	 core_thm= map_prom,
	 info= "map f distributes over foldl + 0",
	 id= "BmTrafoThy.map_prom_trafo",
	 name= "Map promotion",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}

    val fold_prom_trafo : trafo = 
	{ params= [],
	 core_thm= fold_prom,
	 info= "Promotes the function f into the argument of the foldl f e \
	       \expression",
	 id= "BmTrafoThy.fold_prom_trafo",
	 name= "Fold promotion",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}
   

    val horner_rule_trafo : trafo = 
	{params= ["f", "e", "g", "a"],
	 param_info=["(f, e) form a monoid.", "", "(g,a) form a monoid.", ""], 
	 core_thm= horner_rule,
	 info= 
	 "Let(f,e) and (g, a) be two monoids where f distributes over g, \
	 \then the Horner rule allows to remove a map/fold.",	 
	 id= "BmTrafoThy.horner_rule_trafo",
	 name= "Horner rule",
	 instantiate= NONE,
	 sugar= NONE
	}
      

    val accu_lemma_trafo : trafo = 
	{params= [],
	 core_thm= accu_lemma RS sym,
	 info= "Algorithms for problems about list segments can frequently be \
		\expressed by fold accumulations. On lists of length n, evaluation \
		\of the right-hand side requires O(n) computations involving f, \
		\while the left-hand side requires O(n*n) computations.",
	 id= "BmTrafoThy.accu_lemma_trafo",
	 name= "Accumulation lemma",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}
   val sqsum_def_trafo : trafo = 
	{ params= [],
	 core_thm= SquareSum.sqsum_def RS meta_eq_to_obj_eq,
	 info= "Definition of the sqsum function.",
	 id= "BmTrafoThy.sqsum_def_trafo",
	 name= "SqSum definition",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}

    val fold_map_trafo : trafo = 
	{ params= [],
	 core_thm= foldl_map,
	 info= "If the argument of a foldl function is a map expression, \
               \then map can be eliminated.",
	 id= "BmTrafoThy.fold_map_trafo",
	 name= "Fold/map",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}

    val foldr_vs_foldl_trafo : trafo = 
	{ params= [],
	 core_thm= foldr_vs_foldl,
	 info= "This transformation converts a foldr expression \
		\into an foldl expression since foldl is tail recursiv.",
	 id= "BmTrafoThy.foldr_vs_foldl_trafo",
	 name= "Foldr to foldl",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}
 

    fun all_trafos () = 
	[mss_def_trafo, segs_def_trafo, map_prom_trafo, map_compose_trafo,
	 fold_prom_trafo, horner_rule_trafo, accu_lemma_trafo,
	 sqsum_def_trafo, fold_map_trafo, foldr_vs_foldl_trafo]

end


(* -- cinnae be bothered wi' the rest ---------------------------

    val maxp_def_trafo : trafo = 
	{ params= [],
	 core_thm= maxp_def RS meta_eq_to_obj_eq,
	 info= "Info",
	 id= "BmTrafoThy.maxp_def_trafo",
	 name= "Maxp definition",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}

    val filter_hom_trafo : trafo = 
	{ params= [],
	 core_thm= filter_hom_def RS fun_cong RS (o_apply RSN (2, trans)),
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}

    val filter_fold_trafo : trafo = 
	{ params= [],
	 core_thm= filter_hom_def RS sym,
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}

    val comp_apply_trafo : trafo = 
	{ params= [],
	 core_thm= o_apply RS sym,
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}


    val comp_assoc_trafo : trafo = 
	{ params= [],
	 core_thm= o_assoc,
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}


    val comp_assoc_sym_trafo : trafo = 
	{ params= [],
	 core_thm= o_assoc RS sym,
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}


    val allappl_Nil_trafo : trafo = 
	{ params= [],
	 core_thm= allappl_Nil,
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}



    val allappl_Cons_trafo : trafo = 
	{ params= [],
	 core_thm= allappl_Cons,
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}


    val Id_unfold_trafo : trafo = 
	{ params= [],
	 core_thm= Id_def RS meta_eq_to_obj_eq,
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	}

    val semi_ring_trafo : trafo = 
	{params= [],
	 core_thm= semi,
	 info= "Info",
	 param_info=[], 
	 instantiate= NONE,
	 sugar= NONE
	} 

    -------------------------------------------------------------- *)









