(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/trafos/csp_trafos.sml,v $
    
   Transformations for CSP
   
   $Date: 1997/09/14 18:01:49 $
   $Revision: 1.5 $
   Author: cxl (Last modification by $Author: cxl $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)



(* --- Instantiating the theory ---------------------------------------- *)


structure CspBasicThy = 

struct

    structure Thy = StdThy

    val topthy= CspTrafos.thy

    val mono= prove_goal topthy
	          "[| s << t; !!x y. x << y ==> P x << P y |] ==> P s << P t"
		  (fn prems=> [rtac ((hd prems) RS(hd (tl prems))) 1]);

    (* the variable representing the context rule congr, as a cterm     *)
    val context_var = 
	let val _$ (_$ (cv$ _)) = concl_of mono
	in  cterm_of (sign_of topthy) cv
	end

    val ref_monos = [ref_ord_refl,rec_ord_implies_ref_ord,
		     mono_mprefix_ref,mono_prefix_ref,mono_ndet_ref,
		     mono_det_ref,mono_Ren_ref,mono_hide_set_ref,
		     mono_PaI_ref,mono_Inter_ref]
 	
    fun prove_mono n = (asm_simp_tac (HOL_ss addsimps ref_monos) n)

    fun prove_ext _  = all_tac  (* no extensionality for CSP yet *)

    val trans= [ref_ord_trans]
    val refl=  [ref_ord_refl]

    type inst = string list

    type trafo	 = 
	   {params       : string list, 
	    instantiate : (Sign.sg-> inst -> thm) TkTypes.option,
	    sugar       : (int-> tactic) TkTypes.option,
	    core_thm    : thm,
	    name        : string,
	    id          : string,
	    info        : string,
	    param_info  : string list
	   }

end




(* --- All the transformations --------------------------------------- *)

structure CspTrafoThy =
    
struct

    open CspBasicThy FoldTrafos

    val proc_eq_imp_ord_ref = ((proc_eq_ref_ord_spec RS iffD1) RS conjunct1);

(* --- Proof scripts -------------------------------------------------- *)

    val COPY1_def : trafo =
	{ params = [],
	  core_thm = Copy.COPY1_def RS meta_eq_to_obj_eq RS proc_eq_imp_ord_ref,
	  info = "Unfolding of the definition of the COPY1 process.\n\n",
	  param_info = [],
	  name= "COPY1_def",
	  id= "CspTrafoThy.COPY1_def",
	  instantiate= NONE,
	  sugar= NONE
	  }

    val COPY2_def : trafo =
	{ params = [],
	  core_thm = Copy.COPY2_def RS meta_eq_to_obj_eq RS proc_eq_imp_ord_ref,
	  info = "Unfolding of the definition of the COPY1 process.\n\n",
	  param_info = [],
	  name= "COPY2_def",
	  id= "CspTrafoThy.COPY2_def",
	  instantiate= NONE,
	  sugar= NONE
	  }

    val R1_corethm= prove_goal CspTransfo1.thy 
	 "! m a. (m ~= e & a ~= e & m ~= f & a ~= f) --> \
	 \(/ X. (e -> (f -> X))) <<\
	 \ (((/ X. (e -> (m ->(a -> X)))) [|{m,a}|] \
	 \                             (/ X. (m ->(f ->(a -> X)))))\\{m,a})"
	(fn _ =>[(REPEAT (rtac allI 1)),
		 (rtac impI 1),
		 (rtac R1_pref 1),
		 (ALLGOALS (asm_simp_tac HOL_ss))]);
 
    val R1pref_trafo : trafo = 
	{ params= [ "m", "a" ],
	  core_thm= R1_corethm,
	  info="This transformation introduces a small \"buffer\" state.\n\n",
	  param_info=["Signals event", "Acknowlegment for previous signal"],
	  name= "BufferIntro",
	  id= "CspTrafoThy.R1pref_trafo",
	  instantiate= NONE,
	  sugar= NONE
	  }

    val R5b_trafo : trafo =
	{ params= [],
	  core_thm= R5b RS proc_eq_imp_ord_ref,
	  info= "This trafo allows hiding to be pushed to the outside.",
	  param_info=[""],
	  name= "Hide",
	  id= "CspTrafoThy.R5b_trafo",
	  instantiate= NONE,
	  sugar= NONE
	}
	
    val R6_trafo : trafo = 
	{ params= [],
	  core_thm= R6 RS proc_eq_imp_ord_ref,
	  info= "This trafo distributes synchronization over interleaving.",
	  param_info=[""],
	  name= "Sync_Inter_Dist",
	  id= "CspTrafoThy.R6_trafo",
	  instantiate= NONE,
	  sugar= NONE
	}

    val R3_corethm= prove_goal CspTransfo1.thy 

	"!c1 c2 a1 a2. (c1~=c2 & P1[|{c1,c2}|]SKIP=P1 & P2[|{c1,c2}|]SKIP=P2) \
	\ --> P1 ||| P2 << ((((P1[c1<-a1]) ||| (P2[c2<-a2])) [|{c1,c2}|] \
        \            (/X. ((c1 ->(a1 -> X)) |] (c2 ->(a2 -> X))))) \\ {c1,c2})"
	(fn _ =>[(REPEAT (rtac allI 1)),
		 (rtac impI 1),
		 (rtac R3 1),
		 (ALLGOALS (asm_simp_tac HOL_ss))]);
	  
    val R3_trafo : trafo =
	{ params= [ "c1", "c2", "a1", "a2" ],
	  core_thm= R3_corethm,
	  info= "This trafo introduces multi-channel signal/event-state multiplexer",
	  param_info= ["", "", "", ""],
	  name= "Channel Intro",
	  id= "CspTrafoThy.R3_trafo",
	  instantiate= NONE,
	  sugar= NONE
	}

    

    val Renaming_trafo : trafo =
	{ params = [],
	  param_info = [],
	  core_thm= ref_ord_refl,
	  info= "Propagates renaming througout specification.",
	  name= "Rename",
	  id= "CspTrafoThy.Renaming_trafo",
	  instantiate= NONE,
	  sugar= SOME 
	  (fn n=> (asm_simp_tac(merge_ss(list_ss,Alph_ss) addsimps 
		   [R7,NelemAlphRec RSN(3,NelemAlphRec RS AlphParInt),
		    empty_def RS meta_sym,Int_def,conj_disj_distr1,conj_disj_distr2,
		    renamF_cont,RenRecPref,RenRecLpref,RenFPref,RenFLpref]@
		    Copy.chan.simps addcongs [conj_cong]) n))
	}

    fun all_trafos()= 

	[FoldTrafos.unfold_trafo, FoldTrafos.fold_trafo,
	 COPY1_def, COPY2_def, R1pref_trafo, R5b_trafo,
	 R6_trafo, R3_trafo, Renaming_trafo]

end





