
(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/trafos/un_fold.sml,v $
    
   TAS --The Fold/Unfold transformations.
   
   $Date: 1997/08/02 22:22:56 $
   $Revision: 1.4 $
   Author: cxl (Last modification by $Author: cxl $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


structure FoldTrafos = 

struct

    val foldThm = prove_goal HOL.thy "! s t. s= t--> s= t"
                                    (fn _ => [asm_simp_tac HOL_ss 1])

    val unfoldThm = prove_goal HOL.thy "! s t. s= t--> t= s"
                                    (fn _ => [asm_simp_tac HOL_ss 1])


(* ``conditional fold'' -- hmm *)				    

    val condFoldThm = prove_goal HOL.thy "! p s t.((p--> s= t)& p)--> s= t"
	                            (fn _ => [strip_tac 1, etac conjE 1, 
					      asm_simp_tac HOL_ss 1])

    val condUnfoldThm = prove_goal HOL.thy "! p s t.((p--> s= t)& p)--> t= s"
	                            (fn _ => [strip_tac 1, etac conjE 1, 
					      asm_simp_tac HOL_ss 1])

    val unfold_trafo =
	{ params= [ "s", "t" ],
	 core_thm= unfoldThm,
	 info=
	 "This transformation \"unfolds\" a definition or equation s= t, \
	 \replacing s with t.<p> \
	 \Despite its simplicity, unfold and its dual fold are amongst the \
	 \most powerful and often-used transformations.",
	 param_info=["The left side of the equation",
		     "The right side of the equation"], 
	 name= "Unfold",
	 id= "FoldTrafos.unfold_trafo",
	 instantiate= NONE,
	 sugar= NONE
	}

    val fold_trafo = 
	{ params= [ "s", "t" ],
	 core_thm= foldThm,
	 info=
	 "This transformation \"folds\" a definition or equation s= t, \
	 \replacing t with s.<p> \
	 \Despite its simplicity, fold and its dual unfold are amongst the \
	 \most powerful and often-used transformations.",
	 param_info=["The left side of the equation",
		     "The right side of the equation"], 
	 name= "Fold",
	 id= "FoldTrafos.fold_trafo",
	 instantiate= NONE,
	 sugar= NONE
	}



end



