(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/trafos/z_trafos.sml,v $
    
   Transformations for Z
   
   $Date: 1997/09/15 11:00:31 $
   $Revision: 1.10 $
   Author: cxl (Last modification by $Author: kol $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)



(* --- Instantiating the theory ---------------------------------------- *)


open ZEnv Theory

structure ZBasicThy = 

struct
 
    structure Thy = 
	struct
	    open Basic_zproofenv 

	    type theory= ZEnv.ztheory
		
	    fun use thynm =  (use_zthy thynm; ztheory_of thynm)
	    fun goal(s, t)=   zgoal s t
		
	    val eq            = zeq_thy
	    fun parents_of _  = [ZTrafos.zthy] (* zparents_of *)
	    val theory_of_thm = ztheory_of_thm 
	    val theoryfile_suffix = "zthy"
		
	    fun name_of th = !(hd(#stamps(Sign.rep_sg(#2(zsign_of th)))))

	    type proofstate= zproof
	    val  restore   = ignore o restore_zproof
	    val  save      = save_zproof
	end	

    val topthy= ZTrafos.zthy;

 
    val mono=  arg_cong
    (* the variable representing the context rule congr, as a cterm     *)
    val context_var = 
        let val _$ (_$ (cv$ _)) = concl_of arg_cong
        in  cterm_of (#sign(rep_thm arg_cong)) cv
        end
    (* no need to prove monotonicity *)
    fun prove_mono _ = no_tac

    (* extensionality by repeated application of ext *)
    fun prove_ext n  = REPEAT (rtac ext n)
 
    val trans= [trans]
    val refl=  [refl]

    type inst = string list

    type trafo	 = 
	   {params       : string list, 
	    instantiate : (Sign.sg-> inst -> thm) TkTypes.option,
	    sugar       : (int-> tactic) TkTypes.option,
	    core_thm    : thm,
	    name        : string,
	    id          : string,
	    info        : string,
	    param_info  : string list
	   }

end




(* --- All the transformations --------------------------------------- *)

structure ZTrafoThy =
    
struct

    open ZBasicThy FoldTrafos


    fun FoldSchemaBinder_tac thm = 
    simp_tac (prod_ss addsimps (thm :: [SB_def, SB0_def]));

    fun FoldSchemaDECL_tac thm = 
    simp_tac (prod_ss addsimps (thm :: [DECL_def]));

    fun FoldSchema_tac thm = 
    simp_tac (prod_ss addsimps (thm :: [SB_def, SB0_def,DECL_def]));


   (*** This should later on be parameterised with the nummer of the schemas and a clever choice of names ***)
   
   val [AddBirthday,AllReadyKnown, AllReadyTest,BirthdayBook1,
     BirthdayBook,FindBirthday, 
     RAddBirthday,Remind,Success] = (map snd BB.schemas);

(*
    val domain = zgoal BB.zthy "(AddBirthday (b, b', d, k, k', n)) --> (k ~= k')";

    val simpleThm =  prove_goal 
	HOL.thy "!f.(! x y. f x y= f y x) --> (f a b)= (f b a)"
	        (fn _ => [asm_simp_tac HOL_ss 1])
*)
		
    val FoldSB0_in_SchemaDef: trafo =
	{ params= [ ],
	  core_thm= SB0_def,
	  info="Folding the Definition of the Schema binder (SB0_def) into \n"^ 	       "the Definition of a Z Schema\n\n",
	  param_info=["The Schema Definition"], 
	  name= "SB0inSchemaDef",
	  id= "ZTrafoThy.FoldSB0_in_SchemaDef",
	  instantiate= NONE,
	  sugar= SOME (simp_tac (prod_ss addsimps [AddBirthday,SB0_def]))
	  }

   val FoldSB_in_SchemaDef: trafo =
	{ params= [  ],
	  core_thm= SB_def,
	  info="Folding the Definition of the Schema binder (SB_def) into \n"^ 	       "the Definition of a Z Schema\n\n",
	  param_info=["The Schema Definition"], 
	  name= "SBinSchemaDef",
	  id= "ZTrafoThy.FoldSB_in_SchemaDef",
	  instantiate= NONE,
	  sugar= SOME (simp_tac (prod_ss addsimps [AddBirthday,SB_def]))
	  }
  
   val FoldBirthdaybook_in_AddBirthday: trafo =
	{ params= [  ],
	  core_thm= BirthdayBook,
	  info="Folding the Definition of the Schema BirthdayBook  into \n"^ 	       "the Definition of the Z Schema AddBirthday\n\n",
	  param_info=["The Schema Definition"], 
	  name= "BirthddayBookinSchemaDef",
	  id= "ZTrafoThy.FoldBirthdaybook_in_SchemaDef",
	  instantiate= NONE,
	  sugar= SOME (simp_tac (prod_ss addsimps [AddBirthday,BirthdayBook]))
	  }

   val FoldDECL_in_AddBirthday: trafo =
	{ params= [  ],
	  core_thm= DECL_def,
	  info="Folding the Definition of the Schema binder (DECL_def) into \n"^ 	       "the Definition of a Z Schema\n\n",
	  param_info=[],
	  name= "DECLinSchemaDef",
	  id= "ZTrafoThy.FoldDECL_in_SchemaDef",
	  instantiate= NONE,
	  sugar= SOME (simp_tac (prod_ss addsimps [AddBirthday,DECL_def]))
	  }

   val FoldDECL_in_SchemaDef: trafo =
	{ params= [  ],
	  core_thm= DECL_def,
	  info="Folding the Definition of the Schema binder (DECL_def) into \n"^ 	       "the Definition of a Z Schema\n\n",
	  param_info=[],
	  name= "DECLinSchemaDef",
	  id= "ZTrafoThy.FoldDECL_in_SchemaDef",
	  instantiate= NONE,
	  sugar= SOME (simp_tac (prod_ss addsimps [DECL_def]))
	  }


   val FoldALL_in_SchemaDef: trafo =
	{ params= [  ],
	  core_thm= DECL_def,
	  info="Folding the Definition of the Schema binder (SB_def, SB0_def)\n"
	  ^"and the Schema Definition Operator (DECL_Def) into \n"^ 	
	  "the Definition of a Z Schema\n\n",
	  param_info=[],
	  name= "DALLinSchemaDef",
	  id= "ZTrafoThy.FoldDECL_in_SchemaDef",
	  instantiate= NONE,
	  sugar= SOME (simp_tac (prod_ss addsimps [AddBirthday,DECL_def,SB0_def, SB_def]))
	  }

   val Symmetry : trafo =
       { params = [],
	 core_thm = symmetry,
	 info = "Symmetry of equality",
	 param_info= [],
	 name = "Symmetry",
	 id   = "ZTrafoThy.Symmetry",
	 instantiate= NONE,
	 sugar= NONE
	 }
	 

(* --- Proof scripts -------------------------------------------------- *)

    fun all_trafos()= 

	[FoldDECL_in_SchemaDef, FoldSB0_in_SchemaDef, FoldSB_in_SchemaDef,	        FoldBirthdaybook_in_AddBirthday, FoldALL_in_SchemaDef, FoldDECL_in_AddBirthday, Symmetry]

end



