(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/widgets.sml,v $
 
   TAS -- Widgets for TAS
   
   $Date: 1997/08/22 15:07:35 $
   $Revision: 1.9 $
   Author: cxl (Last modification by $Author: cxl $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)
 


structure Widgets :
    sig
	(* construction area widgets *)
	val area_widgs  : TkTypes.WinId*        (* master window id *)
	                  bool*                  (* oneWindow *)
	                  TkTypes.WidId*         (* mother widget id *)
	                  string*                (* name of con/area *)
			  TkTypes.SimpleAction*  (* update proof obligations *)
			  TkTypes.SimpleAction*  (* close con/area *)
			  TkTypes.SimpleAction*  (* show history *)
			  TkTypes.SimpleAction*  (* forward *)
			  TkTypes.SimpleAction*  (* backward *)
			  (unit-> int)          (* curr. level *)
			   -> TkTypes.Widget list
	val setLevel    : TkTypes.WidId-> int-> unit
	val setNumPrfOb : TkTypes.WidId-> int-> unit

	(* Widgets id's, relative to the mother widget id passed to area_widgs 
	 *)
        (* the id of the text widget displaying the proof obligations *)
	val prfStateWidId : TkTypes.WidId-> TkTypes.WidId 

	(* the id of the proof obligations text widget *)
	val prfOblWidId   : TkTypes.WidId -> TkTypes.WidId

	(* the id of the window in which this widget can be found 
	 * Careful, above widget only exists if this window exists! *)
	val prfOblWinId : TkTypes.WinId

	(* open proof obligations window. First argument is the
	 * gen_gui window, second the id of the con/area "master
	 * widget", third is the name of the transformational
	 * development, fourth the function to update the proof
	 * obligations 
	 *) 
	val showPrfObligations: TkTypes.WinId* TkTypes.WidId* string*
	                        TkTypes.SimpleAction-> unit

    end =

    struct

	open TkTypes SmlTk

(* --- 1. Proof obligation widget/window ---------------------------------- *)

	val prfOblWinId = "prfObWin000"

	fun prfOblWidId w = subWidId(w, "wPOTxWid")		     

	fun showPrfObligations(win, w, nm, showPrfObls)= 
	    let val title= "Proof obligations: "^nm
		fun widgs win w= 
		  [Label(subWidId(w, "wPOLab"),
			   [Side Left, Side Top, PadX 2, PadY 2],
			   [Text "Proof Obligations:" (* bold/largish *) ],
			   []),	
		   TextWid(prfOblWidId w, RightScb, "", [],
			   [Side Top, PadX 2, PadY 2], 
			   [TextWidReadOnly true], []),
		   Button(subWidId(w, "wPOClose"),
			   [Side Bottom, Side Right],
			   [Text "Close", 
			    Command (fn () => closeWindow prfOblWinId) ], [])]
	    in  if not (occursWin prfOblWinId) then
		   openWindow(prfOblWinId, [WinTitle title, 
					    WinSizeFrom Program,
					    WinGroup win,
					    WinTransient (SOME win)], 
			      widgs prfOblWinId w, showPrfObls)
		else ()
	    end			  			   		  

(* --- 2. The construction area itself ----------------------------------- *)

        (* derived widget id's *)
	fun prfOblButtonWidId w = subWidId(w, "xPO")
	fun levelInfoWidId w    = subWidId(w, "xLev")
	fun prfStateWidId w     = subWidId(w, "prfState")

	fun levelStr l      = Text ("Level: "^(Int.toString l))
	fun setLevel w nu   = addConf (levelInfoWidId w) [levelStr nu]

	fun setNumPrfOb w num = addConf(prfOblButtonWidId w) 
	                          [Text ("Proof Obligations: "^
					 (Int.toString num))]

	fun area_widgs(win, oneWindow, w, nm, prfobs, cc, shw, frw, bck, level)= 
	
	    let    
		val editArea = TextWid(prfStateWidId w, RightScb,
					 "", [], 
					 [Fill Both, Expand true, Side Top], 
					 [Relief Raised, Borderwidth 2,
					  Background Grey,
					  TextWidReadOnly true], [])
		    
		val closeButton = Button(subWidId(w, "wCloseButton"),
					 [PadX 1, PadY 1, Side Right], 
					 [Text "Close", Relief Raised, Borderwidth 1, 
					  Command cc
					  ], [])

		val backButton = Button(subWidId(w, "wBackw"),
					[PadX 1, PadY 1, Side Left],
					[Text "<", Relief Raised, Borderwidth 1,
					 Command bck], [])
		    
		val histButton  = Button(subWidId(w, "whist"),
					 [PadX 1, PadY 1, Side Left],
					 [Text "History", Relief Raised, Borderwidth 1,
					  Command shw], [])

		val forwButton  = Button(subWidId(w, "wForw"),
					 [PadX 1, PadY 1, Side Left],
					 [Text ">", Relief Raised, Borderwidth 1,
					  Command frw], [])

		val buttons = Frame(subWidId(w, "wButFrm"), 
				    [backButton, histButton, forwButton, closeButton], 
				    [PadX 2, PadY 2, Fill X, Side Bottom], [], [])
	    
		val levelInfo = Message(levelInfoWidId w,
					[Side Left, Fill X, PadX 2, PadY 2], 
					[levelStr (level())], [])
	    
		val prfOblButton = Button(prfOblButtonWidId w, 
					  [Side Right, Fill X, PadX 2, PadY 2],
					  [Text "Proof Obligations",
					   Command (fn ()=> 
						     showPrfObligations(win, w, nm, prfobs)
						   )], [])

		val info=Frame(subWidId(w, "wInfoFrm"),
			       [levelInfo, prfOblButton],
			       [Side Top, Fill X], [], [])

		val wsTitle= Label(subWidId(w, "wTitle"), 
				   [Side Top, Fill X, PadY 2], 
				   [Text nm, Relief Groove, Borderwidth 5],
				   [])

		val allwidgets = [info, buttons, editArea]

	    in  if oneWindow then wsTitle::allwidgets else allwidgets
	    end
					 
    end

