(* ***********************************************************************

   Project: sml/Tk: an Tk Toolkit for sml
   Author: Stefan Westmeier, University of Bremen
  $Date: 1996/09/30 19:07:13 $
  $Revision: 1.3 $
   Purpose of this file: Functions related to Text Widget Annotations

   *********************************************************************** *)

signature ANNOTATION =
    sig
	open BasicTypes

	exception ANNOTATION of string

	type widgetPackFun   (* = bool -> TclPath -> IntPath -> Widget -> unit             *)
	type widgetAddFun    (* = Widget list -> Widget -> WidPath -> Widget list          *)
	type widgetDelFun    (* = Widget list -> WidId  -> WidPath -> Widget list          *)
	type widgetUpdFun    (* = Widget list -> WidId  -> WidPath -> Widget-> Widget list *)

	type widgetAddFunc   (* = WinId -> WidPath -> Widget -> unit                       *)
	type widgetDelFunc   (* = WidId -> unit                                            *)


	val selTextWidWidId       : Widget -> WidId
	val selTextWidScrollType  : Widget -> ScrollType
	val selTextWidText        : Widget -> string
	val selTextWidAnnotations : Widget -> Annotation list
	val selTextWidPack        : Widget -> Pack list
	val selTextWidConfigure   : Widget -> Configure list
	val selTextWidBinding     : Widget -> Binding list

(*	val updTextWidWidId       : Widget -> WidId           -> Widget *)
	val updTextWidScrollType  : Widget -> ScrollType      -> Widget
(*	val updTextWidText        : Widget -> string          -> Widget *)
	val updTextWidAnnotations : Widget -> Annotation list -> Widget
	val updTextWidPack        : Widget -> Pack list       -> Widget
	val updTextWidConfigure   : Widget -> Configure list  -> Widget
	val updTextWidBinding     : Widget -> Binding list    -> Widget

	val getTextWidWidgets                 : Widget -> Widget list
	val getTextWidAnnotationWidgetAssList : Widget -> (Annotation * Widget list) list
	val addTextWidWidget                  : (widgetAddFun) -> 
	                                        Widget -> Widget -> WidPath -> Widget
	val deleteTextWidWidget               : (widgetDelFun) -> 
	                                        Widget -> WidId  -> WidPath -> Widget
	val updTextWidWidget                  : (widgetUpdFun) -> 
	                                        Widget -> WidId  -> WidPath -> Widget -> Widget


	val selAnnotationType            : Annotation -> AnnotationType
	val selAnnotationId              : Annotation -> AnnId
	val selAnnotationConfigure       : Annotation -> Configure list
	val selAnnotationBinding         : Annotation -> Binding list
	val selAnnotationMarks           : Annotation -> (Mark * Mark) list
	val selAnnotationWidId           : Annotation -> WidId
	val selAnnotationWidgets         : Annotation -> Widget list
	val selAnnotationWidgetConfigure : Annotation -> Configure list


	val updAnnotationConfigure : Annotation -> Configure list -> Annotation
	val updAnnotationBinding   : Annotation -> Binding list   -> Annotation
	val updAnnotationWidgets   : Annotation -> Widget list    -> Annotation
(*
	val updItemWidgetConfigure : CItem -> Configure list -> CItem
	val updItemItems           : CItem -> CItemId list   -> CItem
	val updItemIcon            : CItem -> IconKind       -> CItem
*)


	val get        : Widget -> AnnId -> Annotation
	val getBinding : Widget -> AnnId -> Key -> Action

	val upd        : Widget -> AnnId -> Annotation -> Widget

	val add        : widgetPackFun -> 
	                 Widget -> Annotation -> Widget
	val delete     : widgetDelFunc -> 
	                 Widget -> AnnId -> Widget


	val addAnnotationConfigure : Widget -> AnnId -> Configure list -> Widget
	val addAnnotationBinding   : Widget -> AnnId -> Binding   list -> Widget


        val pack : widgetPackFun -> TclPath -> IntPath -> Annotation -> unit


        val newId   : unit -> CItemId
        val newFrId : unit -> WidId


	val readSelection : Widget -> (Mark * Mark) list

	val readMarks : Widget -> AnnId -> (Mark * Mark) list

    end
