(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/basic_util.sml,v $
 
   Basic Library
  
   Interface to routines of original Gofer-standard-Prelude
 
   $Date: 1997/05/26 15:09:17 $
   $Revision: 1.17 $
   Author: bu et al. (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


structure BasicUtil : BASIC_UTIL = 
struct

infix 1 ELEM;
infix 1 /=;
infix 1 --;
nonfix prefix;

(* *********************************************************************** *)
(*									   *)
(* IMPLEMENTATION: BASIC UTILITIES		 			   *)
(*									   *)
(* *********************************************************************** *)

(* Interface to original basic library (so similarity to a very rudimentary  
   Gofer standard.prelude is not completely coincidential) *)

exception GET

(* prefix : ''a list -> ''a list -> bool *)
fun   prefix [] ys = true
    | prefix (x::xs) [] = false
    | prefix (x::xs) (y::ys) = (x=y andalso prefix xs ys);

fun   isPrefix s1 s2 = prefix (explode s1) (explode s2)

(* get :: ('a -> bool) -> 'a list -> 'a *)
fun   get p []      = raise GET 
    | get p (x::xs) = if p x then x else get p xs

fun   getx p [] ex      = raise ex
    | getx p (x::xs) ex = if p x then x else getx p xs ex


(* delete :: (a -> bool) -> 'a list -> 'a list *)
fun   delete p [] = []
    | delete p (x::xs) = if p x then delete p xs else x :: delete p xs;

(* deleteEq :: Eq a => a -> 'a list -> 'a list *)
fun deleteEq x = delete (fn y => x=y);

(* updateFun :: (a -> bool) -> (a -> a) -> 'a list -> 'a list *)
fun   updateFun p f [] = []
    | updateFun p f (x::xs) =
           (if p x then f x else x) :: updateFun p f xs;

(* updateVal :: (a -> bool) -> a -> 'a list -> 'a list *)
fun   updateVal p y [] = []
    | updateVal p y (x::xs) = (if p x then y else x) :: updateVal p y xs;


fun   a ELEM ([]) = false
    | a ELEM (b::rs) = (a=b orelse a ELEM rs);


(* Adoptions to the gofer - library     *)

(* dropWhile           :: (a -> Bool) -> [a] -> [a] *)
fun dropWhile p []       = []
   |dropWhile p (xs as(x::xs2)) = if p x  then dropWhile p xs2 else xs;
 


(* span, break         :: (a -> Bool) -> [a] -> ([a],[a]) *)
fun   span p = (fn []   => ([],[])
                | x::xs => (if p x  then 
			let val (ys,zs) = span p xs; in (x::ys, zs) end
         	      else ([],x::xs)));


fun   break p              = span ((op not) o p);

fun takeWhile p l = 
    let val (l',_) = span p l
    in l' end

fun   fst (a,_) = a;
fun   snd (_,b) = b;

fun filter p = (fn [] => [] 
		|  a :: s => if p a then a :: filter p s else filter p s);

fun a /= b = not(a = b);

fun  forall p [] = true
   | forall p (a::s) = p a andalso forall p s;


fun foldr f (l, e) =
  let fun itr [] = e
        | itr (a::l) = f(a, itr l)
  in  itr l  end;

fun foldl f z []      = z
   |foldl f z (x::xs) = foldl f (f z x) xs;

fun foldl1 f (x::xs) = foldl f x xs

fun concatL s1 = foldr (op @)(s1,[])
val flatten    = concatL (* who thought of _that_ name ? *)

fun concatWith s []     = ""
  | concatWith s [t]    = t
  | concatWith s (t::l) = t^s^(concatWith s l)

fun concat s = foldr (op ^) (s, "");

fun app f [] = ()
   |app f (a::S) = (f a; app f S);

fun sort (less: 'a*'a -> bool) =
  let fun insert (x, []) = [x]
        | insert (x, y::ys) =
              if less(y, x) then y :: insert (x, ys) else x::y::ys;
      fun sort1 [] = []
        | sort1 (x::xs) = insert (x, sort1 xs)
  in  sort1  end;


fun wordsS s    = case dropWhile SysDep.isSpace s of
                     [] => []
                  |  s1 => let val (w,s2) = break SysDep.isSpace s1
                           in (implode w) :: (wordsS s2) end;

fun words s    = wordsS (explode s); 

fun zip ([],ys) = []
  | zip (xs,[]) = []
  | zip (x::xs,y::ys) = (x,y)::zip(xs,ys)


fun i -- j = if (i:int) <= j then i::((i+1)--j) else []


val atoi   = SysDep.numval o (takeWhile SysDep.isDigit) o explode

(* ----------- Combinators ------------------------------- *)

(* standard combinators *)

fun C f x y = f y x

fun K x y   = x

fun S f g x = (f x) (g x)

(* derived combinators *)

val I = S K K 

val P       = C o K  (* delay, Postpone, Procrastinate or Put Off evaluation of 
		      * arguments eg. P(output)(std_out, "hello") *)


(* a semaphore to avoid concurrent access to the construction area *)

    structure Sema  = 
	struct
	    val  edsgar = ref false
		
	    fun P () = if (!edsgar) then false
		       else (edsgar := true; true)
			   
	    fun V () = edsgar := false

	end

		 
end;

