(* *********************************************************************** *)
(*									   *)
(* Project: sml/Tk: an Tk Toolkit for sml	 			   *)
(* Author: Burkhart Wolff, University of Bremen	 			   *)
(* Date: 25.7.95				 			   *)
(* Purpose of this file: Functions related to "Tk-Bindings"	 	   *)
(*									   *)
(* *********************************************************************** *)

structure Bind : BIND = 
struct

local open BasicTypes in 

infix 1 bindElem;


fun bindEq (Bind(k1,c1)) (Bind(k2,c2)) = k1 = k2;

fun bindElemH(b,[]) = false
  | bindElemH(b,(x::xs)) = bindEq b x orelse bindElemH(b,xs);

val op bindElem = bindElemH;

fun noDblP [] = true
  | noDblP (x::xs) = not (x bindElem xs) andalso noDblP xs;

(* ***********************************************************************

   selectors on Binding's

   *********************************************************************** *)


fun selKey (Bind(k,c)) = k;

fun selAction (Bind(k,c)) = c;

fun getAction k [] = (fn e => ())
  | getAction k (x::xs) = if selKey x = k then selAction x 
			  else getAction k xs;

(* ***********************************************************************

   defaults for Binding's

   *********************************************************************** *)

(* defaultBindPack : WidgetType -> Key -> string *)
fun defaultBindPack _ _ = "";

(* ***********************************************************************

   updating Binding's

   *********************************************************************** *)


fun addOneBind []      c = [c]
  | addOneBind (x::xs) c = if bindEq x c then c::xs else x::addOneBind xs c;

fun add old new = BasicUtil.concatL (map (addOneBind old) new);


fun deleteOneBind cs c = BasicUtil.filter (not o(bindEq c)) cs;

fun delete old new = map selKey (BasicUtil.foldl deleteOneBind old new);


(* ***********************************************************************

   Binding's  ==>  Tcl

   *********************************************************************** *)

(* packOneWidgetBind : TclPath -> IntPath -> Binding -> string *)
fun packOneWidgetBind tp (w, p) (Bind(k,com)) = 
    "bind " ^ tp ^ " " ^ k ^ " {" ^ Com.commToTcl ^ " \"WBinding " ^ w ^ 
    " " ^ p ^ " " ^ k ^ " " ^ TkEvent.show() ^ " \"}"

(* packBind : TclPath -> IntPath -> Binding list -> string *)
fun packWidget tp ip bs = 
    map (packOneWidgetBind tp ip) bs


(* packOneCanvasBind : TclPath -> IntPath -> CItemId -> Binding -> string *)
fun packOneCanvasBind tp (w, p) cid (Bind(k,com)) = 
    tp ^ " bind " ^ cid ^ " " ^ k ^ " {" ^ Com.commToTcl ^ 
    " \"CBinding " ^ w ^ " " ^ p ^ " " ^ cid ^ " " ^ k ^ 
    " " ^ TkEvent.show() ^ " \"}";

(* packBind : TclPath -> IntPath -> Binding list -> string *)
fun packCanvas tp ip cid bs = 
    map (packOneCanvasBind tp ip cid) bs


(* packOneTagBind : TclPath -> IntPath -> AnnId -> Binding -> string *)
fun packOneTagBind tp (w, p) aid (Bind(k,com)) = 
    tp ^ " tag bind " ^ aid ^ " " ^ k ^ " {" ^ Com.commToTcl ^ 
    " \"TBinding " ^ w ^ " " ^ p ^ " " ^ aid ^ " " ^ k ^ 
    " " ^ TkEvent.show() ^ " \"}"

(* packBind : TclPath -> IntPath -> Binding list -> string *)
fun packTag tp ip tn bs = 
    map (packOneTagBind tp ip tn) bs


(* unpackOneBind : TclPath -> WidgetType -> Key -> string *)
fun unpackOneWidgetBind tp wt k = 
    "bind " ^ tp ^ " " ^ k ^ " {" ^ defaultBindPack wt k ^ "}";

(* unpackBind : TclPath -> WidgetType -> Key list -> string *)
fun unpackWidget tp wt ks = 
    map (unpackOneWidgetBind tp wt) ks


end;

end;

