(* ***********************************************************************

   Project: sml/Tk: an Tk Toolkit for sml
   Author: Stefan Westmeier, University of Bremen
  $Date: 1996/09/25 11:06:02 $
  $Revision: 1.3 $
   Purpose of this file: Functions related to Canvas Items

   *********************************************************************** *)

signature C_ITEM =
    sig
	open BasicTypes

	exception CITEM of string

	type widgetPackFun   (* = bool -> TclPath -> IntPath -> Widget -> unit             *)
	type widgetAddFun    (* = Widget list -> Widget -> WidPath -> Widget list          *)
	type widgetDelFun    (* = Widget list -> WidId  -> WidPath -> Widget list          *)
	type widgetUpdFun    (* = Widget list -> WidId  -> WidPath -> Widget-> Widget list *)

	type widgetAddFunc   (* = WinId -> WidPath -> Widget -> unit                       *)
	type widgetDelFunc   (* = WidId -> unit                                            *)


	val selCanvasWidId      : Widget -> WidId
	val selCanvasScrollType : Widget -> ScrollType
	val selCanvasItems      : Widget -> CItem list
	val selCanvasPack       : Widget -> Pack list
	val selCanvasConfigure  : Widget -> Configure list
	val selCanvasBinding    : Widget -> Binding list

	val updCanvasWidId      : Widget -> WidId          -> Widget
	val updCanvasScrollType : Widget -> ScrollType     -> Widget
	val updCanvasItems      : Widget -> CItem list     -> Widget
	val updCanvasPack       : Widget -> Pack list      -> Widget
	val updCanvasConfigure  : Widget -> Configure list -> Widget
	val updCanvasBinding    : Widget -> Binding list   -> Widget

	val getCanvasWidgets            : Widget -> Widget list
	val getCanvasCItemWidgetAssList : Widget -> (CItem * Widget list) list
	val addCanvasWidget             : (widgetAddFun) -> 
	                                  Widget -> Widget -> WidPath -> Widget
	val deleteCanvasWidget          : (widgetDelFun) -> 
	                                  Widget -> WidId  -> WidPath -> Widget
	val updCanvasWidget             : (widgetUpdFun) -> 
	                                  Widget -> WidId  -> WidPath -> Widget -> Widget


	val selItemType            : CItem -> CItemType
	val selItemId              : CItem -> CItemId
	val selItemConfigure       : CItem -> Configure list
	val selItemBinding         : CItem -> Binding list
	val selItemCoords          : CItem -> Coord list
	val selItemWidgets         : CItem -> Widget list
	val selItemWidgetConfigure : CItem -> Configure list
	val selItemWidId           : CItem -> WidId
	val selItemItems           : CItem -> CItemId list
	val selItemIcon            : CItem -> IconKind

	val updItemConfigure       : CItem -> Configure list -> CItem
	val updItemBinding         : CItem -> Binding list   -> CItem
	val updItemCoords          : CItem -> Coord list     -> CItem
	val updItemWidgets         : CItem -> Widget list    -> CItem
	val updItemWidgetConfigure : CItem -> Configure list -> CItem
	val updItemItems           : CItem -> CItemId list   -> CItem
	val updItemIcon            : CItem -> IconKind       -> CItem


	val get        : Widget -> CItemId -> CItem
	val getBinding : Widget -> CItemId -> Key -> Action

	val upd        : Widget -> CItemId -> CItem -> Widget

	val add        : widgetPackFun -> 
	                 Widget -> CItem   -> Widget
	val delete     : widgetDelFunc -> 
	                 Widget -> CItemId -> Widget

	val addItemConfigure : Widget -> CItemId -> Configure list -> Widget
	val addItemBinding   : Widget -> CItemId -> Binding list   -> Widget


        val pack : widgetPackFun -> TclPath -> IntPath -> CItem -> unit

        val newId   : unit -> CItemId
        val newFrId : unit -> WidId

	val check : CItem -> bool



	val getCoords  : Widget -> CItemId -> Coord list
	val setCoords  : Widget -> CItemId -> Coord list -> unit

	val getWidth   : Widget -> CItemId -> int
	val getHeight  : Widget -> CItemId -> int

	val getIconWidth  : IconKind -> int
	val getIconHeight : IconKind -> int

	val move : Widget -> CItemId -> Coord -> unit

(*
	lower : ...
	raise : ...
	scale : ...

	(* gibt es in mehreren Ausfhrungen --- eine ist "current" *)
	findCurrent : WidId -> CTag
*)

    end
