(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/coord.sml,v $
 
   Coordinates for sml_tk
  
   $Date: 1997/05/22 10:10:40 $
   $Revision: 1.8 $
   Author: stefan (Last modification by $Author: litfox $)
 
   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

structure Coord : COORD = 
struct

infix 1 ELEM;
infix 1 /=;
nonfix prefix;

local open BasicTypes in


exception COORD of string

fun show col = 
    let
	fun showInt i =
	    if ( i >= 0 ) then (makestring i)
	                  else ("-" ^ (makestring (i * ~1)))
	val sl = map (fn ((x,y):Coord) => (showInt x) ^ " " ^ (showInt y)) col
    in
	BasicUtil.concatWith " " sl
    end

fun read str =
    let
	val _   = Debug.print 5 ("Coord.read: \""^str^"\"")
	val cos = BasicUtil.words str
	fun dezip []        = []
	  | dezip (x::y::l) = 
	    let
		val notIsDot = (op not) o (SysDep.isDot)
		val x' = SysDep.numval(BasicUtil.takeWhile notIsDot (explode(x)) )
		val y' = SysDep.numval(BasicUtil.takeWhile notIsDot (explode(y)) )
	    in
		(x',y')::(dezip l)
	    end
	  | dezip _         = raise COORD "Coord.read: odd number of coordinates"
    in
	(dezip cos) handle Overflow => raise COORD "Coord.read: number conversion error"
    end

fun add (x1:int, y1:int) (x2, y2) = (x1 + x2, y1 + y2)

(* old: fun sub (x1, y1) (x2, y2) = (max(0,x1- x2),max(0, y1-y2)) *)

fun sub (x1:int, y1:int) (x2, y2) = (x1- x2, y1-y2)

(* scalar multiplication *)
fun smult (x1:int, y1:int) x      = (x*x1, y1*x)

(* rectangles *)
  type Rect = Coord * Coord
 
  fun between x (y:int) z = (x <= y) andalso (y<= z)
 
  fun inside  (u: int, v: int) ((x1: int, y1: int), (x2, y2)) = 
      (between x1 u x2) andalso (between y1 v y2)
 
  fun moveRect (p1, p2) p3 = (add p1 p3, add p2 p3)


  fun showRect ((x1:int,y1:int), (x2:int, y2:int)) = 
	"["^(makestring x1)^","^(makestring y1)^";"^
	    (makestring x2)^","^(makestring y2)^"] "

end 

end
