(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/fonts.sig,v $

   Fonts for sml_tk -- signature file.
  
   This module tries to provide a wee bit more abstract approach to
   specifying fonts than as in "-*-bollocks-*-*-37-" X-style font
   description.
 
   $Date: 1997/04/07 10:09:37 $
   $Revision: 1.2 $
   Author: cxl (Last modification by $Author: litfox $)

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


signature FONTS_SIG =
    sig

	datatype FontConfig =
	    Bold | Italic | 
	    Tiny | Small | NormalSize | Large | Huge |
	    Scale of real 


	datatype Font = 
	    XFont of string  
	  | Normalfont of FontConfig list      
	  | Typewriter of FontConfig list 
	  | SansSerif of  FontConfig list
	  | Symbol of     FontConfig list
       (*   could (should?) have more here ... *)


      (* selectors and update functions *)
      val selFontConf : Font -> FontConfig list
      val updFontConf : Font * FontConfig list -> Font
    
      val fontDescr : Font -> string  (* get  X style font description for 
				       * fonts 
				       *)

      val checkFont : string -> bool

      val init : unit -> unit         (* initialize fonts, check if all fonts
				       * exist etc.
				       *)

      val Config :
	  {Normalfont : string ref,
	   Typewriter : string ref,
	   SansSerif  : string ref,
	   Symbol     : string ref,
	   BaseSize   : int    ref,
	   ExactMatch : bool   ref,
	   Resolution : int    ref
	  }

      val InitConfig :
	  {NormalFn'     : (bool * bool -> string) ref ,
	   TypewriterFn' : (bool * bool -> string) ref ,
	   SansSerifFn'  : (bool * bool -> string) ref ,
	   SymbolFn'     : (bool * bool -> string) ref
	  }

      val FinalConfig :
	  {NormalFn     : (bool * bool * int -> string) ref ,
	   TypewriterFn : (bool * bool * int -> string) ref ,
	   SansSerifFn  : (bool * bool * int -> string) ref ,
	   SymbolFn     : (bool * bool * int -> string) ref
	  }

    end
