(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/njml1.sml,v $
 
   Implementation of system-dependend functions for SMLNJ 109.
  
   $Date: 1997/11/14 10:42:54 $
   $Revision: 1.14 $
   Author: stefan (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

structure SysDep : 
    sig include SYS_DEP
	val resetCtrlC : unit-> unit
    end
=
struct

    type 'a option = 'a System.Unsafe.Assembly.option 
                        (* looks bizarre, but that's where it is *)

    val cd     = OS.FileSys.chDir

    fun ls dir = 
	let
	    val dirStr = OS.FileSys.openDir dir;
	    fun loopDir dirLst = 
		let
		    val str = OS.FileSys.readDir dirStr
		in
		    if ( str = "" ) then
			dirLst
		    else
			loopDir dirLst@[str]
		end
	    val dirlst = loopDir []
	    val _ = OS.FileSys.closeDir dirStr
	in
	    dirlst
	end

    val pwd    = OS.FileSys.getDir
    val isDir  = OS.FileSys.isDir
    val execute= Unix.streamsOf o Unix.execute

    fun exec (s,sl) = 
	let
	    val pr = Unix.execute (s,sl)
	    val st = Unix.reap pr
	in
	    case st of
		Posix.Process.W_EXITED => true
	      | _                      => false
	end

    val getEnv = OS.Process.getEnv

    fun isFileRdAndEx pn =
(*	OS.FileSys.access (pn,[OS.FileSys.A_READ,OS.FileSys.A_EXEC]) *)
	OS.FileSys.access (pn,[OS.FileSys.A_READ])

    fun isFileRd pn =
	OS.FileSys.access (pn,[OS.FileSys.A_READ])

    fun isDirRdAndWr pn =
	(OS.FileSys.access (pn,[OS.FileSys.A_READ,OS.FileSys.A_WRITE])) andalso
	(OS.FileSys.isDir pn)
	

    fun openFile e f = 
	open_in f
	handle (OS.SysErr(err, _)) => raise (e ("Can't open file "^f^": "^err))
(*             Io {name,...} => raise (e name)
 *)

    val inc    = fn x => x := (!x + 1);

    val nth     = List.nth

    val isPrint    = Char.isPrint
    val isUpper    = Char.isUpper
    val isLower    = Char.isLower
    val isAlpha    = Char.isAlpha
    val isDigit    = Char.isDigit
    val isAlphanum = fn c => (isAlpha c orelse isDigit c)
    val isSpace    = Char.isSpace

    val isDot        = fn c => #"." = c
    val isComma      = fn c => #"," = c
    val isLinefeed   = fn c => #"\n" = c
    val isOpenParen  = fn c => #"(" = c
    val isCloseParen = fn c => #")" = c

    val numval = fn s =>
	let
	    fun numv (SOME ss) = ss
	      | numv NONE      = 0
	in
	    numv (Int31.fromString (implode s))
	end
    handle Overflow => (output(std_err,"WARNING: caught int conversion overflow\n") ; 0)

    (* adaptstring takes care of double quotes and maybe other 	*)
    (* special characters,					*)
    (* to insure that Tcl/Tk does get the right string.		*)
    (* adaptstring :: string -> string				*)
    fun adaptstringH [] = ""
      | adaptstringH (x::xs) = 
	if List.exists (fn y => x=y) (explode "\"\\$[]{}")
	    then "\\" ^ (str x) ^ adaptstringH xs
        else if x = #"\n" then "\\n" ^ adaptstringH xs
             else (str x) ^ adaptstringH xs;

    val adaptString = fn s => adaptstringH(explode s)

    fun min x y = Int.min(x,y)
    fun max x y = Int.max(x,y)


    val oldIntr = Signals.inqHandler(Signals.sigINT)

    (* reset Ctrl-C handler to a sensible value. This is a patch
     * for 109, and will go away in 109.16/17/18...
     *)
    fun resetCtrlC () = (fn _ => ()) (Signals.setHandler(Signals.sigINT, oldIntr))


    (* from Isabelle --- to be used in Makefiles *)
    fun xSmlTk filename banner =
      let val runtime = List.hd (SMLofNJ.getAllArgs())
	  val exec_file = IO.open_out filename
      in (IO.output (exec_file,
			     String.concat
			     ["#!/bin/sh\n",
			      runtime, " @SMLdebug=/dev/null @SMLload=", filename,
			      ".heap\n"]);
	  (*"@SMLdebug=..." sends GC messages to /dev/null*)
				      
	  IO.close_out exec_file;
	  OS.Process.system ("chmod a+x " ^ filename);
	             (* That function is broken and will disable ^C. 
		      * Here's the fix: *)
	  resetCtrlC();
          exportML (filename^".heap");
          print(banner^"\n")
	 )
	  
      end

    fun setPrintDepth n = (Compiler.Control.Print.printDepth := n div 2;
			   Compiler.Control.Print.printLength := n)


end;


