(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/tests+examples/canvas_ex.sml,v $
 
   An Example for Canvasses.

   This example displays a canvas with three boxes (Rectangle canvas items)
   on it. With mouse button 1, one can ``grab'' a box and move it about 
   whilst holding the mouse button pressed. 

   $Date: 1997/03/24 13:45:01 $
   $Revision: 1.4 $
   Author: cxl (Last modification by $Author: litfox $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


structure CanvasEx : 
    sig val go : unit-> string end  =

struct 

  open TkTypes SmlTk


  (* Some parameters *)

  val boxSize      = (50, 55)

  val firstBoxPos  = (20, 20)
  val secondBoxPos = (80, 20)
  val thirdBoxPos  = (140, 20)


  val offset       = ref (0, 0)


  (* getBoxPos and moveBoxPos are the functions being bound to
   * pressing mouseButton1, and moving the mouse with the button pressed,
   * respectively. Note that they are bound directly to the canvas items, 
   * so we do not need to check which item the mouse is over, but rather
   * bind a closure with the item's id to the corresponding item. 
   *)

  fun getBoxPos wid cid (TkEvent(_,_, x, y, _, _)) =
      let                         
	  val wpos    = readCItemCoords wid cid 
      in 
	  (output (std_out, "Grabbed box " ^ cid ^ "\n");
	   offset     := subCoord (x, y) (hd wpos))
      end
  
  and moveBoxPos wid cid (TkEvent(_,_, x, y, _, _)) =
      let
	  val wpos      = readCItemCoords wid cid      
	  val nu_pos    = subCoord (x, y) (! offset)  
	  val wsize     = subCoord (hd (tl wpos)) (hd wpos)
	  val nu_coords = nu_pos :: (addCoord nu_pos wsize) :: [] 
      in
	  (output(std_out, "Moved box "^ cid^"\n");
	   setCItemCoords wid cid nu_coords)
      end 

  and boxBindings wid boxId = 
      [ Bind("<ButtonPress-1>",  getBoxPos wid boxId),
        Bind("<Button1-Motion>", moveBoxPos wid boxId)
       ]
  and littleBoxes wid =
      let fun oneBox(cid, pos, colour) = 
	  CRectangle(cid, pos, addCoord pos boxSize, 
		     [FillColor colour, Outline Black], boxBindings wid cid)
      in
	  [
	   oneBox(newCItemId(), firstBoxPos, Red),
	   oneBox(newCItemId(), secondBoxPos, Blue),
	   oneBox(newCItemId(), thirdBoxPos, Green)
	  ]
      end

  (* This defines the canvas with the three boxes on it *)

  val yeAuldCanvasse =
      let val canvasId = newWidgetId()
      in Canvas(canvasId, NoneScb,
		littleBoxes canvasId,
		[Side Top, Fill X, Expand true],
		[Height 300, Width 400, Relief Groove, Background Grey],
		[])
      end

 
  fun quitButton win = Button(newWidgetId(),
			      [Side Top, Fill X, Expand true],
			      [Text "Quit", Command (fn ()=> closeWindow win)],
			      [])
      
  val testwin = 
      let val winid = newWinId()
      in  (winid, 
	   [WinTitle "Little Boxes",
	    WinAspect(4,3,4,3),
	    WinMinSize(400,300),
	    WinMaxSize(500,400)], 
	   [yeAuldCanvasse, quitButton winid], fn ()=> ())
      end

  fun go () = startTclExn [ testwin ];

end

