
(*
 *
 *  Project: sml/Tk: an Tk Toolkit for sml
 *  Author: Burkhart Wolff, University of Bremen
 * $Date: 1997/03/21 17:51:39 $
 * $Revision: 1.6 $
 *  Purpose of this file: PopUp example
 *
 *)

structure PopUpEx :
    sig val go : unit -> string end =
struct

open TkTypes SmlTk

(* Warning Window *)
fun warnwin msg yes no = 
    ("warning", 
     [WinTitle "Warning"], 
     [Label("l1",
	    [Fill X, Expand true],
	    [Text msg, Relief Flat, Width 25],
	    []),
      Frame("yn",
	    [Button("yes",
		    [Side Left,  Fill X, Expand true],
		    [Text "YES", Command yes],
		    []),
	     Button("no",
		    [Side Right, Fill X, Expand true],
		    [Text "NO",  Command  no],
		    [])
	     ],
	    [],
	    [],
	    [])
      ],
     fn () => ());

fun warning msg yes no = openWindow (warnwin msg yes no);



(* Enter Window *)


val enterwin = 
    let 
	val inputok  = fn () => (changeTitle (mainWinId()) (readTextAll "e1");
				 closeWindow "entername" )
    in
	("entername", 
	 [WinTitle "Please enter name"], 
	 [Label("l2",[Side Left],[Text "name:"],[]),
	  Entry("e1",[],[Width 20],
		[Bind("<Return>",(fn (_:TkEvent) => inputok()))])
	  ],
	 fn () => ())
    end

val playername= fn () => openWindow enterwin;

(* Main Window *)

val entername= MCommand [Text "Enter name", Command playername];

val menu     = 
    let
	val nogoon   = fn () => closeWindow "warning";
	val yesquit  = fn () => closeWindow (mainWinId()); 
	val yesreset = fn () => ()
	val newgame  = fn() => warning "really reset?" yesreset nogoon
	val quitgame = fn () => warning "really quit?"  yesquit  nogoon
    in
	Frame("menu",
	      [Menubutton(newWidgetId(),true,
			  [MCommand([Text "New",Command newgame]),
			   MSeparator,
			   MCommand([Text "Quit",Command quitgame])
			   ],
			  [Side Left],
			  [Text "File"],
			  []), 
	       Menubutton(newWidgetId(),true,
			  [MCommand([Text "Enter name", Command playername])
			   ],
			  [Side Left],
			  [Text "Special"],
			  []),
	       Menubutton("itemmenu",true,
			  [MCommand([Text "Add",Command playername]),
			   MCommand([Text "Delete",Command (fn () => ())])
			   ],
			  [Side Left],
			  [Text "Item"],
			  [])
	       ],
	      [Fill X],
	      [Relief Raised,Borderwidth 2],[])
    end

fun popup wid = Popup(wid,false,
		      [MCommand([Text "Add",Command (fn () => ())]),
		       MCommand([Text "Delete",Command (fn () => ())]),
		       MSeparator,
		       MCommand([Text "Properties",Command playername])
		       ])

val board    = 
    let
	val pos = ref(0:int,0:int)

	fun grabIt (wid:WidId) (cid:CItemId) (TkEvent(_,_,x,y,_,_)) =
	    pos := (x,y)

	fun moveIt (wid:WidId) (cid:CItemId) (TkEvent(_,_,x,y,_,_)) = 
	    let
		val cit_col  = readCItemCoords wid cid
		val (xP,yP)  = !pos
		val _        = (pos:=(x,y))
		val delta    = (x-xP,y-yP)
	    in
		moveCItem wid cid delta
	    end

	fun popitupM (TkEvent(_,_,_,_,xr,yr)) = 
	    popUpMenu "itemmenu" NONE (xr,yr)
	fun popitupP (TkEvent(_,_,_,_,xr,yr)) = 
	    popUpMenu "p1" (SOME 4) (xr,yr)
	fun popitupD (TkEvent(_,_,_,_,xr,yr)) = 
	    createAndPopUpMenu (popup "p2") (SOME 4) (xr,yr)
	fun popitdownD _ =
	    delWidget "p2"
	fun itbds (wid:WidId) (cid:CItemId) = 
	    [Bind("<ButtonPress-2>",popitupM),
	     Bind("<Shift-ButtonPress-3>",popitupD),
	     Bind("<Shift-ButtonRelease-3>",popitdownD),
	     Bind("<ButtonPress-3>",popitupP),
	     Bind("<ButtonPress-1>",grabIt wid cid),
	     Bind("<Button1-Motion>",moveIt wid cid)
	     ]
    in
	Frame("fr1",
	      [Canvas("c1",LeftScb,
		      [COval("it1",(50,50),(100,100),
			     [FillColor Red],itbds "c1" "it1"),
		       COval("it2",(100,100),(150,150),
			     [FillColor Red],itbds "c1" "it2")
		       ],
		      [],[],[])
	       ],
	      [Side Left, Fill X],
	      [Width 200,Height 200,Relief Raised,Borderwidth 2], [])
    end


val area    = [menu,board,(popup "p1")];

val initwin  = [(mainWinId(), [WinTitle "Popup Example"], area,fn () => ())];


val go = fn () => startTclExn initwin


end;

