(* ***************************************************************************

   $Source: /home/uniform/rep/sml/sml_tk/toolkit/drag_and_drop.sig,v $

   Signatures for drag_and_drop.

   DD_ITEMS is the signature for the drag&drop-items, and 
   DRAG_DROP_SIG is the export signature.
 

   $Date: 1997/07/29 20:46:56 $
   $Revision: 1.10 $
   Authour: cxl (Last modification $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)
 
signature DDITEMS = 
    
sig
    type item
    
    val selItemId    : item-> TkTypes.CItemId

    val selDropZone  : item-> Coord.Rect
 
    val isImmobile   : item-> bool

    val grab     : item-> unit
    val move     : item-> TkTypes.Coord-> unit
    val release  : item-> unit

    val enter    : item -> item list -> bool
    val leave    : item -> unit

    val select   : item -> unit
    val deselect : item-> unit

    val drop  : item -> item list -> bool (* false <==> drop is destructive,
					   *            dropped items vanish
					   *)

(*

  two "semantic" points to take note of:
  - after a drop, a leave is generated for the item which has been dropped on;
  - a leave is generated _only_ if the preceding enter has returned true,
    otherwise we assume the visited item doesn't want to know

*)

   type il= item list
	
   structure CB : sig include CLIPBOARD_W                  
                      sharing type obj = il
		  end

end


signature DRAG_DROP_SIG = 

sig
    type item
    type DDCanvas
	
    exception DragAndDrop of string
    
    (* initialize area *)
    val init   : TkTypes.WidId-> DDCanvas
	
    (* place a new object on d&d canvas *)
    val place  : DDCanvas-> item -> unit

    (* delete an object from the d&d canvas *)
    val delete : DDCanvas-> item -> (unit-> unit)

    (* return all items the dropzone of which is at given point *)
    val overDropZone  : DDCanvas->TkTypes.Coord-> item list

    (* selected items *)
    val selectedItems : unit-> item list

    (* get all items on a d&d canvas *)
    val allItems      : DDCanvas-> item list

end


(*
functor DragAndDrop(DDitems: DDITEMS) : 
    sig
	include DRAG_DROP_SIG
        sharing type item = DDitems.item
    end = ?
 *)






