(* ***************************************************************************

   $Source: /home/uniform/rep/sml/sml_tk/toolkit/drag_and_drop.sml,v $

   A small drag&drop package for sml_tk. 

   It is generic over drag&items as given by signature DDITEMS.
  
   See the documentation for more details. 
   "tests+examples/boxes.sml" contains a small example of how to use this 
   package. 
 

   $Date: 1997/08/04 19:28:26 $
   $Revision: 1.20 $

   Author: cxl (Last modification $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


functor DragAndDrop(DDitems: DDITEMS) : 
    sig
	include DRAG_DROP_SIG
        sharing type item = DDitems.item
    end =
struct 

    open TkTypes SmlTk BasicUtil

    type item= DDitems.item

    type DDCanvas = WidId

    exception DragAndDrop of string


    (* local variables *)


    val dropZones      = ref ([]: (WidId* item* Rect) list)

    fun printDropZones() = ddebug("dropZones "^ 
				  (concatWith ", " 
				   ((map (DDitems.selItemId o #2) 
					       (!dropZones)))))
    and ddebug(str)      = Debug.print 11 ("DD: "^str)
 
    type grabItem  = item* Rect* Coord
    val grabItems  = ref ([]: grabItem list)
    val selItems   = ref ([]: item list)
    val oldPos     = ref (0, 0: int) 
    val canDrop    = ref false
    val grabPos     = ref (0, 0: int)

    datatype enterStatus = Entered of item 
	                 | NothingEntered
			 | LeftCanvas

    val enteredItem    = ref NothingEntered

    (* equality for items *)
    fun eq it1 it2 = ((DDitems.selItemId it1)= DDitems.selItemId it2)

    (* apply function on items on grabItem list *)
    fun appIt f = app (fn (it, _, _)=> f it)

    fun overDropZones (cnv, x, y) =
	map #2 (filter (fn (c0, _, r)=> 
			   (inside (x, y) r) (* andalso (c0= cnv) *))
		(! dropZones))
   (* this could be done in a much more efficient way, eg.
    * use btrees or something (oh aye ;-) 
    *)


    (* get current drop zone (i.e. on-the-canvas coordinates) of item
     * it (taken from !dropZones for efficiency, wow).
     * Note that c_item id's are global, so there's no need to check
     * the widget it (I hope :-) 
     *)
    fun getCurrentDropZone it =
	#3 (getx ((eq it) o #2) (! dropZones) 
	    (DragAndDrop ("DD.internal error in getCurrentDropZone("^
			  (DDitems.selItemId it)^")")))

    (* delete item from drop zone list *)
    fun delDropZone item =
 	dropZones := filter (not o (eq item) o #2) (!dropZones)


    (* not clear wethere we want bindings on *all* items of CTag or
     * just the first one.. currently the former:
     *)
    fun addTagBinding wid id bindings =
    let val item = getCItem wid id
    in  case item of
	CTag(_, ls) => app (fn id=> addTagBinding wid id bindings) ls
      | _           => addCItemBind wid id bindings
    end


    (* "private" version of delCItem, this one deletes 
     * `subitems' of CTag-items (delCItem _doesn't_).
     *)
(* nice but slowish: *)
    fun recDelete wid cid = 
    let val cit = getCItem wid cid	    
    in  (delCItem wid cid;
	 case cit of 
	     CTag(_, subitems) => app (recDelete wid) subitems
	   | _                 => ()
	)
    end
(* bit faster: 
    fun recDelete wid cid =
    let fun recDel widg cid = 
	let val item = CItemTree.get wid cid
	    val nw   = CItem.delete WidgetTree.deleteWidgetGUI widg cid
	    val _    = WidgetTree.updWidgetGUI nw
	in  case item of
	    CTag(_, subitems) => app (recDel widg) subitems
	  | _                 => ()
	end
    in  recDel (WidgetTree.getWidgetGUI wid) cid
    end *)

    (* default cursors *)   

    (* the "hand" appearing when you are ready to grab an item *)
    val enterCursor = Cursor(XCursor("hand1", NONE))

    (* the crosshair for dragging an item *)
    val dragCursor = Cursor(XCursor("fleur", NONE))


    (* the clipboard -- unless we reexport it, this structure is just 
     * a shortcut *)
    structure CB = DDitems.CB 

    (* unfortunately, we cinnae use setCItemCoords with CTag(..) items,
     * so we have to moveCItem the buggers-- hence the following function:
     *)
    fun moveItem dd id where = 
	moveCItem dd id (subCoord where (hd (readCItemCoords dd id)))
   
    fun btwyc dd (item, dz, oldItemPos) =
	(* "back to whence you came",
	 * reinstall a grabbed item and dropZone at original position 
	 *)
	(dropZones := (dd, item, dz):: (!dropZones);
	 moveItem dd (DDitems.selItemId item) oldItemPos;
	 DDitems.release item
	)

    fun moveGrabIt dd off (item, dz, oldItemPos) =
	(* move a grabbed item, plus its drop zone, to a new position.	 
	 *)
	btwyc dd (item, moveRect dz off, addCoord oldItemPos off)
	 

    (* enter/leave are bound to particular canvas items 
     * these _should_ only be called when there's no grab. Unfortunately,
     * the wish seems to generate spurious enter/leave events, so we better 
     * check. Defensive programming an' aw.
     *)
    fun enterItem ddCanvas it _ =
	 if not (DDitems.isImmobile it)
	    andalso (null (!grabItems)) then
	    (addConf ddCanvas [enterCursor];	     
	     enteredItem := Entered it	     
	     (* possibly upWidgetConf ddCanvas [enterCursor] *)
	    )
	 else ()
	
    and leaveItem ddCanvas it _ =
	if null (!grabItems) then
	    (addConf ddCanvas [Cursor(NoCursor)]; 
	     enteredItem := NothingEntered
	    )
	else ()

    (* press/releaseGrabButton are bound to the canvas *)
    and pressSelButton ddCanvas (TkEvent(_, _, x, y, _, _)) =
        case !enteredItem of
	    Entered overIt =>
		if not (List.exists (eq overIt) (!selItems)) then
		    (selItems := overIt:: (!selItems);
		     DDitems.select overIt;
		     ddebug ("selected item: "^(DDitems.selItemId  overIt))
		    )
		else ()
          | NothingEntered =>
	        (app DDitems.deselect (!selItems);
		 selItems := [])
	  | _ => ()
	
    and pressGrabButton ddCanvas (TkEvent(_, _, x, y, _, _)) =
	case !enteredItem of 
	    Entered overIt =>
		let fun where it  =  hd (readCItemCoords ddCanvas 
					  (DDitems.selItemId it))
		    (* lose grabbed item if also selected: *)
		    val items     = (filter (not o (eq overIt)) (!selItems))
		    (* grab a selected item *)
		    fun grabIt it = 
			let val cur_dz = getCurrentDropZone it
			in  (delDropZone it; (it, cur_dz, where it))
			end
	        in  ((* revert map of grabbed items, since they are "the wrong
		      * way around" -- last ones selected first *)
		     grabItems    := rev (map grabIt (overIt :: items));
		     enteredItem  := NothingEntered;
		     oldPos       := (x, y);
		     grabPos      := (x, y);
		     selItems     := [];
		     (* looks like bloody basic eh? *)
		     addConf ddCanvas [dragCursor];
		     appIt DDitems.grab (!grabItems);
		     ddebug ("grabbed items: "^
			     (concatWith " "
			      (map (DDitems.selItemId o #1) (!grabItems))))
		    )
		end
	  | _ => ()

    and grabbedMotion ddCanvas (TkEvent(_, _, x, y, _, _)) =
	if (null (!grabItems)) then ()
	else
	    let val grabIds = map #1 (!grabItems)
		fun mvGrabItem it = DDitems.move it (subCoord (x,y) (!oldPos))
	    in
		(app mvGrabItem grabIds;
		 oldPos := (x, y);
		 case !enteredItem of 
		    Entered it => 
			if inside (x, y) (getCurrentDropZone it)
			then () (* have already entered, are still inside *)
			else (* have left entered item *)
			    (enteredItem := NothingEntered;
			     if !canDrop then DDitems.leave it
			     else ();
			     canDrop := false
			    )
		  | _ => (* Have we entered an item? *)
			let val over= overDropZones (ddCanvas, x, y)
			in case over of
			    oo::_ => (enteredItem := Entered oo;
				      canDrop:= DDitems.enter oo grabIds;
				      ddebug("have entered "^
					     (DDitems.selItemId oo)^
					     ": "^(makestring (!canDrop)))
				     )
			  | []   => ()
			end
		 )
	    end
    and releaseGrabButton ddCanvas (ev as TkEvent(_, _, x, y, _, _)) =
	if (null (!grabItems)) then ()
	else
	    (case !enteredItem of
		 Entered it =>
		     if !canDrop then 
			  ((* first, do the drop operation. *)
			   if DDitems.drop it (map #1 (!grabItems)) then
			      (* non-destructive drop, reinstall item
			       * at original position *)
			       app (btwyc ddCanvas) (!grabItems)
			   else
			       (* destructive, argument items vanish *)
			       appIt ((recDelete ddCanvas) o DDitems.selItemId)
			                                         (!grabItems);
		               (* don't need to delete dropZone *)
			   (* generate leaving event for entered item *)
			   DDitems.leave it)
		      else
			 (* can't drop, reinstall at grab position 
			  * w/ original dropzone *)
			 app (btwyc ddCanvas) (!grabItems)		   
	       | NothingEntered =>
	             (* huvnae entered anything, so we huv tae move the items *)
		      app (moveGrabIt ddCanvas (subCoord (x,y) (!grabPos)))
			    (!grabItems)
	       | LeftCanvas =>
		     (* off the canvas, put items into clipboard 
		      * This is awkward -- we reinstall the item on the 
		      * DDcanvas, complete with dropZone, and have it de-
		      * leted by the callback function of the clipboard.
		      * Thus, only objects appearing elsewhere are de-
		      * leted from the DDcanvas. 
		      *)			  
		     let fun delIt it =
			 (delDropZone it;
			  recDelete ddCanvas (DDitems.selItemId it)
			  (* handle exceptions here ?! *)
			 )
		     in (app (btwyc ddCanvas) (!grabItems);
			 DDitems.CB.put (map #1 (!grabItems)) ev 
			                (fn()=> appIt delIt (!grabItems))
			)
		     end;
	     (* In any case, reset variables and the cursor.
	      *)
	     enteredItem := NothingEntered;
	     grabItems := [];
	     canDrop := false;
	     addConf ddCanvas [Cursor(NoCursor)]
	     )

		
    and ddItemBindings canId item =
	[ Bind("<Enter>", enterItem canId item),
	  Bind("<Leave>", leaveItem canId item)
	]

    and place ddCanvas item = 
	let val cid  = DDitems.selItemId item
	    val wher = readCItemCoords ddCanvas cid
	    val nudz = Coord.moveRect (DDitems.selDropZone item) (hd wher)
			           (* (hd (readCItemCoords ddCanvas cid)) *)
	in
	    (ddebug ("place "^ cid^", dropZone "^ (Coord.showRect nudz));
	     dropZones := (ddCanvas, item, nudz) :: !dropZones;
	     addTagBinding ddCanvas cid (ddItemBindings ddCanvas item) 
	    )
	end


    fun leaveCanvas _ =
	enteredItem := LeftCanvas
		    
    fun enterCanvas _ =
	enteredItem := NothingEntered

    fun canvasBindings canId =
	[ Bind("<ButtonPress-1>",   pressGrabButton canId),
	  Bind("<ButtonRelease-1>", releaseGrabButton canId),
	  Bind("<ButtonPress-2>",   pressSelButton canId),
	  Bind("<Button1-Motion>",  grabbedMotion canId),
	  Bind("<Button1-Leave>",   leaveCanvas),
	  Bind("<Button1-Enter>",   enterCanvas)
	];


    fun init canvasId = 
	(* Raises an exception of passed a widget which isnae a canvas, or 
	 * a canvas wi' items on it. 
	 *)
	case getWidget canvasId
	of Canvas(wid, _, [], _, _, _) =>
	    let val _ = 
	    (addBind wid (canvasBindings wid);
	     dropZones := []; 
             ddebug("init " ^ wid)
	    )
	    in  wid
	    end
	|  Canvas(wid, _, x:: xs, _, _, _) =>
	        raise DragAndDrop "init: called with non-empty canvas."
        |  w => raise DragAndDrop "init: argument not a canvas."
  	       
    fun delete ddCanvas item = 
	(case !enteredItem of
	     Entered it => if eq item it then 
		 (enteredItem := NothingEntered;
		  addConf ddCanvas [Cursor(NoCursor)]
		  )
			   else ()
	   | _ => ();
	 grabItems := filter (not o (eq item) o #1) (!grabItems);
	 selItems  := filter (not o (eq item)) (!selItems);
	 if (null (!grabItems)) 
	     then addConf ddCanvas [Cursor(NoCursor)] else ();
	 delDropZone item;
	 (* Don't delete the CItem just now -- return a closure which 
	  * will do that (in case the using application wants to keep the
	  * the item around. Eg. gen_gui will delete objects appearing in
	  * the construction area, but will keep the image untl the con.area
	  * is closed again.
	  * Have to remove bindings now: *)
	 addTagBinding ddCanvas (DDitems.selItemId item)
	          [Bind("<Enter>", K()), Bind("<Leave>", K())];
	 (* and this will delete the CItem: *)
	 fn ()=> recDelete ddCanvas (DDitems.selItemId item)
	)


    fun overDropZone ddCanvas (x, y) = overDropZones(ddCanvas, x, y)
	
    fun selectedItems ()= (!selItems)  (* plus grabItems ?!? *)

    fun allItems ddCanvas = 
	map #2 (filter (fn (cnv, _, _)=> cnv= ddCanvas) (!dropZones))


(* --- Some implementation notes. ------------------------------- 

 - Although it would undoubtedly be better to export DDCanvas as an
 abstract datatype, this is not possible because GenGUI uses the fact
 that DDCanvas is a widget to install forward references to the
 exported functions from within the argument structure of D&D.

 - More than one d&d canvas: I'm note sure this works the way the 
 d&d module is implemented just now. One might have to do some more checks
 to ensure that grabbing/entering etc. does only effect items on the same
 canvas. (All of this would be piss-easy if SML had dynamic modules. Oh well.) 

 ---cxl. 

 *)
 
	
end


