(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/toolkit/enter_windows.sml,v $
 
   Windows to enter substitutions or related data structures (like
   substitutions).
 
   $Date: 1997/03/24 19:09:02 $
   $Revision: 1.13 $
   Author: bu/kol/cxl (Last modification by $Author: cxl $)

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen

  ************************************************************************** *)
 


signature ENTER_UTILWIN =
sig 

    (* a substitution is a list [(p_i,str_i)] of pairs of strings,
     * where p_i is the "parameter names" and str_i its value. 
     * In the following, newSubst creates a new list of substitutions,
     * ie. str_i are (initially) empty, whereas subst takes an existing
     * substitution. 
     * The first argument is the title of the window.
     *)

    val newSubst  : string* string list* ((string * string) list -> unit) ->
	                 unit
    val subst     : string* (string * string) list* 
	                 ((string * string) list -> unit) -> unit



end 


functor EnterWindows(conf: UTILWIN_CONFIG) : ENTER_UTILWIN

= 


struct

    open TkTypes SmlTk 

    fun doSubst(width, sep, wintitle, subst, cc) =

	let (* Width of variable entry boxes: max. length of a var. name +2 *)
	    val varWidth = (foldr
		            (fn ((a, _), m) => if (size a)> m then (size a)
					      else m) 0 subst)+ 2

		(* some widget ids *) 
	    fun lhsWidId(w, n) = subWidId(w, "substLhs"^Int.toString n)
	    fun rhsWidId(w, n) = subWidId(w, "substRhs"^Int.toString n)
	    fun substFrmId w   = subWidId(w, "substFrm")
	    fun addButtonId w  = subWidId(w, "substBttn")
	    fun clsButtonId w  = subWidId(w, "substCls")

	    fun zipIt s = ListPair.zip (BasicUtil.-- (1, length s), s)
		
	    (* entry box for one substitution *)
	    fun oneEntry w (n, (par, str)) = 
		Frame(newWidgetId(), 
		      [Entry(lhsWidId(w, n),
			     [Side Left],
			     [Width varWidth, Font conf.enterTextFont], []),
		       Label(newWidgetId(), [Side Left], [Text sep], []),
		       Entry(rhsWidId(w, n),
			     [Side Right], 
			     [Width width, Font conf.enterTextFont],[])
		       ], [Side Top], [], [])

	    (* frame with all substitutions. Needs to be one frame so we
	     * can add new subst-entry boxes *)
	    fun allSubsts w substs =
		Frame(substFrmId w, 
		      map (oneEntry w) (zipIt subst),
		      [Side Top], [], [])

	    (* close window, read values, continue with cc *)
	    fun closeSubst(win, wid, n) =
		let fun getSub n = (readTextAll(lhsWidId(wid, n)), 
				    readTextAll(rhsWidId(wid, n)))
    		    val subst = List.filter (fn (p, _)=> not(p=""))
			                    (map getSub (BasicUtil.-- (1, n)))
		in  closeWindow win; 
		    cc subst
		end

	    (* add another entry box. Note reconfiguration of the commands
	     * bound to the add-button and the close-button 
	     *)
	    fun addSubst(win, wid, n) =
		(addConf (addButtonId wid)
		         [Command (fn()=> addSubst(win, wid, n+1))];
		 addConf (clsButtonId wid)
		         [Command (fn()=> closeSubst(win, wid, n+1))];
		 addWidget win (substFrmId wid) (oneEntry wid (n+1, ("", "")))
		)

	    fun substFrame(win, wid) =
		Frame(newWidgetId(),
		      [allSubsts wid subst, 
		       Button(addButtonId wid, [Side Top, Fill X],
			      [Width varWidth, Text "Add Parameter", Font conf.buttonFont,
			       Command (fn()=> 
					addSubst(win, wid, length subst))],
			      [])],
		      [Side Top, Fill X], [], [])

	    fun buttonFrm(win, wid) =
		Frame(newWidgetId(),
		      [Button(clsButtonId wid,
			      [Side Right], 
			      [Text "OK", Width conf.buttonWidth, 
			       Font conf.buttonFont, 
			       Command (fn()=> closeSubst(win, wid,
							  length subst))],
			      []),
		       Button(newWidgetId(),
			      [Side Left],
			      [Text "Cancel", Width conf.buttonWidth, 
			       Font conf.buttonFont,
			       Command (fn()=> closeWindow win)],
			      [])], [Fill X, Side Bottom], [], [])

	    (* initializiation function *)
	    fun fillSubst wid (n, (p, str)) = 
		(insertTextEnd (lhsWidId(wid, n)) p;
		 insertTextEnd (rhsWidId(wid, n)) str)

	    val win = newWinId()
	    val wid = newWidgetId()	  

	in  
	    openWindow(win,
		       [WinTitle wintitle],
		       [substFrame(win, wid), buttonFrm(win, wid)],
		       fn ()=> app (fillSubst wid) (zipIt subst))
	end



    fun newSubst (title, pars, cc) = 
	doSubst(60, " |-> ", title, map (fn str=> (str, "")) pars, cc)

    fun subst (title, subst, cc) =
	doSubst(60, " |-> ", title, subst, cc)

end
