(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/toolkit/icons.sml,v $
   
   Subsidiary module for GenGUI, comprising functions to handle icons. 

   Icons are three images of the same size, the normal image, a highlighted
   image (displayed to indicate it is ready for something to be dropped on),
   and an outline, displayed while the icon is being dragged. 
 
   $Date: 1996/12/20 15:56:26 $
   $Revision: 1.4 $
   $Author: cxl $

   (C) 1996, Universitaet Bremen
 
  ************************************************************************** *)


signature ICON_SIG =
    sig
	type icon

	(* get an icon. The first is the directory, the second 
	 * the name of the icon. We could allow for more icons here
	 *)
	val getIcon    : string * string -> icon

	(* the undefined icon *)
	val noIcon     : icon

	val selWidth   : icon -> int
	val selHeight  : icon -> int

        val selImage   : icon -> BasicTypes.IconKind
        val selHiLite  : icon -> BasicTypes.IconKind
        val selOutline : icon -> BasicTypes.IconKind

	exception IconErr of string
    end

structure Icons : ICON_SIG = 

struct

    open BasicTypes

    type icon = IconKind * IconKind * IconKind * int * int

    exception IconErr of string
	
    fun selWidth  (_, _, _, x, _)= x    
    fun selHeight (_, _, _, _, x)= x
    fun selImage  (i, _, _, _, _)= i
    fun selHiLite (_, i, _, _, _)= i
    fun selOutline(_, _, i, _, _)= i

    val noIcon = (NoIcon, NoIcon, NoIcon, 0, 0)

    fun getIconData (dir, file) = 
	let val i = SysDep.openFile IconErr (dir^"/data-"^file) 
            val w = BasicUtil.atoi (input_line i)
	    val h = BasicUtil.atoi (input_line i)
	    val _ = close_in i
        in  (w, h)
	end


	
    fun getIcon(dir, file) =
	let val (w, h) = getIconData(dir, file)
	    val im     = FileImage(dir^"/"^file, SmlTk.newWidgetId())
	    val iH     = FileImage(dir^"/hi-"^file, SmlTk.newWidgetId())
	    val iO     = FileImage(dir^"/out-"^file, SmlTk.newWidgetId())
		(* rudimentary check if files exist: *)
	    val _      = close_in(SysDep.openFile IconErr (dir^"/"^file))
	    val _      = close_in(SysDep.openFile IconErr (dir^"/hi-"^file))
	    val _      = close_in(SysDep.openFile IconErr (dir^"/out-"^file))
	in
	   (im, iH, iO, w, h)	  
	end

	    (* this disnae work: readIconWidth/Height need the image to be
	     * displayed, whereas typically you want to know the height and
	     * width of the icon in order to display it correctly. 
	     *)
	    (* val w  = readIconWidth  im *)
	    (* val h  = readconHeight im *)


end



