(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/toolkit/tests+examples/markup_ex.sml,v $
 
   sml_tk Markup Languages: an example.

   $Date: 1997/07/25 09:17:28 $
   $Revision: 1.5 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


structure MarkupEx : sig val go : unit-> unit end =

    struct

    open TkTypes SmlTk

    structure ColourTags =
	struct

	    type widgetinfo= WidId
	    
	    exception AnnotationError of string

	    datatype tag = redTag | blueTag | greenTag | boxTag

	    fun matchingTag "red"   = SOME redTag
	      | matchingTag "blue"  = SOME blueTag
	      | matchingTag "green" = SOME greenTag
	      | matchingTag "box"   = SOME boxTag
	      | matchingTag  _      = NONE


            fun anno col where = TATag(newAnnotationId(), [where],
				       [Relief Raised, Foreground col], [])				  
            fun annotationForTag redTag _ _ marx   = anno Red marx
	      | annotationForTag blueTag _ _ marx  = anno Blue marx 
	      | annotationForTag greenTag _ _ marx = anno Green marx
	      | annotationForTag boxTag _ _ marx   = 
		   TATag(newAnnotationId(), [marx],
				       [Relief Raised, Borderwidth 2], [])

	    fun escape "neq" = SOME "<>"
	      | escape _     = NONE
	   
            fun warning w = Debug.print 19 ("SML Warning: "^w)

	    val error= Fail

	end

    structure Taggit = SmlTkMarkup(ColourTags)

    val sometext= 
	"In this <red>wonderful<\\red> test text,\n"^ 
	"You should see <blue>blue<\\blue> and <green>green<\\green> bits,\n"^
	"never "^"mind the <box>boxed<\\box> ones."^
	"\n\n\n1234<red>5<\\red>6789<blue>0<\\blue>12345.\n"^
	"Here's some special characters: +&<*! "^
	"Rock&amp;roll or what? 1&neq;0 Can you &see; this?\n"^
	"\n\n\n<red>Thank you for <blue>your<\\red> attention.<\\blue>"

    fun textWidget win =
	let val twid      = newWidgetId()
	    val (str, ann)= Taggit.getAnnotation twid sometext
	in  TextWid(twid, NoneScb, str, ann,
		    [Fill X, Side Top], [], [])
	end
		   

    fun quitButton win =
	Button(newWidgetId(),
                   [Side Bottom, Fill X, Expand true],
                   [Relief Ridge, Borderwidth 2,
                    Text "Quit", Command (fn ()=> closeWindow win)], []) 


   val mainWin =
        let val wid = mainWinId();
        in (wid, [WinTitle "Colour Tag Test Window"], 
            [textWidget wid, quitButton wid],
             fn()=> ())
        end
 
    fun go () = output(std_out, SmlTk.startTclExn[mainWin])
                   
end
