(* ***************************************************************************

   $Source: /home/uniform/rep/sml/sml_tk/toolkit/tests+examples/simpleinst.sml,v $

   Test and example program for GenGUI.

   This example only knows one object types: texts. Texts can be concatened
   by dropping one onto the other, or they can be edited in the construction
   area. There is also the possibility to import a text by calling up the
   file browser and dragging one file into the construction system. 

   Use SimpleInst.go() to start. 

   $Date: 1997/08/25 10:45:46 $
   $Revision: 1.30 $
   Author: cxl (Last modification by $Author: cxl $)
 
   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen

  ************************************************************************** *)
 


structure SimpleInstAppl (* : APPL_SIG *) =

struct

    open TkTypes SmlTk 

    (* We have just one object type here, which text objects.
     * They consist of a string (their content), a reference to a string
     * (their name) and some initial coordinates.
     *)

    datatype objtype = text 

    fun obj_type   _   = text
    fun objlist_type _ = SOME text

    datatype object     = textobj of string* string ref
	type objectlist = object list

    type new_object  = object * TkTypes.Coord * TkTypes.AnchorKind

    fun is_tool _    = false
    fun is_primary _ = true

    fun sel_name (textobj(_, nm)) = !nm
     
    fun set_name (textobj(_, nm), nuname) =  nm := nuname

    fun sel_text   (textobj (t,_))= t

    fun icon _ = Icons.getIcon(getSrcPath()^
			       "toolkit/tests+examples/icons", 
			       "note.gif")

 
   (* Configuring GenGUI *)

   structure Conf = 
       struct
	   val width         = 500
	   val height        = 300
	   val wsWidth       = 350
	   val wsHeight      = 300
	   val wsXY          = SOME (50, 470)
	   fun wsTitle nm    = "Edit text: "^nm

	   val iconNameWidth = 100
           val iconNameFont  = TkTypes.SansSerif [TkTypes.Tiny]

	   val background    = Grey

	   val moveOpaque    = true
	       
	   val oneWindow     = false

	   fun trashcanIcon()= Icons.getIcon(getSrcPath()^"toolkit/icons",
	                                        "trashcan.gif")
	   val trashcanCoord = (width-50, (height div 2)- 50)	 
	       

	   val delta         = 70

       end
       

    (* Instantiating the utility windows *)

    structure utilWin = UtilWin(DefaultConfig)

    (* The standard operations *)

    fun show   (textobj(tx, nm)) = 
	utilWin.display (!nm) (Conf.width div 2) NONE tx (fn _ => ()) 

    fun stat   (textobj(tx, nm)) = 
	let fun count p = List.length o (BasicUtil.filter p)
	    val tc = explode tx
	    val nl = count SysDep.isLinefeed tc
	    val nc = List.length tc
	    val nspc = count SysDep.isSpace tc
	    val na = ((count SysDep.isAlpha tc) * 100) div nc
	    val st = "\nNumber of lines : "^(makestring (nl:int))^
	             "\nNumber of chars : "^(makestring (nc:int))^
	       	     "\nNumber of spaces: "^(makestring (nspc:int))^
		     "\nPercentage of alphanumerical char's: "^
		        (makestring (na:int))^"\n"
	in  utilWin.display ("Statistics for "^(!nm))
	                     (Conf.width div 2) NONE st (fn _ => ())
	end


    fun delete (_) = ()


    (* Initially appearing objects. *)

    fun init () =
	[(textobj("Bring me my spear of burning gold,\n"^
		  "Bring me the arrows of desire,\n"^
	 	  "Bring me my bow, O clouds unfold,\n"^
		  "Bring me my chariot of fire\n", 
	          ref "Jerusalem 1"), (100, 10), Center),
	 (textobj("I will not cease from mental fight\n"^
		  "Nor shall my sword rest in my hand\n"^
		  "Till we have built Jerusalem\n"^
		  "In England's green and pleasant land\n",
		  ref "Jerusalem 2"), (100, 10), South)]

    (* There are no custom operations *)     
    fun mon_ops text  = []

    (* There is just one binary operation: concatenating texts *)
    fun tconc (t1, wh) [] = [ (t1, wh, South) ]
      | tconc (t1, wh) t  = [ (textobj(BasicUtil.concatWith "\n" 
				            (map sel_text (t1::t)),
			               ref (BasicUtil.concatWith " and " 
					    (map sel_name (t1::t)))),
			       wh, South) ]

    fun bin_ops (text, text) = SOME tconc



   (* 
    * The Construction Area.
    * 
    * The Construction Area essentially consists of a text widget
    * which can be used to edit the text. If another text is dragged
    * down from the manipulation area, it will appended at the end.
    *)

    fun  txId wsId = wsId^"xTxEd"
	
    type ca = WidId
	   
    fun area_ops _ wid ls = 
	let fun concatCR t = 
	    BasicUtil.foldr (fn(str1,str2) => str1^"\n"^str2) (t, "\n")
	in SmlTk.insertTextEnd (txId wid) (concatCR (map sel_text ls)) 
	end
    
	
    fun area_open(win, textobj(tx, nm), cc) = 
	let val wsWId = newWidgetId()
	    val title = Label(wsWId^"xTitle", [Side Top, Fill X], 
			        [Relief Groove, Borderwidth 2,
				 Text (!nm)], [])
	    val txwid = TextWid((txId wsWId), RightScb, tx, [],
				[Fill Both], 
				[], [])
	    fun close txid cc nm () = 
		                cc (textobj (SmlTk.readTextAll txid, nm))
	    val quit  = Button(wsWId^"xQuit", [Side Right, Side Bottom],
			        [Text "Close",
				 Command(close (txId wsWId) cc nm)
				], [])
	    val widgs  = [quit, txwid]			    
	in  (wsWId, if (Conf.oneWindow) then title::widgs
		    else widgs, BasicUtil.K())
	end
    
    val area_init = BasicUtil.K() (* no init necessary *)
	

   (* Communicating with the Filer:
    *
    * First, we need to instantiate the clipboard: 
    *)

   structure CB = Clipboard(type obj= object list)

   (* Instantiate the filer.
    * We need to provide it with a function to convert files to 
    * texts (fileToObj below); we'll do so by reading the file's contents
    * into the text of the object.
    *)
   structure Filer = 
       ClipFiler(structure M= 
	  struct 
	      exception NoFile of string
	      structure CB= CB
	      fun fileToObj (dir, file) =
		  let val filenm= dir^"/"^file
		      val objnm = ref ("File: "^file)
		      val txt  = let fun read_file si = 
			  if end_of_stream si then ""
			  else (input_line si)^(read_file si)
				     val is  = open_in filenm
				     val txt = read_file is
				     val _   = close_in is
				 in txt
				 end handle NoFile f => "NoFile: "^f
		  in [textobj(txt, objnm)]
		  end	     
	     fun dirToObj dir =
		 [textobj(dir, ref "Directory")]
	  end)

end

structure SimpleInst =
    (* sig val go: unit -> unit end  *)
struct

    open TkTypes SmlTk

    structure utilWin   = SimpleInstAppl.utilWin	
    structure SimpleGUI = GenGUI (structure appl= SimpleInstAppl)

    exception Result of SimpleGUI.gui_state

    fun quitButton win =
	let fun confirmQuit() = utilWin.confirm 
		                "Do you really want to quit?" 
				(fn()=> let val st= SimpleGUI.state();
				        in  (closeWindow win;
					     raise Result st)
					end)
	in  Button(newWidgetId(),
		   [Side Bottom, Fill X, Expand true],
		   [Relief Ridge, Borderwidth 2,
		    Text "Quit", Command confirmQuit], []) 
	end
	
    structure TestMe= EnterWindows(DefaultConfig)

(*
    fun substButton () = 
	let fun prtSubst s = 
	        Debug.print 19 (concat (map (fn (a, b)=> " ("^a^","^b^") ") s))
	in  Button(newWidgetId(),
	       [Side Bottom, Fill X, Expand true],
	       [Relief Ridge, Borderwidth 2,
		Text "Edit a substitution", 
		Command (fn _ => TestMe.subst ("Test substitution",
						  [("a", "Ah?"),
						   ("b", "Beh!"),
						   ("c", "Zeh.")],
			                          prtSubst))],[])
	end
 *)

    fun filerButton win = 
	Button(newWidgetId(),
	       [Side Bottom, Fill X, Expand true],
	       [Relief Ridge, Borderwidth 2,
		Text "Import File", 
		Command (fn _ => SimpleInstAppl.Filer.enter_file 
			         (fn (d, f)=> Debug.print 19 
					      ("Selected file "^d^"/"^f)))],[]) 
 
     
    val mainWin =
	let val wid = mainWinId();
	in (wid, [WinTitle "cxl's Comprehensive Text Manipulation System",
		  WinGeometry (NONE, SOME (50, 50))],
	    [SimpleGUI.main_wid wid, quitButton wid, filerButton wid],
	     fn ()=> SimpleGUI.init (SimpleGUI.initial_state()))
	end

    fun go () =  (SmlTk.startTcl[mainWin]; SimpleGUI.initial_state())
		 handle SimpleGUI.GenGUI err => 
		     (Debug.print 19 ("GenGUI error: "^err);
		      SimpleGUI.initial_state())
		      | Result state => state 
end 







