(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/widget_tree.sig,v $
 
   Functions related to Path-Management.
  
   $Date: 1997/03/13 15:56:42 $
   $Revision: 1.5 $
   Author: Stefan Westmeier (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

signature WIDGET_TREE =
    sig
	open BasicTypes

	(* *********************************************************************** *)
	(* CHECKING the INTEGRITY of WIDGETS                                       *)
	(* *********************************************************************** *)

	val checkWidId      : string -> bool

	val checkOneWidgetConfigure : WidgetType -> Configure      -> bool
	val checkWidgetConfigure    : WidgetType -> Configure list -> bool
	val checkOneWidgetBinding   : WidgetType -> Key          -> bool
	val checkWidgetBinding      : WidgetType -> Binding list -> bool

	val checkOneMConfigure      : WidgetType -> Configure -> bool
	val checkMItem      : MItem -> bool


	val checkWidget     : Widget -> bool

	(* *********************************************************************** *)
	(* SELECTING WIDGETS from the internal GUI state 			   *)
	(* *********************************************************************** *)

	val getWidgetGUI     : WidId   -> Widget
	val getWidgetGUIPath : IntPath -> Widget

	(* *********************************************************************** *)
	(* ADDING WIDGETS to the internal GUI state	 			   *)
	(* *********************************************************************** *)

	val addWidgetPathAssGUI : WinId -> WidPath -> Widget -> unit
	val addWidgetsPathAssGUI : WinId -> WidPath -> Widget list -> unit

	val addWidgetGUI  : WinId -> WidPath -> Widget      -> unit
	val addWidgetsGUI : WinId -> WidPath -> Widget list -> unit

	(* *********************************************************************** *)
	(* DELETING WIDGETS from the internal GUI state 			   *)
	(* *********************************************************************** *)

	val deleteWidgetGUI     : WidId -> unit
	val deleteWidgetGUIPath : IntPath -> unit

	(* *********************************************************************** *)
	(* UPDATING WIDGETS in the internal GUI state		         	   *)
	(* *********************************************************************** *)

	val updWidgetGUI      : Widget -> unit
	val updWidgetGUIPath  : IntPath -> Widget -> unit


	(* *********************************************************************** *)
	(* ADDING WIDGETS to the "real" GUI                                        *)
	(* *********************************************************************** *)

	val packWid0 : bool -> string -> TclPath -> IntPath -> WidId
	    -> Pack list -> Configure list -> string -> Binding list -> unit

	val packWid : bool -> string -> TclPath -> IntPath -> WidId
	    -> Pack list -> Configure list -> Binding list -> unit

	val packTextWid : bool -> TclPath -> IntPath -> WidId  -> ScrollType 
	    -> string -> Annotation list
	    -> Pack list -> Configure list -> Binding list -> unit

	val packListbox : bool -> TclPath -> IntPath -> WidId -> ScrollType
	    -> Pack list -> Configure list -> Binding list -> unit

	val packCanvas : bool -> TclPath -> IntPath -> WidId -> ScrollType -> CItem list
	    -> Pack list -> Configure list -> Binding list -> unit

	val packMenu    : bool -> TclPath -> IntPath -> WidId -> bool -> MItem list
	    -> Pack list -> Configure list -> Binding list -> unit

	val packWidget  : bool -> TclPath -> IntPath -> Widget      -> unit
	val packWidgets : bool -> TclPath -> IntPath -> Widget list -> unit

	val packMenuItem  : TclPath -> IntPath -> WidId -> MItem -> int list -> unit
	val packMenuItems : TclPath -> IntPath -> WidId -> MItem list -> int list -> unit


	(* *********************************************************************** *)
	(* UPDATING WIDGETS in the "real" GUI				           *)
	(* *********************************************************************** *)
(*
	val updConfigurePack  : WidId -> Configure list -> unit
	val updBindingPack    : WidId -> Binding list   -> unit

	val updWidgetPackPath : IntPath -> unit
	val updWidgetPack     : Widget  -> unit
*)
	(* *********************************************************************** *)
	(* EXPORTED FUNCTIONS					         	   *)
	(* *********************************************************************** *)

	val selectWidget     : WidId -> Widget
	val selectWidgetPath : IntPath -> Widget

	val addWidget        : WinId -> WidId -> Widget -> unit
	val deleteWidget     : WidId -> unit
(*
	val updateWidget     : Widget -> unit
*)

	(* *********************************************************************** *)
	(* IMPLEMENTATION: WIDGET CONTENTS		 			   *)
	(* *********************************************************************** *)

	val select              : WidId   -> Configure list
	val selectCommand       : WidId   -> SimpleAction
	val selectCommandPath   : IntPath -> SimpleAction
	val selectMCommandMPath : IntPath -> int list -> SimpleAction
	val selectMCommand      : WidId   -> int list -> SimpleAction
	val selectMCommandPath  : IntPath -> int list -> SimpleAction
	val selectBindings      : WidId -> Binding list
	val selectBindKey       : WidId   -> Key -> Action
	val selectBindKeyPath   : IntPath -> Key -> Action
	val selectWidth         : WidId -> int
	val selectHeight        : WidId -> int
	val selectRelief        : WidId -> RelKind


	val configure        : WidId -> Configure list -> unit
	val newconfigure     : WidId -> Configure list -> unit
	val configureCommand : WidId -> SimpleAction   -> unit
	val addBindings      : WidId -> Binding list   -> unit
	val newBindings      : WidId -> Binding list   -> unit
	val configureWidth   : WidId -> int            -> unit
	val configureRelief  : WidId -> RelKind        -> unit
	val configureText    : WidId -> string         -> unit

	val insertText       : WidId -> string -> Mark -> unit
	val insertTextEnd    : WidId -> string         -> unit
	val deleteText       : WidId -> Mark * Mark    -> unit
	val clearText        : WidId                   -> unit


	val focus   : WinId -> unit
	val deFocus : WinId -> unit

	val grab    : WinId -> unit
	val deGrab  : WinId -> unit

	val popUpMenu : WidId -> (int SysDep.option) -> Coord -> unit
	val createAndPopUpMenu : Widget -> (int SysDep.option) -> Coord -> unit

    end
