(* *********************************************************************** *)
(*									   *)
(* Project: sml/Tk: an Tk Toolkit for sml	 			   *)
(* Author: Burkhart Wolff, University of Bremen	 			   *)
(* Date: 25.7.95				 			   *)
(* Purpose of this file: Abstract data Type Window			   *)
(*									   *)
(* *********************************************************************** *)


structure Window : WINDOW = 
struct

local open BasicUtil in
local open BasicTypes in

infix 1 ELEM;
infix 1 /=;


(* *********************************************************************** *)
(*									   *)
(* IMPLEMENTATION: WINDOWS			 			   *)
(*									   *)
(* *********************************************************************** *)


(* I'm not sure if it could be called before the window is added to   *)
(* the internal GUI state. Therefore True as well if no window is     *) 
(* present as if it really is the first in the GUI state.             *)
(* isInitWin :: WinId -> GUI s -> (bool, GUI s) *)

(* Moved To BASIC_TYPES for visibility reasons
fun   isInitWin w = 
     (fn ([], _) => true | (win::wins, _) => (w = (winId win))) (!GUI_state);
*)


(* CHECKING the INTEGRITY of a WINDOW *)

(* window title may contain alphanumerical characters only *)

fun checkWinId s =
    let
	fun checkWinIdH []      = false
	  | checkWinIdH (x::xs) = SysDep.isLower x andalso forall SysDep.isAlphanum xs
    in
	checkWinIdH (explode s)
    end;

fun checkTitle x = forall SysDep.isPrint (explode x);

fun check (win as (w, wcnfgs, wids,_)) = 
    let
	val mbt = Config.selWinTitle win
	val bb  = checkWinId w
    in
	case mbt of
	    NONE   => bb
	  | SOME t => checkTitle t andalso bb
    end;


fun appendGUI w = updWindowsGUI(getWindowsGUI() @ [w]);

fun addGUI (w as (winId,wcnfgs,widgs,act)) = 
    if check w then
	if Paths.occursWindowGUI (selWindowWinId w) then
	    raise WINDOWS ("Two identical window names not allowed: " ^ 
			   (selWindowWinId w))
	else 
	    let
		val tmpWin = (winId,wcnfgs,[],act)
	    in
		(appendGUI tmpWin;
		 WidgetTree.addWidgetsGUI winId "" widgs)
	    end
    else 
	raise WINDOWS ("Definition of window " ^ selWindowWinId w ^ " is not OK");

fun deleteGUI w  = 
    let
	val wins  = getWindowsGUI()
	val ass   = getPathAssGUI()
	val anws  = getTclAnswersGUI()
	val nwins = BasicUtil.delete ((fn x => w=x) o selWindowWinId) wins
	val nass  = Paths.deleteWindow w ass
    in 
	updGUI(nwins,nass,anws)
    end;

val deleteAllGUI = updGUI([], [], []);


(* 2F. EXPORTED FUNCTIONS *)

fun openW (w as (win, wconfigs, widgets, init_action)) = 
    (addGUI w; 
     if isInitWin win then
	 (app (fn wcnfg => Com.putTclCmd (Config.packWinConf "." wcnfg)) wconfigs;
	  WidgetTree.packWidgets true "" (win, "") widgets)
     else 
	 (Com.putTclCmd ("toplevel ." ^ win); 
	  app (fn wcnfg => Com.putTclCmd (Config.packWinConf ("."^win) wcnfg)) wconfigs;
	  WidgetTree.packWidgets true ("."^win) (win, "") widgets);

	 init_action());

fun close win =
    if isInitWin win then
	(Com.exitTcl();
         deleteAllGUI)
    else 
	(Com.putTclCmd ("destroy ." ^ win);
         deleteGUI win);

fun changeTitle winid title =
    let 
	val win   = getWindowGUI winid
	val wc    = selWindowConfigures win
	val wc'   = Config.addWinConf wc [WinTitle title]
	val win'  = updWindowConfigures win wc'
    in
	if checkTitle title then
	    (updWindowGUI winid win';
	     if isInitWin winid then
		 Com.putTclCmd (Config.packWinConf "." (WinTitle title))
	     else 
		 Com.putTclCmd (Config.packWinConf ("."^winid) (WinTitle title)))
	else 
	    raise WINDOWS ("Title " ^ title ^ " for window " ^ winid ^ " is not OK")
    end;

end; end; 
(* end; *)

end; 


