(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/trafos/hol_trafos.sml,v $
    
   TAS -- HOL Transformations
   
   $Date: 1997/09/14 18:01:50 $
   $Revision: 1.13 $
   Author: cxl (Last modification by $Author: cxl $)
 
   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)



(* --- Instantiating the theory ---------------------------------------- *)


structure HolBasicThy = 

struct


    structure Thy = StdThy

    val topthy= TAS_HOL0.thy

    val mono=  arg_cong
    (* the variable representing the context rule congr, as a cterm     *)
    val context_var = 
	let val _$ (_$ (cv$ _)) = concl_of arg_cong
	in  cterm_of (#sign(rep_thm arg_cong)) cv
	end
    (* no need to prove monotonicity *)
    fun prove_mono _ = no_tac

    (* extensionality by repeated application of ext *)
    fun prove_ext n  = REPEAT (rtac ext n)

    (* transitivity & reflexivity *)
    val trans = [trans]
    val refl  = [refl]

    (* standard goal package and theory reader *)
    fun goal(t, s) = Goals.goal t s
    val use        = use_thy;


    type inst = string list

    type trafo	 = 
	   {params       : string list, 
	    instantiate : (Sign.sg-> inst -> thm) TkTypes.option,
	    sugar       : (int-> tactic) TkTypes.option,
	    core_thm    : thm,
	    name        : string,
	    id          : string,
	    info        : string,
	    param_info  : string list
	   }

end
 

(* --- All the transformations --------------------------------------- *)

structure HolTrafoThy =
    
struct

    open HolBasicThy FoldTrafos

    (* this bit should go into isa/isa_toolkit/trafos/... *)
    val simpleThm =  prove_goal 
	HOL.thy "!f.(! x y. f x y= f y x) --> (f a b)= (f b a)"
	        (fn _ => [asm_simp_tac HOL_ss 1])


    val commute_trafo : trafo = 
	{ params= [ "f" ],
	  core_thm= simpleThm,
	  info="A simple transformation exploiting commutativity\n\n",
	  param_info=["The commutative operation"], 
	  name= "Commute",
	  id= "HolTrafoThy.commute_trafo",
	  instantiate= NONE,
	  sugar= NONE
	  }



    val commute_plus_trafo : trafo = 
	{ params= [],
	  core_thm= add_commute,
	  info="A simple transformation exploiting commutativity\n"^
          "for the plus operator.\n\n",
	  param_info=[], 
	  name= "CommutePlus",
	  id= "HolTrafoThy.commute_plus_trafo",
	  instantiate= NONE,
	  sugar= NONE
	  }


    val add_mult_trafo : trafo = 
	{ params= [],
	  core_thm= add_mult_distrib2,
	  info="A simple transformation exploiting distributing\n"^
          "multiplication over addition.\n\n",
	  param_info=[], 
	  name= "Distribute",
	  id= "HolTrafoThy.add_mult_trafo",
	  instantiate= NONE,
	  sugar= NONE
	  }
      
		
    val commuteAC_trafo: trafo =
	{ params= [ "f" ],
	  core_thm= simpleThm,
	  info="A simple transformation exploiting commutativity,\n"^
	  "with some sugar proving commutativity of arithmetic binary operations.\n\n",
	  param_info=["The commutative operation"], 
	  name= "Commute (AC)",
	  id= "HolTrafoThy.commuteAC_trafo",
	  instantiate= NONE,
	  sugar= SOME (simp_tac (arith_ss addsimps add_ac))
	  }

    val gs_trafo: trafo =
	{ core_thm= global_search_synthm,
	  params = [ "Ih", "r0", "Split", "Extract", "subspace", "Satisfies" ],
	  param_info = [ "Describes the search spaces which can arise from programs",
			"Embedding of the program domain into the search space", 
			"Splits a searchspace into two sub-searchspaces.",
			"Extracts the solutions from the solution space",
			"Defines when a searchspace is a subspace of another",
			"Determines which elemements of the search " ],
	  info= 
	  "The basic idea of global search is to represent and manipulate sets of \
	   \candidate solutions. Global serach works as follows: starting from an \
	   \initial set that contains all solutions to the given problem instance, \
	   \the algorithm repeatedly extracts candidates, splits sets, and eleminates \
	   \sets via filters until no sets remain to be split.",
	  name= "GlobalSearch",
	  id= "HolTrafoThy.gs_trafo",
	  instantiate= NONE,
	  sugar= SOME (fn n=>  EVERY [rewrite_tac [GSA_def],
				      rtac conjI n, rtac conjI (n+1),
				      rtac conjI (n+2), rtac conjI (n+3)])
	  }
	    
    val SoPC_trafo : trafo =
	{ core_thm= SoPC.split_of_post,
	  params = [ "I", "B", "E", "H"],
	  param_info = ["The invariant.", 
			"The termination condition",
			"The induction start",
			"The induction step"],
	  info= "Split of Postcondition:\nTransforms a suitably \
	   \conjoined specification into a tail-recursive form",
	  name= "SoPC",
	  id= "HolTrafoThy.SoPC_trafo",
	  instantiate= NONE,
	  sugar= NONE
	  }


    val Embedding_trafo : trafo =
	{ core_thm= Embedding.embedding,
	  params = [ "E", "B", "H1", "H2", "L"],
	  param_info = [".", 
			".",
			".",
			".",
			","],
	  info= "Embedding",
	  name= "Embedding",
	  id= "HolTrafoThy.Embedding_trafo",
	  instantiate= NONE,
	  sugar= NONE
	  }

   val binomi_trafo : trafo =
	{ core_thm= Sqrt.binomi,
	  params = [],
	  param_info = [],
	  info= "",
	  name= "binomi",
	  id= "HolTrafoThy.binomi_tafo",
	  instantiate= NONE,
	  sugar= NONE
	  }

   val insert_let_trafo : trafo =
	{ core_thm= FinDiff.insert_let,
	  params = [],
	  param_info = [],
	  info= "",
	  name= "InsertLet",
	  id= "HolTrafoThy.insert_let_trafo",
	  instantiate= NONE,
	  sugar= NONE
	  }

   val sqr_trafo : trafo =
	{ core_thm= Sqrt.sqr_def RS meta_eq_to_obj_eq,
	  params = [],
	  param_info = [],
	  info= "Definition of sqr",
	  name= "sqr",
	  id= "HolTrafoThy.sqr_trafo",
	  instantiate= NONE,
	  sugar= NONE
	  }


    fun all_trafos()= 

	[commute_trafo, commuteAC_trafo,
	 FoldTrafos.unfold_trafo, FoldTrafos.fold_trafo, gs_trafo,
	 SoPC_trafo, Embedding_trafo, sqr_trafo, binomi_trafo,
	 insert_let_trafo, commute_plus_trafo, add_mult_trafo]

end





